<?php
/* includes/libs/errorhandling.lib.php
.---------------------------------------------------------------------------.
|    Software:                                                              |
|     Version: 1.0                                                          |
|        Date: 2009-08-11                                                   |
| Description: rewrites the PHP-error-handling with own functions           |
|     Contact: info@andreas-guder.de                                        |
| ------------------------------------------------------------------------- |
|      Author: Andreas Guder                                                |
|     Contact: info@andreas-guder.de                                        |
| Copyright (c) 2009-2010, Andreas Guder. All Rights Reserved.              |
| ------------------------------------------------------------------------- |
|     License: you are allowed to use, modify, distribute                   |
|              and sublicense copies of the software                        |
| This library is distributed in the hope that it will be useful - WITHOUT  |
| ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or     |
| FITNESS FOR A PARTICULAR PURPOSE.                                         |
'---------------------------------------------------------------------------ü
*/
$error_message = '
<h1>Leider ist ein Fehler aufgetreten</h1>
<p>Unser Webmaster wurde bereits benachrichtigt. Wir bem&uuml;hen uns, den Fehler so schnell wie m&ouml;glich zu beheben. In der Regel geschieht dies innerhalb eines Werktages.</p>';
define('ERROR_INFO_MAIL', 'error-reporting@andreas-guder.de');
define('DEFAULT_MAIL_FROM', 'jtl-plugin@'.$_SERVER['SERVER_NAME']);
define('ERROR_INFO_TEXT', $error_message);
define('ERROR_LOGGING_PATH', AGI_PLUGIN_PATH);
define('SCRIPT_NAME', 'JTL-Plugin v'.AGI_PLUGIN_VERSION);
/*
  Expects:
    Constant ERROR_INFO_MAIL    -> eMail-Adress where Error-Mails should be send to
    Constant DEFAULT_MAIL_FROM  -> dafault eMail From
    Constant ERROR_LOGGING_FILE -> File where errors should be logged
    Constant ERROR_INFO_TEXT    -> the Text that is printed out if there is an php-error
    Constant SCRIPT_NAME        -> name of the script/software, to identify it
    define('ERROR_INFO_MAIL', 'your@mail.mod');
    define('DEFAULT_MAIL_FROM', 'your@mail.mod');
    define('ERROR_LOGGING_PATH', '/zum/Logfile/');
    define('ERROR_INFO_TEXT', 'SORRY, there is an error.');
    define('SCRIPT_NAME', 'My software');
 */
  
/**
 * this function send the error-mails to the the reciver/administrator
 * ERROR_INFO_MAIL, DEFAULT_MAIL_FROM should be defined
 *
 * @param   string   $subject: subject of the eMail, e.g. 'Error in your Script'
 * @param   string   $body: eMail-Body, e.g. "There is an error in..."
 * @param   integer  $prio: Priority of the eMail (range 1 to 4 ), e.g. '3'
 * @return  
 *
 */
function error_lib_sendmail($subject, $body, $prio=3)
{
  if (defined('ERROR_INFO_MAIL') && ERROR_INFO_MAIL != '')
  {
    $extheader_client = "From: ".DEFAULT_MAIL_FROM."\n"		// Nötige HEADER-Angaben
          ."Content-Type: text/plain; charset=\"UTF-8\"\n"
          ."Content-Transfer-Encoding: 8bit\n"
          ."Return-Path: ".ERROR_INFO_MAIL."\n"
          ."Reply-To: ".ERROR_INFO_MAIL."\n"
          ."X-Mailer: PHP ".phpversion()." / PHP Mail";
    mail(ERROR_INFO_MAIL,$subject,$body, $extheader_client);
  }
  return;
}

/**
 * this function print out an user-info on the screen
 * ERROR_INFO_TEXT should be defined
 *
 * @param 
 * @return  
 *
 */
function error_lib_display_info($msg = '')
{
  
  echo ERROR_INFO_TEXT;
  
  echo chr(10).chr(10).'<br /><br />'.$msg;
  
}

/**
 * the error-handler itself, 
 * writes NOTICE to ERROR_LOGGING_PATH/notice.log 
 * and WARNINGS or ERRORS to ERROR_LOGGING_PATH/error.log
 * ERROR_LOGGING_PATH an SCRIPT_NAME should be defined
 *
 * @param   PHP      $typ: Error-Typ, e.g. (E_NOTICE) 
 * @param   string   $message: error-message
 * @param   string   $file: file where the error occurs
 * @param   string   $row: row of the file where the error occurs
 * @param   array    $context: array of all defined variables
 * @return  
 *
 */
function agi_error_handling($typ=E_USER_NOTICE, $message, $file, $row, $context)
{
  if (strpos($file,'/usr/share/php') !== false || strpos($file,'/pear/') !== false)
    return;
  //Array to write out error-numbers
  $errortype = array(
    1=>"Error",
    2=>"Warning",
    8=>"Notice",
    256=>"User Error",
    512=>"User Warning",
    1024=>"User Notice",
    2048=>"Notice"
  );
  $msg = "[".date("Y-m-d H:i:s")."]\n"; // Date and Time
  $msg .= "\tServer: ".(empty($_SERVER['HTTP_HOST']) ? SERVER_HOST : $_SERVER['HTTP_HOST'])."\n";
  $msg .= "\tTyp: $errortype[$typ]\n";
  $msg .= "\tMessage: $message\n\tFILE: $file\n\tRow: $row\n";
  
  // array of all arrays/variables which NOT be shown in the logfile
  $exclude = array(
    "HTTP_POST_VARS",
    "HTTP_COOKIE_VARS",
    "HTTP_SERVER_VARS",
    "HTTP_ENV_VARS",
    "_ENV",
    "HTTP_POST_FILES",
    "HTTP_GET_VARS",
    "_FILES",
    "_REQUEST"
  );

  ini_set("track_errors", 1); // enable error-tracking
  
  switch ($typ) 
  {
    case 8: // If NOTICE, only write the error to the logfile
    case 1024:
    case 2048:
      @$fp=fopen(ERROR_LOGGING_PATH."error.log", "a");
      
      if (false===$fp)
      { // Not able to open the logfile, so send a mail
        error_lib_sendmail("[URGENT / NOTICE]", $msg, 3);
      }
      else
      {
        // Write the error-message to the logfile
        fputs($fp, $msg);
        
        // if it's possible, write the script-source befor and behind the line where the error occures to the logfile
        
        @$file=file($file); // read the script-source
        
        if (false===$file)
        {
          fputs($fp,"Konnte Quelle nicht einlesen\n");
          fputs($fp,"Das System meldet: $php_errormsg\n\n");
        }
        else
        {
          // print out 3 rows before and behind the error
          $padding=3;
          
          if (count($file)<$row)
          { // Ooops, the error isn't in this file
            fputs($fp,"Fehler nach Dateiende");
          }
          else
          {
            // What's the first line to print out?
            if (0>$row-$padding)
            {
              $start_reading = 0; // 0 is the first line
            }
            else
            {
              // it starts somewhere in the file
              $start_reading = $row-$padding;
            }
            
            // when should I stop reading?
            $stop_reading = $row+$padding;
            for ($counter=$start_reading; $counter < count($file); $counter++)
            {
              // is this the end?
              if ($counter > $stop_reading)
              { // break the loop
                break;
              }
              // remove linebreaks
              $file[$counter]=preg_replace("/[\n\r]/","",$file[$counter]);
              //write out the source
              fputs($fp,"$counter:$file[$counter]\n");
            }
            fputs($fp,"\nVariableninhalte:\n");
            
            // remove variables, that not be shown
            foreach ($exclude as $key)
            {
              unset($context[$key]);
            }
            ob_start();
            print_r($context);
            $context_values = ob_get_contents();
            ob_end_clean();
            fputs($fp,$context_values);
            fputs($fp,"\n\n");
          }
          
          // Send the email
          error_lib_sendmail("[NOTICE] in ".SCRIPT_NAME,$msg,3);
        }
      }
    break;
    case 1: // WARNING OR ERROR display an screen-message, send a mail an write to logfile
    case 2: 
    case 256:
    case 512:
      error_lib_display_info($msg); // Print out the screen-info
      @$fp=fopen(ERROR_LOGGING_PATH."error.log","a"); 
      
      if (false===$fp)
      { // Not able to open the logfile, so send a mail
        error_lib_sendmail("[URGENT / ERROR] Logfile failed in ".SCRIPT_NAME, $msg, 2);
      }
      else
      {
        // Write the error-message to the logfile
        fputs($fp, $msg);
        
        // if it's possible, write the script-source befor and behind the line where the error occures to the logfile
        
        @$file=file($file); // read the script-source
        
        if (false===$file)
        {
          fputs($fp,"Konnte Quelle nicht einlesen\n");
          fputs($fp,"Das System meldet: $php_errormsg\n\n");
        }
        else
        {
          // print out 3 rows before and behind the error
          $padding=3;
          
          if (count($file)<$row)
          { // Ooops, the error isn't in this file
            fputs($fp,"Fehler nach Dateiende");
          }
          else
          {
            // What's the first line to print out?
            if (0>$row-$padding)
            {
              $start_reading = 0; // 0 is the first line
            }
            else
            {
              // it starts somewhere in the file
              $start_reading = $row-$padding;
            }
            
            // when should I stop reading?
            $stop_reading = $row+$padding;
            for ($counter=$start_reading; $counter < count($file); $counter++)
            {
              // is this the end?
              if ($counter > $stop_reading)
              { // break the loop
                break;
              }
              // remove linebreaks
              $file[$counter]=preg_replace("/[\n\r]/","",$file[$counter]);
              //write out the source
              fputs($fp,"$counter:$file[$counter]\n");
            }
            fputs($fp,"\nVariableninhalte:\n");
            
            // remove variables, that not be shown
            foreach ($exclude as $key)
            {
              unset($context[$key]);
            }
            ob_start();
            print_r($context);
            $context_values = ob_get_contents();
            ob_end_clean();
            fputs($fp,$context_values);
            fputs($fp,"\n\n");
          }
          
          // Send the email
          error_lib_sendmail("[ERROR] in ".SCRIPT_NAME,$msg,2);
        }
      }
      exit;
    break;
  } // end switch
}

/**
 * Rewrite the php.ini error-settings
 */
ini_set("error_reporting",E_ALL ^E_NOTICE ^E_STRICT ^E_DEPRECATED); // enable alle error-messages
//ini_set("display_errors",0); // dont show error-messages
//ini_set("log_errors",1); // enable error-logging
//ini_set("error_log",ERROR_LOGGING_PATH); // set logging-path
set_error_handler("agi_error_handling"); // define my error-handle-function

?>