<?php
/* includes/plugins/agi_shopgate_plugin/version/107/plugin.class.php
.---------------------------------------------------------------------------.
|    Software: AGI Shopgate - JTL - Connector                               |
|     Version: 2.15 JTL4                                                    |
|        Date: 2016-09-15                                                   |
|     Contact: info@andreas-guder.de                                        |
| ------------------------------------------------------------------------- |
|      Author: Andreas Guder                                                |
|     Contact: info@andreas-guder.de                                        |
| Copyright (c) 2013 Andreas Guder [info@andreas-guder.de]                  |
|               All Rights Reserved.                                        |
|                      Based on Shopgate Example-Plugin                     |
'---------------------------------------------------------------------------ü
IONCUBE!!
*/
define('SHOPGATE_PLUGIN_VERSION', AGI_PLUGIN_VERSION.'.2.15'); // So angeben! Shopgate erkennt den Doppelpunkt nicht
require_once DIR_BASE_PATH.'/classes/interface.JTL-Shop.PluginLizenz.php';
require_once AGI_PLUGIN_PATH.'licence/class.PluginLicence.php';
class AGIShopgatePluginBase extends ShopgatePlugin
{
  public $request_shop_number = 0;
  public $kSprache = 1;
  public $kKundengruppe = 1;
  public $oExportArticle = null;
  public $shopURLEndSlash = false;
  private $startExecutionTime = 0;
  private $maxExecutionTime = 55;
  private $fallback_steuersatz = 19;
  protected $bAllowKonfiguration = false; // 20160509 konfiguration
  public $jtl_plugin_configuration = array();
  protected $aTeinstellungenFields = array();
  
  /** Variablen für Produktexport */
  protected $aVersandart = array();
  protected $cLieferLand = 'DE';
  protected $cNurAbhaengigeVersandart = "N";
  protected $aKundengruppen = array();
  protected $aSteuerklassen = array();
  protected $aAttributNichtExportieren = array();
  protected $aMerkmaleNichtExportieren = array();
  protected $aAddDescAttribute = array();
  protected $inaktivStatus_Arr = array();
  public $cWrapDescriptionAttribute = '<p><strong>%s:</strong> %s</p>';
  private $cTrennzeichen = ';';
  private $tSprache = array('vat'=>'MwSt.');
  
  /// NECESSARY SHOPGATE FUNCTIONS ///
  /// BOF STARTUP-FUNCTIONS ///
  public function startup() 
  {
    $liz = new agi_shopgate_pluginPluginLicence();
    if (!$liz->checkLicence(AGI_PLUGIN_LICENCE))
    {
      Jtllog::writeLog('Shopgate-Plugin: '.$liz->getErrorNotice(), JTLLOG_LEVEL_ERRROR);
      throw new ShopgateLibraryException(ShopgateLibraryException::CONFIG_PLUGIN_NOT_ACTIVE, 'JTL-Plugin: '.$liz->getErrorNotice(), true);
      exit;
    }
    $this->startExecutionTime = time();
    $mt = @ini_get('max_execution_time');
    if (!empty($mt) && $mt > -1)
      $this->maxExecutionTime = $mt-5;
    
    $this->kKundengruppe = gibStandardKundengruppe();
    if (JTL_VERSION < 400)
      gibKundengruppe($this->kKundengruppe); // jtl < 4
    $t = gibStandardsprache(true);
    $this->kSprache = $t->kSprache;
    if (isset($_REQUEST['shop_number']) && !empty($_REQUEST['shop_number']))
    {
      $this->request_shop_number = (int) $_REQUEST['shop_number'];
      $this->config = new ShopgateConfig(); 
      $this->config->loadByShopNumber((int) $_REQUEST['shop_number']);
      $this->config->setServer('live');
      $this->config->setApiUrl('https://api.shopgate.com/merchant/');
      $this->config->setEncoding('UTF-8');
      $this->config->setPluginName('ShopgatePluginJTL');
      $this->config->setEnableAddOrder(1);
      $this->config->setEnableUpdateOrder(1);
      $this->config->setEnableCheckStock(1);
      $this->config->setEnableGetCustomer(1);
      $this->config->setEnableRegisterCustomer(1);
      $this->config->setEnableGetItems(1);
      $this->config->setEnableGetCategories(1);
      $this->config->setEnableCron(1);
      $this->config->setEnableGetSettings(1);
      $this->config->setEnableCheckCart(1);
      $this->config->setEnableRedeemCoupons(1);
      $this->config->setEnableGetReviews(1);
      
      $this->config->setEnableGetOrders(0);
      $this->config->setEnableGetItemsCsv(0);
      $this->config->setEnableGetCategoriesCsv(1);
      $this->config->setEnableGetReviewsCsv(0);
      
      if (isset($this->config->jtlSprache) && !empty($this->config->jtlSprache))
        $this->kSprache = $this->config->jtlSprache;
    }
    else
      throw new ShopgateLibraryException(ShopgateLibraryException::PLUGIN_API_UNKNOWN_SHOP_NUMBER, '', true);
    
    // lese diverse Optionen
    $aRes = $GLOBALS["DB"]->executeQuery("SELECT * FROM `xplugin_agi_shopgate_plugin_config` ORDER BY `cKey`", 9);
    foreach ($aRes as $row)
    {
      $this->jtl_plugin_configuration[$row['cKey']] = $row['cValue'];
    }
    $aRes = $GLOBALS["DB"]->executeQuery("SELECT * FROM `twaehrung` WHERE `cStandard`='Y'", 1);
    if($aRes)
      $this->_setSessionVar('Waehrung', $aRes);
    else
    {
      $aRes = $GLOBALS["DB"]->executeQuery("SELECT * FROM `twaehrung` LIMIT 0,1", 1);
      if($aRes)
        $this->_setSessionVar('Waehrung', $aRes);
      else
        throw new ShopgateLibraryException(999, 'JTL-Plugin: Shopwaehrung kann nicht geladen werden', true);
    }
    if (substr(URL_SHOP,-1) == '/')
      $this->shopURLEndSlash = true;
    setzeSteuersaetze();
    $this->_setSessionVar('kSprache', $this->kSprache); 
    $this->_setSessionVar('cISOSprache', 'ger');
    $aRes = $GLOBALS["DB"]->executeQuery("SELECT `cISO` FROM `tsprache` WHERE `kSprache`=".$this->kSprache, 1);
    if ($aRes)
    {
      $this->_setSessionVar('cISOSprache', $aRes->cISO);
      $aRes = $GLOBALS["DB"]->executeQuery("SELECT * FROM `tsprache` WHERE `kSprache`=".$this->kSprache, 1);
      $this->_setSessionVar('Sprachen', array($aRes));
    }
    else
    {
      $t = gibStandardsprache(true);
      $aRes = $GLOBALS["DB"]->executeQuery("SELECT * FROM `tsprache` WHERE `kSprache`=".$t->kSprache, 1);
      $this->_setSessionVar('Sprachen', array($aRes));
    }
    $aRes = $GLOBALS["DB"]->executeQuery("SELECT * FROM `tsprachwerte` WHERE `cName` IN ('vat','shippingTimeLP','shippingTime','productInflowing','supplierStockNotice','deliveryStatus','productAvailableFrom','preorderPossible') AND `kSprachISO`=".$this->kSprache, 9);
    foreach ($aRes as $row)
      $this->tSprache[$row['cName']] = $row['cWert'];
    
    // lade teinstellungen
    $aTeinstellungenFields = array("'kundenregistrierung_standardland'","'bewertung_anzeigen'","'bewertung_freischalten'","'kundenregistrierung_nur_lieferlaender'","'global_erscheinende_kaeuflich'");
    $aRes = $GLOBALS["DB"]->executeQuery("SELECT * FROM `teinstellungen` WHERE `cName` IN (".implode(',', $aTeinstellungenFields).")", 9);
    foreach ($aRes as $row)
    {
      $this->aTeinstellungenFields[$row['cName']] = $row['cWert'];
    }
    $this->cLieferLand = $this->aTeinstellungenFields['kundenregistrierung_standardland'];
    /* BOF 20160509 */
    if (isset($this->bExportProduktKonfiguration) && $this->bExportProduktKonfiguration===true && class_exists('Konfigurator'))
      $this->bAllowKonfiguration = true;
    $this->agiStartup();
    /* EOF 20160509 */
  }
  /// EOF STARTUP-FUNCTIONS ///
  
  /// BOF CATEGORIES-FUNCTIONS ///
  public function createCategories($limit = null, $offset = null, array $uids = array())
  {
    $aKategories = array();
    $aKategorieIDs = $this->_loadCategories($limit, $offset, $uids);
    foreach ($aKategorieIDs as $nID)
    {
      $oKategorie = new Kategorie($nID, $this->kSprache, $this->kKundengruppe);
      $item = array(
        'nID'       => $oKategorie->kKategorie,
        'name'      => $oKategorie->cName,
        'active'    => 1,
        'parentID'  => $oKategorie->kOberKategorie,
        'cDeeplink' => $this->shopURLEndSlash ? URL_SHOP.$oKategorie->cURL : URL_SHOP.'/'.$oKategorie->cURL,
        'nSort'     => 100000-$oKategorie->nSort,
        'hasImage'  => $oKategorie->nBildVorhanden,
        'cBild'     => $oKategorie->cBild
      );
      $aKategories[$nID] = $this->agiManipulateCategoriesArrayItem($item);
    }
    $aKategories = $this->agiOverwriteCategoriesArray($aKategories, $limit, $offset, $uids);
    
    foreach ($aKategories as $key => $value)
    {
      $categoryItem = new Shopgate_Model_Catalog_Category();
      $categoryItem->setUid($key);
      $categoryItem->setName($this->_pss(html_entity_decode($value['name'],ENT_QUOTES,"ISO-8859-15")));
      $categoryItem->setParentUid($value['parentID']);
      $categoryItem->setIsActive($value['active']); // AGI 20150202
      $categoryItem->setDeeplink($this->_pss($value['cDeeplink']));
      //$categoryItem->setIsAnchor(1);
      $categoryItem->setSortOrder($value['nSort']);
      if ($value['hasImage'])
      {
        $imageItem = new Shopgate_Model_Media_Image();
        $imageItem->setUid($key);
        //$imageItem->setSortOrder(10);
        $imageItem->setAlt($this->_pss($value['name']));
        $imageItem->setUrl($value['cBild']);
        $imageItem->setTitle($this->_pss($value['name']));
        $categoryItem->setImage($imageItem);
      }
      $this->addCategoryModel($categoryItem);
      if (time()-$this->startExecutionTime > $this->maxExecutionTime)
        break;
    }
  }
  public function createCategoriesCsv()
  {
    $aKategories = array();
    $aKategorieIDs = $this->_loadCategories();
    foreach ($aKategorieIDs as $nID)
    {
      $oKategorie = new Kategorie($nID, $this->kSprache, $this->kKundengruppe);
      $item = array(
        'nID'       => $oKategorie->kKategorie,
        'name'      => $oKategorie->cName,
        'active'    => 1,
        'parentID'  => $oKategorie->kOberKategorie,
        'cDeeplink' => $this->shopURLEndSlash ? URL_SHOP.$oKategorie->cURL : URL_SHOP.'/'.$oKategorie->cURL,
        'nSort'     => 100000-$oKategorie->nSort,
        'hasImage'  => $oKategorie->nBildVorhanden,
        'cBild'     => $oKategorie->cBild
      );
      $aKategories[$nID] = $this->agiManipulateCategoriesArrayItem($item);
    }
    $aKategories = $this->agiOverwriteCategoriesArray();
    
    foreach ($aKategories as $key => $value)
    {
      $row = $this->buildDefaultCategoryRow();
			$row['parent_id']       = $value['parentID'];
			$row['category_number'] = $key;
			$row['category_name']   = $this->_pss($value['name'], true);
			$row["is_active"]       = 1;
			$row['url_deeplink']    = $this->_pss($value['cDeeplink']);
      $row['order_index']     = $value['nSort'];
      if ($value['hasImage'])
        $row["url_image"] = $value['cBild'];
			$this->addCategoryRow($row);
      if (time()-$this->startExecutionTime > $this->maxExecutionTime)
        break;
    }
  }

  protected function _loadCategories($limit = null, $offset = null, array $uids = array())
  {
    if (!empty($uids))
    {
      $inUids = '';
      foreach ($uids as $value)
      {
        if (!empty($inUids))
          $inUids .= ',';
        $inUids .= "'".$value."'";
      }
      $result = $GLOBALS["DB"]->executeQuery(
        "SELECT 
          tkategorie.kKategorie
        FROM tkategorie
        LEFT JOIN tkategoriesichtbarkeit ON 
          tkategoriesichtbarkeit.kKategorie = tkategorie.kKategorie AND tkategoriesichtbarkeit.kKundengruppe=".$this->kKundengruppe."
        WHERE tkategorie.kKategorie IN ($inUids) AND tkategoriesichtbarkeit.kKategorie IS NULL
        ORDER BY tkategorie.kOberKategorie, tkategorie.kOberKategorie",
        9
      );
    }
    else
    {
      $sLimit = '';
      if (!empty($limit))
        $sLimit = (int) $limit;
      if (!empty($limit) && !empty($offset))
        $sLimit = ((int) $offset).','.$sLimit;
      if (!empty($sLimit))
        $sLimit = ' LIMIT '.$sLimit;
      $result = $GLOBALS["DB"]->executeQuery(
        "SELECT 
          tkategorie.kKategorie
        FROM tkategorie
        LEFT JOIN tkategoriesichtbarkeit ON 
          tkategoriesichtbarkeit.kKategorie = tkategorie.kKategorie AND tkategoriesichtbarkeit.kKundengruppe=".$this->kKundengruppe."
        WHERE tkategoriesichtbarkeit.kKategorie IS NULL
        ORDER BY tkategorie.kOberKategorie, tkategorie.kOberKategorie".$sLimit,
        9
      );
    }
    $aKategorieIDs = array();
    if (!empty($result))
    {
      foreach ($result as $value)
        $aKategorieIDs[$value['kKategorie']] = $value['kKategorie'];
    }
    return $aKategorieIDs;
  }
  /// EOF CATEGORIES-FUNCTIONS ///
  
  /// BOF ITEM-FUNCTIONS ///
  public function createItems($limit = null, $offset = null, array $uids = array())
  {
    $this->_loadCreateItemsConfig();
    $this->agiCreateItemsOnStart();
    $aItemIDs = $this->_loadItems($limit, $offset, $uids);
    
    foreach ($aItemIDs as $key => $value)
    {
      $this->_loadOExportArticle($key);
      if (empty($this->oExportArticle->kArtikel))
        continue;
      $this->_prepareItemObject();
      if (time()-$this->startExecutionTime > $this->maxExecutionTime)
        break;
    }
  }
  protected function _loadOExportArticle($kArtikel)
  {
    $oArtikelOptionen = new stdClass(0);
    $oArtikelOptionen->nMerkmale = 1;
    $oArtikelOptionen->nAttribute = 1;
    $oArtikelOptionen->nArtikelAttribute = 1;
    $oArtikelOptionen->nKategorie = 1;
    $oArtikelOptionen->nKeinLagerbestandBeachten = 1;
    $oArtikelOptionen->nVariationKombiKinder = 1;
    /* BOF 20160509 */
    if ($this->bAllowKonfiguration)
      $oArtikelOptionen->nKonfig = 1;
    /* EOF 20160509 */
    //$oArtikelOptionen->nMedienDatei = 1; // Shop versucht sonst die URLs per CURL zu laden
    
    $this->oExportArticle = new Artikel();
    $this->oExportArticle->fuelleArtikel(
      $kArtikel,
      $oArtikelOptionen,
      $this->kKundengruppe,
      $this->kSprache,
      true // do not use cache
    );
  }
  protected function _loadCreateItemsConfig()
  {
    $this->cLieferLand = isset($_SESSION['cLieferlandISO']) ? $_SESSION['cLieferlandISO'] : 'DE';
    /** BOF Bestandspuffer und Lagerbestand 2014-04-07 */
    $this->jtl_plugin_configuration['aBestandspuffer'] = array();
    $this->jtl_plugin_configuration['maxBestand']      = 0;
    if (array_key_exists('cBestandspuffer', $this->jtl_plugin_configuration) && !empty($this->jtl_plugin_configuration['cBestandspuffer']))
    {
      $aBestandspuffer = array();
      $tmp = explode(',', $this->jtl_plugin_configuration['cBestandspuffer']);
      $maxBestand = 0;
      foreach ($tmp as $row)
      {
        $vars = explode(':', $row);
        if (count($vars) == 2 && !empty($vars[0]))
        {
          if (strtolower($vars[0]) == 'max')
            $maxBestand = empty($vars[1]) ? 0 : $vars[1];
          else
            $aBestandspuffer[(int)$vars[0]] = (int)$vars[1];
        }
      }
      ksort($aBestandspuffer, SORT_NUMERIC);
      $this->jtl_plugin_configuration['aBestandspuffer'] = $aBestandspuffer;
      $this->jtl_plugin_configuration['maxBestand']      = $maxBestand;
    }
    /** EOF Bestandspuffer und Lagerbestand 2014-04-07 */
    
    // Worte auslesen, die den Artikel deaktivieren
    if(array_key_exists('cInaktivStatusString', $this->jtl_plugin_configuration) && !empty($this->jtl_plugin_configuration['cInaktivStatusString']))
    {
      $temp = explode(';', $this->jtl_plugin_configuration['cInaktivStatusString']);
      foreach ($temp as $value)
      {
        $tmp = trim($value);
        if (!empty($tmp))
          $this->inaktivStatus_Arr[] = strtolower($tmp);
      }
    }
    
    // standard-versandart
    $this->aVersandart[$this->jtl_plugin_configuration['nStandardVersandklasse']] = $GLOBALS["DB"]->executeQuery(
      "SELECT *
        FROM tversandart
        WHERE cNurAbhaengigeVersandart='" . $this->cNurAbhaengigeVersandart . "'
        AND cLaender like \"%" . $this->cLieferLand . "%\"
        AND (
          cVersandklassen='-1' or 
          (cVersandklassen like \"% " . $this->jtl_plugin_configuration['nStandardVersandklasse'] . " %\" or cVersandklassen like \"% " . $this->jtl_plugin_configuration['nStandardVersandklasse'] . "\")
        )
        AND (cKundengruppen='-1' or cKundengruppen like '%;" . intval($this->kKundengruppe) . ";%') order by nSort
      ",
      2
    );
    
    // attribute, die nicht exportiert werden
    if (array_key_exists('cAttributNichtExportieren', $this->jtl_plugin_configuration) && !empty($this->jtl_plugin_configuration['cAttributNichtExportieren']))
    {
      $tmp = explode(',',$this->jtl_plugin_configuration['cAttributNichtExportieren']);
      foreach ($tmp as $value)
      {
        $value = trim($value);
        if (!empty($value))
          $this->aAttributNichtExportieren[] = strtolower($value);
      }
    }
    
    // merkmale, die nicht exportiert werden
    if (array_key_exists('cMerkmaleNichtExportieren', $this->jtl_plugin_configuration) && !empty($this->jtl_plugin_configuration['cMerkmaleNichtExportieren']))
    {
      $tmp = explode(',',$this->jtl_plugin_configuration['cMerkmaleNichtExportieren']);
      foreach ($tmp as $value)
      {
        $value = trim($value);
        if (!empty($value))
          $this->aMerkmaleNichtExportieren[] = strtolower($value);
      }
    }
    
    if (array_key_exists('cAttributeAnBeschreibung', $this->jtl_plugin_configuration) && !empty($this->jtl_plugin_configuration['cAttributeAnBeschreibung']))
    {
      $tmp = explode(',',$this->jtl_plugin_configuration['cAttributeAnBeschreibung']);
      foreach ($tmp as $value)
      {
        $value = trim($value);
        if (!empty($value))
          $this->aAddDescAttribute[] = strtolower($value);
      }
    }
    
    if (array_key_exists('cWrapAttributeAnBeschreibung', $this->jtl_plugin_configuration) && !empty($this->jtl_plugin_configuration['cWrapAttributeAnBeschreibung']))
    {
      $tmp = str_replace(array('[wert]', '[inhalt]'), array('%s', '%s'), $this->jtl_plugin_configuration['cWrapAttributeAnBeschreibung']);
      $tcount = array(
        'wert'  => substr_count($this->jtl_plugin_configuration['cWrapAttributeAnBeschreibung'], '[wert]'),
        'inhalt'=> substr_count($this->jtl_plugin_configuration['cWrapAttributeAnBeschreibung'], '[inhalt]')
      );
      if ($tcount['wert'] === 1 || $tcount['inhalt'] === 1)
        $this->cWrapDescriptionAttribute = $tmp;
    }

    // lade KundengruppenIds
    $aRes = $GLOBALS["DB"]->executeQuery("SELECT * FROM `tkundengruppe`", 9);
    foreach ($aRes as $row)
      $this->aKundengruppen[] = $row['kKundengruppe'];
      
    // lade steuerklassen
    $aRes = $GLOBALS["DB"]->executeQuery("SELECT * FROM `tsteuerklasse`", 9);
    foreach ($aRes as $row)
      $this->aSteuerklassen[$row['kSteuerklasse']] = $row;
    
    // lade teinstellungen
    $aTeinstellungenFields = array("'artikeldetails_attribute_anhaengen'","'artikel_lagerbestandsanzeige'","'artikeldetails_lieferantenbestand_anzeigen'","'artikeldetails_lieferstatus_anzeigen'","'artikeldetails_xselling_standard_anzeigen'","'artikeldetails_xselling_kauf_anzeigen'","'artikeldetails_xselling_kauf_anzahl'","'artikel_artikelanzeigefilter'");
    $aRes = $GLOBALS["DB"]->executeQuery("SELECT * FROM `teinstellungen` WHERE `cName` IN (".implode(',', $aTeinstellungenFields).")", 9);
    foreach ($aRes as $row)
    {
      $this->aTeinstellungenFields[$row['cName']] = $row['cWert'];
    }
  }
  protected function _loadItems($limit = null, $offset = null, array $uids = array())
  {
    if (!empty($uids))
    {
      $inUids = '';
      foreach ($uids as $value)
      {
        if (!empty($inUids))
          $inUids .= ',';
        $inUids .= "'".$value."'";
      }
      $result = $GLOBALS["DB"]->executeQuery(
        "SELECT 
          tartikel.kArtikel
        FROM tartikel
        LEFT JOIN tartikelsichtbarkeit ON 
          tartikelsichtbarkeit.kArtikel = tartikel.kArtikel AND tartikelsichtbarkeit.kKundengruppe=".$this->kKundengruppe."
        WHERE tartikel.kArtikel IN ($inUids) AND tartikel.kVaterArtikel=0 AND tartikelsichtbarkeit.kArtikel IS NULL
        ORDER BY kArtikel",
        9
      );
    }
    else
    {
      $sLimit = '';
      if (!empty($limit))
        $sLimit = (int) $limit;
      if (!empty($limit) && !empty($offset))
        $sLimit = ((int) $offset).','.$sLimit;
      if (!empty($sLimit))
        $sLimit = ' LIMIT '.$sLimit;
      $result = $GLOBALS["DB"]->executeQuery(
        "SELECT 
          tartikel.kArtikel
        FROM tartikel
        LEFT JOIN tartikelsichtbarkeit ON 
          tartikelsichtbarkeit.kArtikel = tartikel.kArtikel AND tartikelsichtbarkeit.kKundengruppe=".$this->kKundengruppe."
        WHERE tartikel.kVaterArtikel=0 AND tartikelsichtbarkeit.kArtikel IS NULL
        ORDER BY kArtikel".$sLimit,
        9
      );
    }
    $aItemIDs = array();
    if (!empty($result))
    {
      foreach ($result as $value)
        $aItemIDs[$value['kArtikel']] = $value;
    }
    return $aItemIDs;
  }
  protected function _prepareItemObject($thisIsChild = 0)
  {
    $tmp = $GLOBALS["DB"]->executeQuery(
      "SELECT `dLetzteAktualisierung`, `kHersteller` FROM `tartikel` WHERE `kArtikel`=".$this->oExportArticle->kArtikel,
      1
    );
    $this->oExportArticle->cDeeplink = $this->shopURLEndSlash ? URL_SHOP.$this->oExportArticle->cURL : URL_SHOP.'/'.$this->oExportArticle->cURL;
    $this->oExportArticle->dLetzteAktualisierung = $tmp->dLetzteAktualisierung;
    $this->oExportArticle->kHersteller = $tmp->kHersteller;
    $this->oExportArticle->nHasChildren = $this->oExportArticle->nIstVater ? 1 : 0;
    $this->oExportArticle->cParentItemNumber = empty($this->oExportArticle->kVaterArtikel) ? '' : $this->oExportArticle->kVaterArtikel;
    
    $this->oExportArticle->fUst = gibUst($this->oExportArticle->kSteuerklasse);
    $this->oExportArticle->fLagerbestand = floor($this->oExportArticle->fLagerbestand);
    
    $this->_loadShippingCost();
    $this->_loadItemCategories();
    $this->_itemGetGrundpreis();
    $this->_itemCheckHightlight();
    $this->_itemCheckStatusAktiv();
    $this->_itemLoadAttributes();
    $this->_itemSetLieferstatus();
    $this->_itemLadeMediendateien();
    $this->_itemPrepareDescription();
    
    $this->agiManipulateItemObject();
    
    // write item
    $productItem = new Shopgate_Model_Catalog_Product();
    if ($thisIsChild)
      $productItem->setIsChild(1);
    
    $productItem->setUid($this->oExportArticle->kArtikel);
    //$productItem->setItemNumberPublic($this->_pss($this->oExportArticle->cArtNr));
    $productItem->setLastUpdate($this->oExportArticle->dLetzteAktualisierung);
    $productItem->setName($this->_pss(html_entity_decode($this->oExportArticle->cName,ENT_QUOTES,"ISO-8859-15")));
    $productItem->setTaxPercent($this->oExportArticle->fMwSt);
    $productItem->setTaxClass($this->_pss($this->aSteuerklassen[$this->oExportArticle->kSteuerklasse]['cName']));
    $productItem->setCurrency($_SESSION['Waehrung']->cISO);
    $productItem->setDescription($this->_pss($this->oExportArticle->cBeschreibung));
    $productItem->setDeeplink($this->oExportArticle->cDeeplink);
    if ($this->oExportArticle->nTopArtikel)
      $productItem->setPromotionSortOrder(10);
    $productItem->setWeight($this->oExportArticle->fArtikelgewicht);
    $productItem->setWeightUnit(Shopgate_Model_Catalog_Product::DEFAULT_WEIGHT_UNIT_KG);
    
    $priceItem = new Shopgate_Model_Catalog_Price();
    $priceItem->setType(Shopgate_Model_Catalog_Price::DEFAULT_PRICE_TYPE_NET);
    $agi_calc_price = $this->oExportArticle->Preise->fVK[1];
    if (!empty($this->oExportArticle->Preise->alterVK[1]) && $this->oExportArticle->Preise->alterVK[1] > $this->oExportArticle->Preise->fVK[1] && isset($this->oExportArticle->Preise->Sonderpreis_aktiv))
    {
      $priceItem->setPrice($this->oExportArticle->Preise->alterVK[1]);
      $priceItem->setSalePrice($this->oExportArticle->Preise->fVK[1]);
    }
    elseif(!empty($this->oExportArticle->Preise->alterVK[1]) && $this->oExportArticle->Preise->alterVK[1] > $this->oExportArticle->Preise->fVK[1])
    {
      $priceItem->setPrice($this->oExportArticle->Preise->alterVK[1]);
      $priceItem->setSalePrice($this->oExportArticle->Preise->fVK[1]);
    }
    else
      $priceItem->setPrice($this->oExportArticle->Preise->fVK[1]);
    $priceItem->setMsrp(round($this->oExportArticle->fUVP*100/(100+$this->oExportArticle->fMwSt) *$_SESSION['Waehrung']->fFaktor,4));
    //$priceItem->setMinimumOrderAmount(empty($this->oExportArticle->fMindestbestellmenge) ? 1 : ceil($this->oExportArticle->fMindestbestellmenge)); // Mindestbestellwert
    $priceItem->setMinimumOrderAmount(0);
    if ($this->oExportArticle->cVPE == 'Y')
    {
      $priceItem->setBasePrice($this->_pss(str_replace('&euro;', 'EURO',$this->oExportArticle->cLocalizedVPE[0])));
    }
    // BOF Staffelpreise
    $hatPreisstaffel = false;
    for ($i =1; $i<=5; $i++)
    {
      if (!empty($this->oExportArticle->Preise->{'nAnzahl'.$i}))
      {
        $hatPreisstaffel = true;
        break;
      }
    }
    $allow_tier_prices = true;
    if (isset($this->oExportArticle->nIstVater) && $this->oExportArticle->nIstVater == 1 && !$thisIsChild)
      $allow_tier_prices = false; // 2.01 20141219

    if ($allow_tier_prices && (count($this->aKundengruppen) > 1 || $hatPreisstaffel))
    {
      foreach ($this->aKundengruppen as $kKundengruppe)
      {
        $preise = new Preise($kKundengruppe, $this->oExportArticle->kArtikel);
        $preise->rabbatierePreise($this->oExportArticle->fMaxRabatt);
        $preise->localizePreise();
      
        if ($agi_calc_price-$preise->fVK[1] <> 0)
        {
          $tierPriceItem = new Shopgate_Model_Catalog_TierPrice();
          $tierPriceItem->setAggregateChildren(0);
          $tierPriceItem->setCustomerGroupUid($kKundengruppe);
          $tierPriceItem->setReduction($agi_calc_price-$preise->fVK[1]);
          $tierPriceItem->setFromQuantity(1);
          $tierPriceItem->setReductionType(Shopgate_Model_Catalog_TierPrice::DEFAULT_TIER_PRICE_TYPE_FIXED);
          $priceItem->addTierPriceGroup($tierPriceItem);
        }
        if ($hatPreisstaffel)
        {
          for ($i =1; $i<=5; $i++)
          {
            if (!empty($preise->{'nAnzahl'.$i}))
            {
              if ($agi_calc_price-$preise->{'fStaffelpreis'.$i}[1] <> 0)
              {
                $tierPriceItem = new Shopgate_Model_Catalog_TierPrice();
                $tierPriceItem->setAggregateChildren(0);
                $tierPriceItem->setCustomerGroupUid($kKundengruppe);
                $tierPriceItem->setFromQuantity($preise->{'nAnzahl'.$i});
                //$tierPriceItem->setToQuantity(20);
                $tierPriceItem->setReduction($agi_calc_price-$preise->{'fStaffelpreis'.$i}[1]);
                $tierPriceItem->setReductionType(Shopgate_Model_Catalog_TierPrice::DEFAULT_TIER_PRICE_TYPE_FIXED);
                $priceItem->addTierPriceGroup($tierPriceItem);
              }
            }
          }
        }
      }
    }
    // EOF Staffelpreise
    $productItem->setPrice($priceItem);
    $count_images = 0;
    if (isset($this->oExportArticle->Bilder))
    {
      foreach ($this->oExportArticle->Bilder as $value)
      {
        if ($thisIsChild == 0 || ($thisIsChild > 0 && strpos($value->cPfadGross, BILD_KEIN_ARTIKELBILD_VORHANDEN) === false))
        {
          $imageItem = new Shopgate_Model_Media_Image();
          $imageItem->setUid($this->oExportArticle->kArtikel.'_'.$value->nNr);
          $imageItem->setAlt(empty($value->cAltAttribut) ? $this->oExportArticle->cName : $value->cAltAttribut);
          $imageItem->setTitle(empty($value->cAltAttribut) ? $this->oExportArticle->cName : $value->cAltAttribut);
          $imageItem->setUrl($this->shopURLEndSlash ? URL_SHOP.$value->cPfadGross : URL_SHOP.'/'.$value->cPfadGross);
          $productItem->addImage($imageItem);
          $count_images++;
        }
      }
    }
    if ($thisIsChild > 0 && $count_images == 0)
    {
      // add parent images
      // lade steuerklassen
      $aRes = $GLOBALS["DB"]->executeQuery("
        SELECT * FROM `tartikelpict` WHERE `kArtikel`=".(int)$this->oExportArticle->kVaterArtikel." ORDER BY `nNr`", 2);
      foreach ($aRes as $row)
      {
        $imageItem = new Shopgate_Model_Media_Image();
        $imageItem->setUid($this->oExportArticle->kArtikel.'_'.$row->nNr);
        $imageItem->setAlt($this->oExportArticle->cName);
        $imageItem->setTitle($this->oExportArticle->cName);
        $imageItem->setUrl($this->shopURLEndSlash ? URL_SHOP.PFAD_PRODUKTBILDER_GROSS.$row->cPfad : URL_SHOP.'/'.PFAD_PRODUKTBILDER_GROSS.$row->cPfad);
        $productItem->addImage($imageItem);
      }
    }
    
    foreach ($this->oExportArticle->aAGIKategorien as $value)
    {
      $categoryPathItem = new Shopgate_Model_Catalog_CategoryPath();
      $categoryPathItem->setUid($value['kKategorie']);
      $categoryPathItem->setSortOrder($this->_getArticleSortInCategory($value['kKategorie']));
      $categoryPathItem->setDeeplink($value['cDeeplink']);
      foreach($value['paths'] as $level => $path)
      {
        $categoryPathItem->addItem($level, $this->_pss($path));
      }
      if (!empty($value['image']))
      {
        $imageItem = new Shopgate_Model_Media_Image();
        $imageItem->setUid($value['kKategorie']);
        $imageItem->setAlt($this->_pss($value['cName']));
        $imageItem->setTitle($this->_pss($value['cName']));
        $imageItem->setUrl($value['image']);
        $categoryPathItem->addImage($imageItem);
      }
      $productItem->addCategoryPath($categoryPathItem);
    }
    
    $shippingItem = new Shopgate_Model_Catalog_Shipping();
    $shippingItem->setIsFree($this->oExportArticle->nOhneVersandkosten);
    $shippingItem->setCostsPerOrder($this->oExportArticle->fVersandkosten);
    $productItem->setShipping($shippingItem);
    
    if (isset($this->oExportArticle->kHersteller) && !empty($this->oExportArticle->kHersteller))
    {
      $manufacturerItem = new Shopgate_Model_Catalog_Manufacturer();
      $manufacturerItem->setUid($this->oExportArticle->kHersteller);
      $manufacturerItem->setTitle($this->_pss($this->oExportArticle->cHersteller));
      $manufacturerItem->setItemNumber($this->oExportArticle->cHAN);
      $productItem->setManufacturer($manufacturerItem);
    }
    
    $stockItem = new Shopgate_Model_Catalog_Stock();
    $stockItem->setStockQuantity($this->oExportArticle->fLagerbestand);
    $stockItem->setAvailabilityText($this->_pss($this->_remove_html($this->oExportArticle->agiLieferstatus)));
    $stockItem->setBackorders(1);
    $stockItem->setIsSaleable($this->oExportArticle->cStatusAktiv == 'inactive' ? 0 : 1);
    $stockItem->setUseStock($this->oExportArticle->nUseStock);
    $stockItem->setMinimumOrderQuantity(empty($this->oExportArticle->fMindestbestellmenge) ? 1 : ceil($this->oExportArticle->fMindestbestellmenge));
    $productItem->setStock($stockItem);
    
    if (isset($this->oExportArticle->Attribute) && !empty($this->oExportArticle->Attribute))
    {
      foreach ($this->oExportArticle->Attribute as $value)
      {
        if (!in_array(strtolower($value->cName), $this->aAttributNichtExportieren))
        {
          $propertyItem = new Shopgate_Model_Catalog_Property();
          $propertyItem->setUid(md5($value->cName));
          $propertyItem->setLabel($this->_pss($value->cName));
          $propertyItem->setValue($this->_pss($value->cWert));
          $productItem->addProperty($propertyItem);
        }
      }
    }
    if (isset($this->oExportArticle->oMerkmale_arr) && !empty($this->oExportArticle->oMerkmale_arr))
    {
      foreach ($this->oExportArticle->oMerkmale_arr as $value)
      {
        if (!in_array(strtolower($value->cName), $this->aMerkmaleNichtExportieren))
        {
          foreach ($value->oMerkmalWert_arr AS $wert)
          {
            $propertyItem = new Shopgate_Model_Catalog_Property();
            $propertyItem->setUid($value->kMerkmal.'_'.$wert->kMerkmalWert);
            $propertyItem->setLabel($this->_pss($value->cName));
            $propertyItem->setValue($this->_pss($wert->cWert));
            $productItem->addProperty($propertyItem);
          }
        }
      }
    }
    if ($this->oExportArticle->cVPE == 'Y')
    {
      $cVpeEinheit = $this->oExportArticle->cVPEEinheit;
      $fVpeWert = $this->oExportArticle->fVPEWert;
      if (preg_match('/^[0-9]{1,}/',$cVpeEinheit, $matches))
      {
        $multiplice = $matches[0];
        $vpe_string = preg_replace('/^[0-9]{1,}/', '', $cVpeEinheit, 1);
        $vpe_string = trim($vpe_string);
        
        $lowwer = strtolower($vpe_string);
        //if (in_array($lowwer, array('g','ml','gramm','milliliter')) && $multiplice > 10)
          //$multiplice *= 10;
        
        $cVpeEinheit = $vpe_string;
        $fVpeWert *= $multiplice;
      }
      if (round($fVpeWert) == $fVpeWert)
        $tmp = round($fVpeWert);
      elseif (round($fVpeWert, 2) == $fVpeWert)
        $tmp = number_format($fVpeWert,2,',','');
      else
        $tmp = number_format($fVpeWert,3,',','');
      
      $propertyItem = new Shopgate_Model_Catalog_Property();
      $propertyItem->setUid('vpe_inhalt');
      $propertyItem->setLabel($this->_pss('Inhalt'));
      $propertyItem->setValue($this->_pss($tmp.' '.$cVpeEinheit));
      $productItem->addProperty($propertyItem);
    }
    
    if (isset($this->oExportArticle->cArtNr) && !empty($this->oExportArticle->cArtNr) && $this->oExportArticle->cArtNr != 'n/a')
    {
      $identifierItem = new Shopgate_Model_Catalog_Identifier();
      $identifierItem->setUid(md5('cArtNr'));
      $identifierItem->setType($this->_pss('sku'));
      $identifierItem->setValue($this->_pss($this->oExportArticle->cArtNr));
      $productItem->addIdentifier($identifierItem);
    }
    if (isset($this->oExportArticle->cISBN) && !empty($this->oExportArticle->cISBN) && $this->oExportArticle->cISBN != 'n/a')
    {
      if (!preg_match('/[^\d\-]/', $this->oExportArticle->cISBN) && strlen($this->oExportArticle->cISBN) <= 16)
      {
        $identifierItem = new Shopgate_Model_Catalog_Identifier();
        $identifierItem->setUid(md5('cISBN'));
        $identifierItem->setType($this->_pss('isbn'));
        $identifierItem->setValue($this->_pss($this->oExportArticle->cISBN));
        $productItem->addIdentifier($identifierItem);
      }
    }
    /*
    if (isset($this->oExportArticle->cASIN) && !empty($this->oExportArticle->cASIN) && $this->oExportArticle->cASIN != 'n/a')
    {
      $identifierItem = new Shopgate_Model_Catalog_Identifier();
      $identifierItem->setUid(md5('cASIN'));
      $identifierItem->setType($this->_pss('asin'));
      $identifierItem->setValue($this->_pss($this->oExportArticle->cASIN));
      $productItem->addIdentifier($identifierItem);
    }
    */
    if (isset($this->oExportArticle->cBarcode) && !empty($this->oExportArticle->cBarcode) && $this->oExportArticle->cBarcode != 'n/a')
    {
      if (!preg_match('/[^\d]/', $this->oExportArticle->cBarcode))
      {
        $identifierItem = new Shopgate_Model_Catalog_Identifier();
        $identifierItem->setUid(md5('cEAN'));
        $identifierItem->setType($this->_pss('ean'));
        $identifierItem->setValue($this->_pss($this->oExportArticle->cBarcode));
        $productItem->addIdentifier($identifierItem);
      }
    }
    if (isset($this->oExportArticle->cUPC) && !empty($this->oExportArticle->cUPC) && $this->oExportArticle->cUPC != 'n/a')
    {
      if (!preg_match('/[^\d]/', $this->oExportArticle->cUPC) && strlen($this->oExportArticle->cUPC) <= 12)
      {
        $identifierItem = new Shopgate_Model_Catalog_Identifier();
        $identifierItem->setUid(md5('cUPC'));
        $identifierItem->setType($this->_pss('upc'));
        $identifierItem->setValue($this->_pss($this->oExportArticle->cUPC));
        $productItem->addIdentifier($identifierItem);
      }
    }
    /*
    if (isset($this->oExportArticle->cUNNummer) && !empty($this->oExportArticle->cUNNummer) && $this->oExportArticle->cUNNummer != 'n/a')
    {
      $identifierItem = new Shopgate_Model_Catalog_Identifier();
      $identifierItem->setUid(md5('cUNNummer'));
      $identifierItem->setType($this->_pss('UNNummer'));
      $identifierItem->setValue($this->_pss($this->oExportArticle->cUNNummer));
      $productItem->addIdentifier($identifierItem);
    }
    */
    
    if (isset($this->oExportArticle->cSuchbegriffe) && !empty($this->oExportArticle->cSuchbegriffe))
    {
      $tmp = explode('  ', $this->oExportArticle->cSuchbegriffe);
      foreach ($tmp as $ttag)
      {
        $ttag = trim($ttag);
        if (!empty($ttag))
        {
          $tagItem = new Shopgate_Model_Catalog_Tag();
          //$tagItem->setUid(md5($ttag));
          $tagItem->setValue($this->_pss($ttag));
          $productItem->addTag($tagItem);
        }
      }
    }
    
    $lagerFilter = '';
    if ($this->aTeinstellungenFields['artikel_artikelanzeigefilter'] == EINSTELLUNGEN_ARTIKELANZEIGEFILTER_LAGER)
      $lagerFilter = "AND (NOT (tartikel.fLagerbestand <= 0 AND tartikel.cLagerBeachten = 'Y') OR tartikel.cLagerVariation = 'Y')";
    if ($this->aTeinstellungenFields['artikel_artikelanzeigefilter'] == EINSTELLUNGEN_ARTIKELANZEIGEFILTER_LAGERNULL)
      $lagerFilter = "AND (NOT (tartikel.fLagerbestand <= 0 AND tartikel.cLagerBeachten = 'Y') OR tartikel.cLagerKleinerNull = 'Y' OR tartikel.cLagerVariation = 'Y')";
    // relations
    if ($this->aTeinstellungenFields['artikeldetails_xselling_standard_anzeigen'] == 'Y')
    {
      $xsell = $GLOBALS["DB"]->executeQuery("SELECT txsell.* from txsell, tartikel 
        WHERE txsell.kXSellArtikel = tartikel.kArtikel " . $lagerFilter. " 
        AND txsell.kArtikel = " . $this->oExportArticle->kArtikel . "
        ORDER BY tartikel.cName",
        2
      );
      if (count($xsell)>0)
      {
        foreach ($xsell as $xs)
        {
          $relationModel = new Shopgate_Model_Catalog_Relation();
          $relationModel->setType('upsell');
          $relationModel->addValue($xs->kXSellArtikel);
          $productItem->addRelation($relationModel);
        }
      }
    }
    
    if ($this->aTeinstellungenFields['artikeldetails_xselling_kauf_anzeigen'] == 'Y')
    {
      $xsell = $GLOBALS["DB"]->executeQuery("SELECT * FROM txsellkauf 
        WHERE kArtikel=".$this->oExportArticle->kArtikel." ORDER BY nAnzahl DESC, RAND() LIMIT ".$this->aTeinstellungenFields['artikeldetails_xselling_kauf_anzahl'],
        2
      );
      if (count($xsell)>0)
      {
        foreach ($xsell as $xs)
        {
          $relationModel = new Shopgate_Model_Catalog_Relation();
          $relationModel->setType('ordered_with');
          $relationModel->addValue($xs->kXSellArtikel);
          $productItem->addRelation($relationModel);
        }
      }
    }
    
    $visibilityItem = new Shopgate_Model_Catalog_Visibility();
    $visibilityItem->setLevel(Shopgate_Model_Catalog_Visibility::DEFAULT_VISIBILITY_CATALOG_AND_SEARCH);
    $visibilityItem->setMarketplace(1);
    $productItem->setVisibility($visibilityItem);
    
    if (isset($this->oExportArticle->aInputAttribute) && !empty($this->oExportArticle->aInputAttribute))
    {
      foreach ($this->oExportArticle->aInputAttribute as $value)
      {
        $inputItem = new Shopgate_Model_Catalog_Input();
        $inputItem->setType(Shopgate_Model_Catalog_Input::DEFAULT_INPUT_TYPE_TEXT);
        $inputItem->setUid($value['kEigenschaft']);
        $inputItem->setRequired($value['nRequired']);
        $inputItem->setLabel($this->_pss($value['cName']));
        $inputItem->setInfoText($this->_pss($value['cInfoText']));
        $inputItem->setAdditionalPrice(0);
        
        $validationItem = new Shopgate_Model_Catalog_Validation();
        $validationItem->setValidationType(Shopgate_Model_Catalog_Validation::DEFAULT_VALIDATION_VARIABLE_STRING);
        $inputItem->setValidation($validationItem);
        $productItem->addInput($inputItem);
      }
    }
    
    if (isset($this->oExportArticle->aSelectAttribute) && !empty($this->oExportArticle->aSelectAttribute))
    {
      foreach ($this->oExportArticle->aSelectAttribute as $value)
      {
        $inputItem = new Shopgate_Model_Catalog_Input();
        $inputItem->setType(Shopgate_Model_Catalog_Input::DEFAULT_INPUT_TYPE_SELECT);
        $inputItem->setUid($value['kEigenschaft']);
        $inputItem->setRequired($value['nRequired']);
        $inputItem->setLabel($this->_pss($value['cName']));
        //$inputItem->setInfoText($this->_pss($value['cInfoText']));// Habe ich einen Infotext für die Attribute? in aArtikel prüfen!
        $sold_out = count($value['aWerte']);
        foreach ($value['aWerte'] as $row)
        {
          if ($this->oExportArticle->cStatusAktiv == 'active' || ($this->oExportArticle->cStatusAktiv == 'stock' && ($row['fLagerbestand'] > 0 || $this->oExportArticle->cLagerVariation == 'N')))
          {
            if (!empty($row['kEigenschaftWert']))
            {
              $optionModel = new Shopgate_Model_Catalog_Option();
              $optionModel->setUid($row['kEigenschaftWert']);
              $optionModel->setLabel($this->_pss($row['cName']));
              $optionModel->setValue($row['kEigenschaftWert']);
              $optionModel->setAdditionalPrice($row['fAufpreisNetto']);
              $inputItem->addOption($optionModel);
            }
            else
              $sold_out--;
          }
          else
            $sold_out--;
        }
        
        $validationItem = new Shopgate_Model_Catalog_Validation();
        $validationItem->setValidationType(Shopgate_Model_Catalog_Validation::DEFAULT_VALIDATION_VARIABLE_INT);
        $inputItem->setValidation($validationItem);
        if ($sold_out > 0)
          $productItem->addInput($inputItem);
      }
    }
    
    /* BOF 20160509 Konfigurator */
    if ($this->bAllowKonfiguration && $this->oExportArticle->bHasKonfig && !empty($this->oExportArticle->oKonfig_arr)) {
      foreach ($this->oExportArticle->oKonfig_arr as $oKonfigGruppe) {
        $nKonfigType = $oKonfigGruppe->getAnzeigeTyp();
        if ($nKonfigType == 0 || $nKonfigType == 3) {
          // mehrfachauswahl
          $oGruppeSprache = $oKonfigGruppe->getSprache();
          foreach ($oKonfigGruppe->oItem_arr as $oKonfigItem) {
            $inputItem = new Shopgate_Model_Catalog_Input();
            $inputItem->setType(Shopgate_Model_Catalog_Input::DEFAULT_INPUT_TYPE_SELECT);
            $inputItem->setUid('agiKonfig_'.$oKonfigGruppe->getKonfiggruppe().'_'.$oKonfigItem->getKonfigitem());
            $inputItem->setRequired(0);
            $inputItem->setLabel($this->_pss($oGruppeSprache->getName()));
            if ($oGruppeSprache->hatBeschreibung())
              $inputItem->setInfoText($this->_pss($oGruppeSprache->getBeschreibung()));
            /* add do not book this option */
            /*
              $optionModel = new Shopgate_Model_Catalog_Option();
              $optionModel->setUid($oKonfigGruppe->getKonfiggruppe().'_'.$oKonfigItem->getKonfigitem().'_0');
              $optionModel->setLabel($this->_pss('Nein'));
              $optionModel->setValue($oKonfigGruppe->getKonfiggruppe().'_'.$oKonfigItem->getKonfigitem().'_0');
              $optionModel->setAdditionalPrice(0);
              $inputItem->addOption($optionModel);
            */
            /* add book this option */
              $oItemSprache = $oKonfigItem->getSprache();
              $optionModel = new Shopgate_Model_Catalog_Option();
              $optionModel->setUid($oKonfigGruppe->getKonfiggruppe().'_'.$oKonfigItem->getKonfigitem().'_'.$oKonfigItem->getArtikelKey());
              if ($oKonfigItem->getUseOwnName())
                $optionModel->setLabel($this->_pss('1x '.$oKonfigItem->getName()));
              else
                $optionModel->setLabel($this->_pss('1x '.$oKonfigItem->getName()));
              $optionModel->setValue($oKonfigGruppe->getKonfiggruppe().'_'.$oKonfigItem->getKonfigitem().'_'.$oKonfigItem->getArtikelKey());
              $fItemPrice = $oKonfigItem->getPreis();
              if($fItemPrice <> 0)
                $optionModel->setAdditionalPrice(round($fItemPrice*100/(100+$this->oExportArticle->fMwSt) *$_SESSION['Waehrung']->fFaktor,4));
              else
                $optionModel->setAdditionalPrice(0);
              $inputItem->addOption($optionModel);
            /* add validation */
            $validationItem = new Shopgate_Model_Catalog_Validation();
            $validationItem->setValidationType(Shopgate_Model_Catalog_Validation::DEFAULT_VALIDATION_VARIABLE_STRING);
            //$validationItem->setValue('^\d+_\d+_\d+$');
            $inputItem->setValidation($validationItem);
            
            $productItem->addInput($inputItem);
          }
        }
        else {
          // einfachauswahl select
          $oGruppeSprache = $oKonfigGruppe->getSprache();
          $inputItem = new Shopgate_Model_Catalog_Input();
          $inputItem->setType(Shopgate_Model_Catalog_Input::DEFAULT_INPUT_TYPE_SELECT);
          $inputItem->setUid('agiKonfig_'.$oKonfigGruppe->getKonfiggruppe());
          $inputItem->setRequired(0);
          $inputItem->setLabel($this->_pss($oGruppeSprache->getName()));
          if ($oGruppeSprache->hatBeschreibung())
            $inputItem->setInfoText($this->_pss($oGruppeSprache->getBeschreibung()));
          foreach ($oKonfigGruppe->oItem_arr as $oKonfigItem) {
            $oItemSprache = $oKonfigItem->getSprache();
            $optionModel = new Shopgate_Model_Catalog_Option();
            $optionModel->setUid($oKonfigGruppe->getKonfiggruppe().'_'.$oKonfigItem->getKonfigitem().'_'.$oKonfigItem->getArtikelKey());
            $optionModel->setLabel($this->_pss($oKonfigItem->getName()));
            $optionModel->setValue($oKonfigGruppe->getKonfiggruppe().'_'.$oKonfigItem->getKonfigitem().'_'.$oKonfigItem->getArtikelKey());
            $fItemPrice = $oKonfigItem->getPreis();
            if($fItemPrice <> 0)
              $optionModel->setAdditionalPrice(round($fItemPrice*100/(100+$this->oExportArticle->fMwSt) *$_SESSION['Waehrung']->fFaktor,4));
            else
              $optionModel->setAdditionalPrice(0);
            $inputItem->addOption($optionModel);
          }
          $validationItem = new Shopgate_Model_Catalog_Validation();
          $validationItem->setValidationType(Shopgate_Model_Catalog_Validation::DEFAULT_VALIDATION_VARIABLE_STRING);
          //$validationItem->setValue('^\d+_\d+_\d+$');
          $inputItem->setValidation($validationItem);
          
          $productItem->addInput($inputItem);
        }
      }
    }
    /* BOF 20160509 Konfigurator */
    
    // Jetzt schauen, ob es Vater-Kind-Artikel gibt, wenn ja, die Attribute abgreifen und ausgeben
    if (isset($this->oExportArticle->nIstVater) && $this->oExportArticle->nIstVater == 1 && !$thisIsChild)
    {
      if (isset($this->oExportArticle->oVariationenNurKind_arr) && !empty($this->oExportArticle->oVariationenNurKind_arr))
      {
        foreach ($this->oExportArticle->oVariationenNurKind_arr as $row)
        {
          if ($this->oExportArticle->nIstVater)
          {
            $attributeGroupItem = new Shopgate_Model_Catalog_AttributeGroup();
            $attributeGroupItem->setUid($row->kEigenschaft);
            $attributeGroupItem->setLabel($this->_pss($row->cName));
            $productItem->addAttributeGroup($attributeGroupItem);
          }
        }
      }
      
      // Kinder auslesen
      $oVariationsKombiKinder_arr = $GLOBALS['DB']->executeQuery("SELECT DISTINCT tartikel.kArtikel
        FROM teigenschaftkombiwert
        JOIN tartikel ON tartikel.kEigenschaftKombi = teigenschaftkombiwert.kEigenschaftKombi
            AND tartikel.kVaterArtikel = " . $this->oExportArticle->kArtikel . "
        LEFT JOIN tartikelsichtbarkeit ON tartikel.kArtikel = tartikelsichtbarkeit.kArtikel
            AND tartikelsichtbarkeit.kKundengruppe=" . $this->kKundengruppe . "
        WHERE tartikelsichtbarkeit.kArtikel IS NULL
        ORDER BY tartikel.kArtikel, teigenschaftkombiwert.kEigenschaft", 2);
      if(is_array($oVariationsKombiKinder_arr) && count($oVariationsKombiKinder_arr) > 0)
      {
        foreach ($oVariationsKombiKinder_arr as $row)
        {
          $this->_loadOExportArticle($row->kArtikel);
          $childItem = $this->_prepareItemObject($row->kArtikel);
          $productItem->addChild($childItem);
        }
      }
    }
    elseif ($thisIsChild)
    {
      if (isset($this->oExportArticle->oVariationenNurKind_arr) && !empty($this->oExportArticle->oVariationenNurKind_arr))
      {
        foreach ($this->oExportArticle->oVariationenNurKind_arr as $row)
        {
          if(!empty($this->oExportArticle->kVaterArtikel) && !empty($row->Werte[0]))
          {
            $attributeItem = new Shopgate_Model_Catalog_Attribute();
            $attributeItem->setUid($row->Werte[0]->kEigenschaftWert);
            $attributeItem->setGroupUid($row->kEigenschaft);
            $attributeItem->setLabel($this->_pss($this->_remove_html($row->Werte[0]->cName)));
            $productItem->addAttribute($attributeItem);
          }
        }
      }
    }
    if (!$thisIsChild)
      $this->addItemModel($productItem);
    else
      return $productItem;
  }
  protected function _loadShippingCost()
  {
    $this->oExportArticle->cLieferland        = $this->cLieferLand;
    $this->oExportArticle->nOhneVersandkosten = 0;
    $this->oExportArticle->fVersandkosten     = 0;
    if ($this->oExportArticle->kVersandklasse != $this->jtl_plugin_configuration['nStandardVersandklasse'])
    {
      if (!array_key_exists($this->oExportArticle->kVersandklasse, $this->aVersandart))
      {
        $this->aVersandart[$this->oExportArticle->kVersandklasse] = $GLOBALS["DB"]->executeQuery(
          " SELECT *
            FROM tversandart
						WHERE cNurAbhaengigeVersandart='" . $this->cNurAbhaengigeVersandart . "'
						AND cLaender like \"%" . $this->cLieferLand . "%\"
						AND (
              cVersandklassen='-1' or 
              (cVersandklassen like \"% " . $this->oExportArticle->kVersandklasse . " %\" or cVersandklassen like \"% " . $this->oExportArticle->kVersandklasse . "\")
            )
						AND (cKundengruppen='-1' or cKundengruppen like '%;" . intval($this->kKundengruppe) . ";%') order by nSort
          ",
          2
        );
      }
      $price = 0;
      $fStandardVersand = 0;
      foreach ($this->aVersandart[$this->jtl_plugin_configuration['nStandardVersandklasse']] as $row)
      {
        $tmp = berechneVersandpreis($row, $this->cLieferLand, null, $this->oExportArticle);
        if ($tmp != -1)
        {
          $fStandardVersand = $tmp;
          break;
        }
      }
      
      $fArtikelVersand = 0;
      foreach ($this->aVersandart[$this->oExportArticle->kVersandklasse] as $row)
      {
        $tmp = berechneVersandpreis($row, $this->cLieferLand, null, $this->oExportArticle);
        if ($tmp != -1)
        {
          if ($tmp > 0 && $tmp < $fArtikelVersand)
            $fArtikelVersand = $tmp;
          elseif ($fArtikelVersand == 0 && $tmp > 0)
            $fArtikelVersand = $tmp;
          break;
        }
        else
          continue;
      }
      $differenz = $fArtikelVersand - $fStandardVersand;
      if ($differenz > 0)
        $differenz = $differenz*100/(100+$this->oExportArticle->fUst);
      $this->oExportArticle->fVersandkosten = $differenz < 0 ? 0 : round($differenz,3);
    }
  }
  protected function _loadItemCategories()
  {
    $this->oExportArticle->aAGIKategorien = array();
    if(count($this->oExportArticle->oKategorie_arr) > 0)
    {
      foreach ($this->oExportArticle->oKategorie_arr as $oCategorie)
      {
        if (is_int($oCategorie)) {
          $oKat = new Kategorie($oCategorie, $this->kSprache, $this->kKundengruppe);
          $kat = array(
            'kKategorie'    => $oKat->kKategorie,
            'cName'         => $oKat->cName,
            'nSort'         => $oKat->nSort,
            'kOberKategorie'=> $oKat->kOberKategorie,
            'cDeeplink'     => $oKat->cURL,
            'paths'         => array(),
            'image'         => $oKat->nBildVorhanden ? $oKat->cBild : ''
          );
          $i = 1;
          foreach ($oKat->cKategoriePfad_arr as $value)
          {
            $kat['paths'][$i] = $value;
            $i++;
          }
        }
        else {
          $kat = array(
            'kKategorie'    => $oCategorie->kKategorie,
            'cName'         => $oCategorie->cName,
            'nSort'         => $oCategorie->nSort,
            'kOberKategorie'=> $oCategorie->kOberKategorie,
            'cDeeplink'     => $oCategorie->cURL,
            'paths'         => array(),
            'image'         => $oCategorie->nBildVorhanden ? $oCategorie->cBild : ''
          );
          $i = 1;
          foreach ($oCategorie->cKategoriePfad_arr as $value)
          {
            $kat['paths'][$i] = $value;
            $i++;
          }
        }
        $this->oExportArticle->aAGIKategorien[] = $kat;
      }
    }
  }
  protected function _itemCheckHightlight()
  {
    if (isset($this->jtl_plugin_configuration['cTopArticleAsHighlight']) && $this->jtl_plugin_configuration['cTopArticleAsHighlight'] == 'Y')
      $this->oExportArticle->nTopArtikel = $this->oExportArticle->cTopArtikel == 'Y' ? 1 : 0;
    else
      $this->oExportArticle->nTopArtikel = 0;
      
    $tmp = $GLOBALS["DB"]->executeQuery(
      "SELECT 
        COUNT(*) AS icount 
      FROM 
        `tartikelsonderpreis` 
      WHERE 
        `kArtikel`='".$this->oExportArticle->kArtikel."' AND `cAktiv`='Y' AND `dStart` <= date(now()) AND (`dEnde` >= date(now()) OR `dEnde`='0000-00-00')
      ",
      1
    );
    if (!empty($tmp->icount))
      $this->oExportArticle->nTopArtikel = 1;
    return true;
  }
  protected function _itemCheckStatusAktiv()
  {
    /** BOF dErscheinungsdatum 2014-01-17 */
    if (isset($this->oExportArticle->nErscheinendesProdukt) && $this->oExportArticle->nErscheinendesProdukt)
    {
      /* BOF Erscheinungsdatum als Lieferzeit 2.13 2015-12-29  */
      if ($this->aTeinstellungenFields['global_erscheinende_kaeuflich'] == 'Y') {
        $this->oExportArticle->cStatusAktiv = 'active';
        $this->oExportArticle->fLagerbestand= 0;
        $this->oExportArticle->nUseStock = 0;
      }
      else {
        $this->oExportArticle->cStatusAktiv = 'active';
        $this->oExportArticle->fLagerbestand= 0;
        $this->oExportArticle->nUseStock = 1;
      }
      return true;
      /* EOF Erscheinungsdatum als Lieferzeit 2.13 2015-12-29 */
    }
    /** EOF dErscheinungsdatum 2014-01-17 */
    /** BOF Bestandspuffer und Lagerbestand 2014-04-07 */
    if (array_key_exists('aBestandspuffer', $this->jtl_plugin_configuration) && !empty($this->jtl_plugin_configuration['aBestandspuffer']) && $this->oExportArticle->fLagerbestand > 0)
    {
      foreach ($this->jtl_plugin_configuration['aBestandspuffer'] as $maxKey => $setValue)
      {
        if ($this->oExportArticle->fLagerbestand <= $maxKey)
        {
          $this->oExportArticle->fLagerbestand -= $setValue;
          if ($this->oExportArticle->fLagerbestand < 0)
            $this->oExportArticle->fLagerbestand = 0;
          break;
        }
      }
    }
    if (array_key_exists('maxBestand', $this->jtl_plugin_configuration) && !empty($this->jtl_plugin_configuration['maxBestand']))
    {
      if ($this->oExportArticle->fLagerbestand > $this->jtl_plugin_configuration['maxBestand'])
        $this->oExportArticle->fLagerbestand = $this->jtl_plugin_configuration['maxBestand'];
    }
    /** EOF Bestandspuffer und Lagerbestand 2014-04-07 */
    
    foreach ($this->inaktivStatus_Arr as $tmp)
    {
      if (strtolower($this->oExportArticle->cLieferstatus) == $tmp)
      {
        $this->oExportArticle->cStatusAktiv = 'inactive';
        $this->oExportArticle->fLagerbestand= 0;
        $this->oExportArticle->nUseStock = 1;
        return true;
      }
    }
    
    if (isset($this->oExportArticle->inWarenkorbLegbar) && $this->oExportArticle->inWarenkorbLegbar == -1)
    {
      $this->oExportArticle->cStatusAktiv = 'inactive';
      $this->oExportArticle->fLagerbestand= 0;
      $this->oExportArticle->nUseStock = 1;
      return true;
    }

    if (isset($this->oExportArticle->cLagerBeachten))
    {
      if ($this->oExportArticle->cLagerBeachten == 'Y')
      {
        if (isset($this->oExportArticle->cLagerKleinerNull) && $this->oExportArticle->cLagerKleinerNull == 'Y')
        {
          $this->oExportArticle->cStatusAktiv = 'active';
          $this->oExportArticle->nUseStock = 0;
        }
        else
        {
          $this->oExportArticle->cStatusAktiv = 'stock';
          $this->oExportArticle->nUseStock = 1;
        }
      }
      else
      {
        $this->oExportArticle->cStatusAktiv = 'active';
        $this->oExportArticle->nUseStock = 0;
      }
    }
    else
    {
      $this->oExportArticle->cStatusAktiv = 'stock';
      $this->oExportArticle->nUseStock = 1;
    }
    
    if ($this->oExportArticle->fLagerbestand < 0)
      $this->oExportArticle->fLagerbestand = 0;
    return true;
  }
  protected function _itemGetGrundpreis()
  {
    $this->oExportArticle->Grundpreis = '';
    if ($this->oExportArticle->cVPE == 'Y' && $this->oExportArticle->fVPEWert > 0)
    {
      $this->oExportArticle->Grundpreis = $this->oExportArticle->cLocalizedVPE[1];
    }
  }
  protected function _itemLoadAttributes()
  {
    $this->oExportArticle->aInputAttribute = array();
    $this->oExportArticle->aSelectAttribute = array();
    $aVaterKindEigenschaftenKeys = array();
    if (isset($this->oExportArticle->oVariationenNurKind_arr) && !empty($this->oExportArticle->oVariationenNurKind_arr))
    {
      foreach ($this->oExportArticle->oVariationenNurKind_arr as $row)
        $aVaterKindEigenschaftenKeys[] = $row->kEigenschaft;
    }
    if (isset($this->oExportArticle->Variationen) && !empty($this->oExportArticle->Variationen) && !$this->oExportArticle->nIstVater)
    {
      foreach ($this->oExportArticle->Variationen as $variation)
      {
        if ($variation->nLieferbareVariationswerte <= 0 || in_array($variation->kEigenschaft, $aVaterKindEigenschaftenKeys))
          continue;
        
        if (strpos(strtolower($variation->cTyp), 'freifeld') !== FALSE)
        {
          $data = array(
            'kEigenschaft'  => $variation->kEigenschaft,
            'cType'         => 'text',
            'cName'         => $variation->cName,
            'cInfoText'     => '',
            'nRequired'     => strpos(strtolower($variation->cTyp), 'pflicht-freifeld') !== FALSE ? 1 : 0
          );
          $this->oExportArticle->aInputAttribute[$variation->kEigenschaft] = $data;
        }
        else
        {
          $data = array(
            'kEigenschaft'  => $variation->kEigenschaft,
            'cType'         => 'select',
            'cName'         => $variation->cName,
            'aWerte'        => array(),
            'nRequired'     => 1
          );
          foreach ($variation->Werte as $value)
          {
            $dataWert = array(
              'kEigenschaftWert'  => $value->kEigenschaftWert,
              'cName'             => $value->cName,
              'fAufpreisNetto'    => isset($value->fAufpreis) ? $value->fAufpreis[1] *$_SESSION['Waehrung']->fFaktor : 0,
              'fAufpreisBrutto'   => isset($value->fAufpreis) ? $value->fAufpreis[0] *$_SESSION['Waehrung']->fFaktor : 0,
              'cArtNr'            => empty($value->cArtNr) ? '' : $value->cArtNr,
              'fGewichtDiff'      => empty($value->fGewichtDiff) ? 0 : $value->fGewichtDiff,
              'fLagerbestand'     => empty($value->fLagerbestand) ? 0 : $value->fLagerbestand
            );
            $data['aWerte'][$value->kEigenschaftWert] = $dataWert;
          }
          if (!empty($data['aWerte']))
            $this->oExportArticle->aSelectAttribute[$variation->kEigenschaft] = $data;
        }
      }
    }
  }
  protected function _itemSetLieferstatus()
  {
    $this->oExportArticle->agiLieferstatus = $this->oExportArticle->cLieferstatus;
    /* BOF Erscheinungsdatum als Lieferzeit 2.13 2015-12-29 */
    if (isset($this->oExportArticle->nErscheinendesProdukt) && $this->oExportArticle->nErscheinendesProdukt) {
      $this->oExportArticle->agiLieferstatus = $this->tSprache['productAvailableFrom'].': '.date('d.m.Y',strtotime($this->oExportArticle->dErscheinungsdatum));
      if ($this->aTeinstellungenFields['global_erscheinende_kaeuflich'] == 'Y')
        $this->oExportArticle->agiLieferstatus .= ' ('.$this->tSprache['preorderPossible'].')';
      if (isset($this->oExportArticle->cEstimatedDelivery) && !empty($this->oExportArticle->cEstimatedDelivery))
      {
        if (array_key_exists('shippingTime', $this->tSprache))
          $this->oExportArticle->agiLieferstatus .= ', '.$this->tSprache['shippingTime'].': '.$this->oExportArticle->cEstimatedDelivery;
        else
          $this->oExportArticle->agiLieferstatus .= ', '.$status_text[] = $this->oExportArticle->cEstimatedDelivery;
      }
      return;
    }
    /* EOF Erscheinungsdatum als Lieferzeit 2.13 2015-12-29 */
    if (isset($this->jtl_plugin_configuration['cNehmeLieferzeit']) && $this->jtl_plugin_configuration['cNehmeLieferzeit'] == 'kombiampelstatuszeit')
    {
      $status_text = array();
      if (!$this->oExportArticle->nErscheinendesProdukt)
      {
        $anzeige = $this->aTeinstellungenFields['artikel_lagerbestandsanzeige'];
        if (
          $anzeige != "nichts" && $this->oExportArticle->cLagerBeachten == 'Y' && 
          ($this->oExportArticle->cLagerKleinerNull == 'N' || $this->aTeinstellungenFields['artikeldetails_lieferantenbestand_anzeigen'] == 'U') && 
          $this->oExportArticle->fLagerbestand <= 0 && $this->oExportArticle->fZulauf > 0 && isset($this->oExportArticle->dZulaufDatum_de)
        )
        {
          if (array_key_exists('productInflowing', $this->tSprache))
          {
            $s_count = substr_count($this->tSprache['productInflowing'], '%s');
            if ($s_count == 2)
              $status_text[] = sprintf($this->tSprache['productInflowing'], $this->oExportArticle->fZulauf, $this->oExportArticle->dZulaufDatum_de);
            elseif ($s_count == 1)
              $status_text[] = sprintf($this->tSprache['productInflowing'], $this->oExportArticle->dZulaufDatum_de);
            else
              $status_text[] = $this->tSprache['productInflowing'];
          }
        }
        elseif (
          $anzeige != "nichts" && $this->aTeinstellungenFields['artikeldetails_lieferantenbestand_anzeigen'] != 'N' &&
          $this->oExportArticle->cLagerBeachten == 'Y' && $this->oExportArticle->fLagerbestand <= 0 && 
          $this->oExportArticle->fLieferantenlagerbestand > 0 && $this->oExportArticle->fLieferzeit > 0 &&
          ($this->oExportArticle->cLagerKleinerNull == 'N' || $this->aTeinstellungenFields['artikeldetails_lieferantenbestand_anzeigen'] == 'U')
        )
        {
          if (array_key_exists('supplierStockNotice', $this->tSprache))
          {
            $s_count = substr_count($this->tSprache['supplierStockNotice'], '%s');
            if ($s_count == 1)
              $status_text[] = sprintf($this->tSprache['supplierStockNotice'], $this->oExportArticle->fLieferzeit);
            else
              $status_text[] = $this->tSprache['supplierStockNotice'];
          }
        }
        elseif ($anzeige=='verfuegbarkeit' || $anzeige=='genau')
        {
          $status_text[] = $this->oExportArticle->Lageranzeige->cLagerhinweis[$anzeige];
        }
        elseif ($anzeige=='ampel')
        {
          $status_text[] = $this->oExportArticle->Lageranzeige->AmpelText;
        }
        
        if (
          !empty($this->oExportArticle->cLieferstatus) &&
          (
            $this->aTeinstellungenFields['artikeldetails_lieferstatus_anzeigen'] == 'Y' || 
            (
              $this->aTeinstellungenFields['artikeldetails_lieferstatus_anzeigen'] == 'L' && $this->oExportArticle->fLagerbestand == 0 && $this->oExportArticle->cLagerBeachten == 'Y' 
            ) || 
            (
              $this->aTeinstellungenFields['artikeldetails_lieferstatus_anzeigen'] == 'A' && 
              (
                $this->oExportArticle->fLagerbestand > 0 || $this->oExportArticle->cLagerKleinerNull == 'Y' || $this->oExportArticle->cLagerBeachten != 'Y'
              )
            )
          )
        )
        {
          if (array_key_exists('deliveryStatus', $this->tSprache))
            $status_text[] = $this->tSprache['deliveryStatus'].': '.$this->oExportArticle->cLieferstatus;
          elseif (array_key_exists('shippingTimeLP', $this->tSprache))
            $status_text[] = $this->tSprache['shippingTimeLP'].': '.$this->oExportArticle->cLieferstatus;
          else
            $status_text[] = $this->oExportArticle->cLieferstatus;
        }
        if (isset($this->oExportArticle->cEstimatedDelivery) && !empty($this->oExportArticle->cEstimatedDelivery))
        {
          if (array_key_exists('shippingTime', $this->tSprache))
            $status_text[] = $this->tSprache['shippingTime'].': '.$this->oExportArticle->cEstimatedDelivery;
          else
            $status_text[] = $this->oExportArticle->cEstimatedDelivery;
        }
        if (!empty($status_text))
          $this->oExportArticle->agiLieferstatus = implode(', ', $status_text);
        
      }
    }
    elseif (isset($this->jtl_plugin_configuration['cNehmeLieferzeit']) && $this->jtl_plugin_configuration['cNehmeLieferzeit'] == 'kombistatuszeit')
    {
      //shippingTimeLP,shippingTime
      $status_text = array();
      if (!empty($this->oExportArticle->cLieferstatus))
      {
        if (array_key_exists('deliveryStatus', $this->tSprache))
          $status_text[] = $this->tSprache['deliveryStatus'].': '.$this->oExportArticle->cLieferstatus;
        elseif (array_key_exists('shippingTimeLP', $this->tSprache))
          $status_text[] = $this->tSprache['shippingTimeLP'].': '.$this->oExportArticle->cLieferstatus;
        else
          $status_text[] = $this->oExportArticle->cLieferstatus;
      }
      if (isset($this->oExportArticle->cEstimatedDelivery) && !empty($this->oExportArticle->cEstimatedDelivery))
      {
        if (array_key_exists('shippingTime', $this->tSprache))
          $status_text[] = $this->tSprache['shippingTime'].': '.$this->oExportArticle->cEstimatedDelivery;
        else
          $status_text[] = $this->oExportArticle->cEstimatedDelivery;
      }
      if (!empty($status_text))
        $this->oExportArticle->agiLieferstatus = implode(', ', $status_text);
    }
    elseif (isset($this->jtl_plugin_configuration['cNehmeLieferzeit']) && $this->jtl_plugin_configuration['cNehmeLieferzeit'] == 'ampel')
    {
      if (isset($this->oExportArticle->Lageranzeige) && isset($this->oExportArticle->Lageranzeige->AmpelText) && !empty($this->oExportArticle->Lageranzeige->AmpelText))
        $this->oExportArticle->agiLieferstatus = $this->oExportArticle->Lageranzeige->AmpelText;
      elseif (isset($this->oExportArticle->Lageranzeige) && isset($this->oExportArticle->Lageranzeige->cLagerhinweis) && !empty($this->oExportArticle->Lageranzeige->cLagerhinweis))
        $this->oExportArticle->agiLieferstatus = $this->oExportArticle->Lageranzeige->cLagerhinweis['verfuegbarkeit'];
    }
    elseif (isset($this->jtl_plugin_configuration['cNehmeLieferzeit']) && $this->jtl_plugin_configuration['cNehmeLieferzeit'] == 'zeit')
    {
      if (isset($this->oExportArticle->cEstimatedDelivery) && !empty($this->oExportArticle->cEstimatedDelivery)) {
        if (array_key_exists('shippingTime', $this->tSprache))
          $this->oExportArticle->agiLieferstatus = $this->tSprache['shippingTime'].': '.$this->oExportArticle->cEstimatedDelivery;
        else
          $this->oExportArticle->agiLieferstatus = $this->oExportArticle->cEstimatedDelivery;
      }
    }
    
    if (empty($this->oExportArticle->agiLieferstatus))
    {
      if (isset($this->oExportArticle->Lageranzeige) && isset($this->oExportArticle->Lageranzeige->AmpelText) && !empty($this->oExportArticle->Lageranzeige->AmpelText))
        $this->oExportArticle->agiLieferstatus = $this->oExportArticle->Lageranzeige->AmpelText;
      elseif (isset($this->oExportArticle->Lageranzeige) && isset($this->oExportArticle->Lageranzeige->cLagerhinweis) && !empty($this->oExportArticle->Lageranzeige->cLagerhinweis))
        $this->oExportArticle->agiLieferstatus = $this->oExportArticle->Lageranzeige->cLagerhinweis['verfuegbarkeit'];
      elseif (isset($this->oExportArticle->cEstimatedDelivery) && !empty($this->oExportArticle->cEstimatedDelivery)) {
        if (array_key_exists('shippingTime', $this->tSprache))
          $this->oExportArticle->agiLieferstatus = $this->tSprache['shippingTime'].': '.$this->oExportArticle->cEstimatedDelivery;
        else
          $this->oExportArticle->agiLieferstatus = $this->oExportArticle->cEstimatedDelivery;
      }
    }
  }
  protected function _itemPrepareDescription()
  {
    $this->oExportArticle->bAgiBenutzeKurzbeschreibung = false;
    if (empty($this->oExportArticle->cBeschreibung))
    {
      $this->oExportArticle->bAgiBenutzeKurzbeschreibung = true;
      $this->oExportArticle->cBeschreibung = $this->oExportArticle->cKurzBeschreibung;
    }
    
    $count_ps = substr_count($this->cWrapDescriptionAttribute, '%s');
    if (array_key_exists('artikeldetails_attribute_anhaengen', $this->aTeinstellungenFields) && $this->aTeinstellungenFields['artikeldetails_attribute_anhaengen'] == 'Y')
    {
      if (isset($this->oExportArticle->Attribute) && !empty($this->oExportArticle->Attribute))
      {
        foreach ($this->oExportArticle->Attribute as $attribut)
        {
          if ($count_ps === 2)
            $this->oExportArticle->cBeschreibung .= sprintf($this->cWrapDescriptionAttribute, htmlentities($attribut->cName, ENT_COMPAT, 'ISO-8859-15'), ' '.$attribut->cWert).chr(10);
          elseif ($count_ps === 1)
            $this->oExportArticle->cBeschreibung .= sprintf($this->cWrapDescriptionAttribute, $attribut->cWert).chr(10);
          else
            $this->oExportArticle->cBeschreibung .= htmlentities($attribut->cName, ENT_COMPAT, 'ISO-8859-15').': '.$attribut->cWert.chr(10);
        }
      }
    }
    elseif (!empty($this->aAddDescAttribute))
    {
      if (isset($this->oExportArticle->Attribute) && !empty($this->oExportArticle->Attribute))
      {
        foreach ($this->oExportArticle->Attribute as $attribut)
        {
          if (in_array(strtolower($attribut->cName), $this->aAddDescAttribute))
          {
            if ($count_ps === 2)
              $this->oExportArticle->cBeschreibung .= sprintf($this->cWrapDescriptionAttribute, htmlentities($attribut->cName, ENT_COMPAT, 'ISO-8859-15'), ' '.$attribut->cWert).chr(10);
            elseif ($count_ps === 1)
              $this->oExportArticle->cBeschreibung .= sprintf($this->cWrapDescriptionAttribute, $attribut->cWert).chr(10);
            else
              $this->oExportArticle->cBeschreibung .= htmlentities($attribut->cName, ENT_COMPAT, 'ISO-8859-15').': '.$attribut->cWert.chr(10);
          }
        }
      }
    }
    // Javascript entfernen
    $this->oExportArticle->cBeschreibung = preg_replace('/<script\b[^>]*>(.*?)<\/script>/is', "", $this->oExportArticle->cBeschreibung); // 2.06
    
    // URLs vervollständigen
    $result = preg_match_all('/(src|href)=([\'"].*?[\'"])/i', $this->oExportArticle->cBeschreibung, $matches);
    if ($result && !empty($matches[2]))
    {
      $replaces = array('from'=>array(),'to'=>array());
      foreach ($matches[2] as $string)
      {
        $nSlash = strpos($string, '//');
        if (!preg_match("~https?://~i", $string) && ($nSlash === FALSE || $nSlash > 1))
        {
          $replaces['from'][] = $string;
          if (substr($string,1,1) == '/')
            $replaces['to'][] = substr($string,0,1).($this->shopURLEndSlash ? substr(URL_SHOP,0,count(URL_SHOP)-1) : URL_SHOP).substr($string,1);
          else
          {
            if ($pos = strrpos($string,'./') !== false)
            {
              $string = substr($string,0,1).substr($string, $pos+2);
              $replaces['to'][] = substr($string,0,1).($this->shopURLEndSlash ? substr(URL_SHOP,0,count(URL_SHOP)-1) : URL_SHOP).substr($string,1);
            }
            else
              $replaces['to'][] = substr($string,0,1).($this->shopURLEndSlash ? URL_SHOP : URL_SHOP.'/').substr($string,1);
          }
        }
      }
      $this->oExportArticle->cBeschreibung = str_replace($replaces['from'], $replaces['to'], $this->oExportArticle->cBeschreibung);
    }
    $this->oExportArticle->cBeschreibung = str_replace(
      '<object type="application/x-shockwave-flash" data="/includes/',
      '<object type="application/x-shockwave-flash" data="'.($this->shopURLEndSlash ? substr(URL_SHOP,0,count(URL_SHOP)-1) : URL_SHOP).'/includes/',
      $this->oExportArticle->cBeschreibung
    );
    $this->oExportArticle->cBeschreibung = str_replace(
      '<param name="movie" value="/includes/',
      '<param name="movie" value="'.($this->shopURLEndSlash ? substr(URL_SHOP,0,count(URL_SHOP)-1) : URL_SHOP).'/includes/',
      $this->oExportArticle->cBeschreibung
    );
    $this->oExportArticle->cBeschreibung = str_replace(
      '<param name="FlashVars" value="flv=/mediafiles/',
      '<param name="FlashVars" value="flv='.($this->shopURLEndSlash ? substr(URL_SHOP,0,count(URL_SHOP)-1) : URL_SHOP).'/mediafiles/',
      $this->oExportArticle->cBeschreibung
    );
    
    if (isset($this->oExportArticle->oMedienDatei_arr) && !empty($this->oExportArticle->oMedienDatei_arr))
    {
      foreach($this->oExportArticle->oMedienDatei_arr as $i => $oMedienDatei)
      {
        if ($oMedienDatei->nErreichbar)
        {
          $link = '<a href="'.$oMedienDatei->cURL.'">'.$oMedienDatei->cURL.'</a>';
          if ($count_ps === 2)
            $this->oExportArticle->cBeschreibung .= sprintf($this->cWrapDescriptionAttribute, htmlentities($oMedienDatei->cName, ENT_COMPAT, 'ISO-8859-15'), ' '.$link).chr(10);
          elseif ($count_ps === 1)
            $this->oExportArticle->cBeschreibung .= sprintf($this->cWrapDescriptionAttribute, $link).chr(10);
          else
            $this->oExportArticle->cBeschreibung .= htmlentities($oMedienDatei->cName, ENT_COMPAT, 'ISO-8859-15').': '.$link.chr(10);
        }
      }
    }
  }
  protected function _itemLadeMediendateien()
  {
    $this->oExportArticle->oMedienDatei_arr = $GLOBALS["DB"]->executeQuery("
      SELECT 
        tmediendatei.kMedienDatei, tmediendatei.cPfad, tmediendatei.cURL, tmediendatei.cTyp, tmediendatei.nSort,
        tmediendateisprache.cName, tmediendateisprache.cBeschreibung, tmediendateisprache.kSprache
      FROM tmediendatei
      JOIN tmediendateisprache ON tmediendateisprache.kMedienDatei = tmediendatei.kMedienDatei
      WHERE tmediendatei.kArtikel=" . $this->oExportArticle->kArtikel . "
        AND tmediendateisprache.kSprache = " . $this->kSprache . "
      ORDER BY tmediendatei.nSort ASC, tmediendateisprache.cName",
    2);
    if(is_array($this->oExportArticle->oMedienDatei_arr) && count($this->oExportArticle->oMedienDatei_arr) > 0)
    {
      $cMedienTyp_arr = array();  // Wird im Template gebraucht um Tabs aufzubauen
      foreach($this->oExportArticle->oMedienDatei_arr as $i => $oMedienDatei)
      {
        $this->oExportArticle->oMedienDatei_arr[$i]->nErreichbar = 0;
        $this->oExportArticle->oMedienDatei_arr[$i]->nExtern = 0;
        if((strlen($oMedienDatei->cPfad) > 0) && file_exists(PFAD_ROOT . PFAD_MEDIAFILES . $oMedienDatei->cPfad))
        {
          $this->oExportArticle->oMedienDatei_arr[$i]->nErreichbar = 1;
          $this->oExportArticle->oMedienDatei_arr[$i]->cURL = $this->shopURLEndSlash ? URL_SHOP.PFAD_MEDIAFILES.$oMedienDatei->cPfad : URL_SHOP.'/'.PFAD_MEDIAFILES.$oMedienDatei->cPfad;
        }
        elseif(strlen($oMedienDatei->cURL) > 0)
        {
          $this->oExportArticle->oMedienDatei_arr[$i]->nErreichbar = 1;
          $this->oExportArticle->oMedienDatei_arr[$i]->nExtern = 1;
        }
      }
    }
  }
  protected function _getArticleSortInCategory($kKategorie)
  {
    $q = "SELECT ta.kArtikel AS id FROM `tkategorieartikel` AS tk LEFT JOIN `tartikel` AS ta ON ta.kArtikel=tk.kArtikel WHERE tk.kKategorie=$kKategorie ORDER BY ";
    if ($this->jtl_plugin_configuration['cTopAtFirst'] == 'Y')
      $q .= 'ta.cTopArtikel DESC, ';
    $q .= 'ta.'.$this->jtl_plugin_configuration['cSortArticle'].' '.$this->jtl_plugin_configuration['cSortArticleOrder'].', ta.cName ASC';
    $aRes = $GLOBALS["DB"]->executeQuery($q, 9);
    $nSort = 999999;
    foreach ($aRes as $row)
    {
      if ($row['id'] == $this->oExportArticle->kArtikel)
        break;
      $nSort--;
    }
    return $nSort;
  }
  
  public function setWrapDescriptionAttributeString($string)
  {
    if (!empty($string) && substr_count($string, '%s') == 2)
      $this->cWrapDescriptionAttribute = $string;
    else
      $this->cWrapDescriptionAttribute = '<p><strong>%s:</strong>%s</p>';
  }
  /// EOF ITEM-FUNCTIONS ///
  
  /// BOF CUSTOMER-FUNCTIONS ///
  public function getCustomer($user, $pass)
  {
    if (SG_DEBUG_LOGGING)
      $this->log('start getCustomer()', ShopgateLogger::LOGTYPE_DEBUG); // AGI debug_logging
    
    // Database connection can be established here
    // Check if the user’s e-mail address ($user) and password ($pass) exist in the shop system
    $JTL_Client = new Kunde();
    $userNotFoundOrPasswordIsWrong = $JTL_Client->holLoginKunde(
      $this->_prepare_string_for_jtl($user), 
      $this->_prepare_string_for_jtl($pass)
    ) === 1 ? FALSE : TRUE;
    // If user data is available, save them in, for instance, the $dbCustomerData array
    if (SG_DEBUG_LOGGING)
      $this->log($userNotFoundOrPasswordIsWrong ? 'Customer not found' : 'Customer found', ShopgateLogger::LOGTYPE_DEBUG); // AGI debug_logging

    // If there is no user or the password is wrong:
    if ($userNotFoundOrPasswordIsWrong)
      throw new ShopgateLibraryException(ShopgateLibraryException::PLUGIN_WRONG_USERNAME_OR_PASSWORD, 'User: '.$user);

    $dbCustomerData = $JTL_Client;

    // Additionally, retrieve user addresses and save them in, for instance, the $dbCustomerAddresses array:
    $addresses = array();
    // load delivery-addresses
    $dbCustomerAddresses = $GLOBALS["DB"]->executeQuery("SELECT * FROM `tlieferadresse` WHERE `kKunde`=".$dbCustomerData->kKunde, 9);
    if (SG_DEBUG_LOGGING)
      $this->log("QRY: SELECT * FROM `tlieferadresse` WHERE `kKunde`=".$dbCustomerData->kKunde, ShopgateLogger::LOGTYPE_DEBUG); // AGI debug_logging
    foreach ($dbCustomerAddresses as $dbCustomerAddress) 
    {
      if (SG_DEBUG_LOGGING)
        $this->log("Found delivery-Address ".$dbCustomerAddress['kLieferadresse'], ShopgateLogger::LOGTYPE_DEBUG); // AGI debug_logging
      
      if ($dbCustomerAddress['cAnrede'] == 'w')
        $gender = 'f';
      elseif ($dbCustomerAddress['cAnrede'] == 'm')
        $gender = 'm';
      else
        $gender = '';
      
      $address = new ShopgateAddress();
      $address->setId('D'.$dbCustomerAddress['kLieferadresse']);
      $address->setAddressType(ShopgateAddress::DELIVERY);
      if (!empty($gender))
        $address->setGender($gender);
      $address->setFirstName($this->_prepare_string_for_shopgate($dbCustomerAddress['cVorname']));
      $address->setLastName($this->_prepare_string_for_shopgate(entschluesselXTEA($dbCustomerAddress['cNachname'])));
      $address->setBirthday($dbCustomerData->dGeburtstag);
      $address->setCompany($this->_prepare_string_for_shopgate(entschluesselXTEA($dbCustomerAddress['cFirma'])));
      $address->setStreet1($this->_prepare_string_for_shopgate(entschluesselXTEA($dbCustomerAddress['cStrasse']).(empty($dbCustomerAddress['cHausnummer']) ? '' : ' '.$dbCustomerAddress['cHausnummer'])));
      $address->setStreet2($this->_prepare_string_for_shopgate($dbCustomerAddress['cAdressZusatz']));
      $address->setCity($this->_prepare_string_for_shopgate($dbCustomerAddress['cOrt']));
      $address->setZipcode($this->_prepare_string_for_shopgate($dbCustomerAddress['cPLZ']));
      $address->setCountry($this->_prepare_string_for_shopgate($dbCustomerAddress['cLand']));
      $address->setPhone($this->_prepare_string_for_shopgate($dbCustomerAddress['cTel']));
      $address->setMobile($this->_prepare_string_for_shopgate($dbCustomerAddress['cMobil']));
      $address->setMail($this->_prepare_string_for_shopgate($dbCustomerAddress['cMail']));
      $addresses[] = $address;
    }
    
    if ($dbCustomerData->cAnrede == 'w')
      $gender = 'f';
    elseif ($dbCustomerData->cAnrede == 'm')
      $gender = 'm';
    else
      $gender = '';
    $address = new ShopgateAddress();
    $address->setId('C'.$dbCustomerData->kKunde);
    $address->setIsInvoiceAddress(1);
    $address->setIsDeliveryAddress(1);
    if (!empty($gender))
      $address->setGender($gender);
    $address->setFirstName($this->_prepare_string_for_shopgate($dbCustomerData->cVorname));
    $address->setLastName($this->_prepare_string_for_shopgate($dbCustomerData->cNachname));
    $address->setBirthday($this->_prepare_string_for_shopgate($dbCustomerData->dGeburtstag));
    $address->setCompany($this->_prepare_string_for_shopgate($dbCustomerData->cFirma));
    $address->setStreet1($this->_prepare_string_for_shopgate($dbCustomerData->cStrasse.(empty($dbCustomerData->cHausnummer) ? '' : ' '.$dbCustomerData->cHausnummer)));
    $address->setStreet2($this->_prepare_string_for_shopgate($dbCustomerData->cAdressZusatz));
    $address->setCity($this->_prepare_string_for_shopgate($dbCustomerData->cOrt));
    $address->setZipcode($this->_prepare_string_for_shopgate($dbCustomerData->cPLZ));
    $address->setCountry($this->_prepare_string_for_shopgate($dbCustomerData->cLand));
    $address->setPhone($this->_prepare_string_for_shopgate($dbCustomerData->cTel));
    $address->setMobile($this->_prepare_string_for_shopgate($dbCustomerData->cMobil));
    $address->setMail($this->_prepare_string_for_shopgate($dbCustomerData->cMail));
    $addresses[] = $address;

    // If the user exists:
    if ($dbCustomerData->cAnrede == 'w')
      $gender = 'f';
    elseif ($dbCustomerData->cAnrede == 'm')
      $gender = 'm';
    else
      $gender = '';
    $customer = new ShopgateCustomer();
    $customer->setCustomerId($dbCustomerData->kKunde);
    $customer->setCustomerToken($dbCustomerData->kKunde);
    if (!empty($dbCustomerData->cKundenNr))
      $customer->setCustomerNumber($dbCustomerData->cKundenNr);
    if (!empty($dbCustomerData->kKundengruppe))
      $customer->setCustomerGroupId($dbCustomerData->kKundengruppe);
    $customer->setGender($gender);
    $customer->setFirstName($this->_prepare_string_for_shopgate($dbCustomerData->cVorname));
    $customer->setLastName($this->_prepare_string_for_shopgate($dbCustomerData->cNachname));
    $customer->setBirthday($this->_prepare_string_for_shopgate($dbCustomerData->dGeburtstag));
    $customer->setPhone($this->_prepare_string_for_shopgate($dbCustomerData->cTel));
    $customer->setMobile($this->_prepare_string_for_shopgate($dbCustomerData->cMobil));
    $customer->setMail($this->_prepare_string_for_shopgate($dbCustomerData->cMail));
    
    if (!empty($dbCustomerData->kKundengruppe))
    {
      $tmp = $GLOBALS["DB"]->executeQuery(
        "SELECT * FROM `tkundengruppe` WHERE `kKundengruppe`=".$dbCustomerData->kKundengruppe,
        1
      );
      if ($tmp)
      {
        $customerGroups = array();
          $group = new ShopgateCustomerGroup();
          $group->setID($dbCustomerData->kKundengruppe);
          $group->setName($this->_pss($tmp->cName));
        $customerGroups[] = $group;
        $customer->setCustomerGroups($customerGroups);
        $customer->setCustomerGroup($this->_pss($tmp->cName));
      }
    }
    
    //var_dump($addresses);
    // Important! Addresses need to be assigned to the customer here
    $customer->setAddresses($addresses);
    
    $customer = $this->agiGetCustomerIndividualAddData($customer);
    
    if (SG_DEBUG_LOGGING)
      $this->log('Submit Customer', ShopgateLogger::LOGTYPE_DEBUG); // AGI debug_logging
 
    return $customer;
  }
  public function registerCustomer($user, $pass, ShopgateCustomer $customer)
  {
    $this->agiRegisterCustomerIndividual($user, $pass, $customer);
    if (SG_DEBUG_LOGGING)
      $this->log('start registerCustomer()', ShopgateLogger::LOGTYPE_DEBUG); // AGI debug_logging
    
    $user = $GLOBALS['DB']->escape(filterXSS($this->_prepare_string_for_jtl($user)));
    $tmp = $GLOBALS["DB"]->executeQuery(
      "SELECT COUNT(*) AS icount FROM `tkunde` WHERE `cMail`='".$user."' AND `nRegistriert`=1",
      1
    );
    if ($tmp->icount > 0)
      throw new ShopgateLibraryException(ShopgateLibraryException::REGISTER_USER_ALREADY_EXISTS, '', true);
    
    $kKundengruppe = $this->kKundengruppe;
    $kSprache = $this->kSprache;
    
    $JTL_Client = new Kunde();
    if ($customer->getGender() == 'f')
      $JTL_Client -> cAnrede  = 'w';
    elseif ($customer->getGender() == 'm')
      $JTL_Client -> cAnrede  = 'm';
    else
      $JTL_Client -> cAnrede  = '';
    
    $JTL_Client -> cVorname = $this->_prepare_string_for_jtl($customer->getFirstName());
    $JTL_Client -> cNachname= $this->_prepare_string_for_jtl($customer->getLastName());
    $JTL_Client -> dGeburtstag = $this->_prepare_string_for_jtl($customer->getBirthday());
    $JTL_Client -> cTel     = $this->_prepare_string_for_jtl($customer->getPhone());
    $JTL_Client -> cMobil   = $this->_prepare_string_for_jtl($customer->getMobile());
    $JTL_Client -> cMail    = $this->_prepare_string_for_jtl($customer->getMail());
    $JTL_Client -> cPasswort = cryptPasswort($this->_prepare_string_for_jtl($pass));
    $JTL_Client -> nRegistriert = 1;
    $JTL_Client -> kKundengruppe= $kKundengruppe;
    $JTL_Client -> kSprache   = $kSprache;
    $JTL_Client -> dErstellt = date('Y-m-d');
    $JTL_Client -> cAktiv    = 'Y';
    $JTL_Client -> cAbgeholt = 'N';
    $JTL_Client -> cHerkunft = 'Shopgate';
    
    $adressen = array();
    $adressList = $customer->getAddresses();
    foreach ($adressList as $adress)
    {
      if ($adress->getIsInvoiceAddress())
        $adressen[] = $adress;
    }
    if (empty($adressen))
    {
      foreach ($adressList as $adress)
        $adressen[] = $adress;
    }
    $first = reset($adressen);
    if ($first)
    {
      $JTL_Client -> cFirma   = $this->_prepare_string_for_jtl($first->getCompany());	
      $JTL_Client -> cStrasse = $this->_prepare_string_for_jtl($first->getStreetName1());
      $JTL_Client -> cHausnummer  = $this->_prepare_string_for_jtl($first->getStreetNumber1());
      $JTL_Client -> cAdressZusatz= $this->_prepare_string_for_jtl($first->getStreet2());
      $JTL_Client -> cPLZ     = $this->_prepare_string_for_jtl($first->getZipcode());
      $JTL_Client -> cOrt     = $this->_prepare_string_for_jtl($first->getCity());
      $JTL_Client -> cLand    = $this->_prepare_string_for_jtl($first->getCountry());
    }
    if (SG_DEBUG_LOGGING)
      $this->log("Save new client", ShopgateLogger::LOGTYPE_DEBUG); // AGI debug_logging
    $JTL_Client -> insertInDB();
    
    if (!$JTL_Client -> kKunde)
      throw new ShopgateLibraryException(ShopgateLibraryException::REGISTER_FAILED_TO_ADD_USER, '', true);
    
    /* return data */
    $customer = new ShopgateCustomer();
    $customer->setCustomerId($JTL_Client->kKunde);
    $customer->setCustomerToken($JTL_Client->kKunde);
    if (!empty($JTL_Client->cKundenNr))
      $customer->setCustomerNumber($JTL_Client->cKundenNr);
    if (!empty($JTL_Client->kKundengruppe))
      $customer->setCustomerGroupId($JTL_Client->kKundengruppe);
    if ($JTL_Client->cAnrede == 'w')
      $gender = 'f';
    elseif ($JTL_Client->cAnrede == 'm')
      $gender = 'm';
    else
      $gender = '';
    $customer->setGender($gender);
    $customer->setFirstName($this->_prepare_string_for_shopgate($JTL_Client->cVorname));
    $customer->setLastName($this->_prepare_string_for_shopgate($JTL_Client->cNachname));
    $customer->setBirthday($this->_prepare_string_for_shopgate($JTL_Client->dGeburtstag));
    $customer->setPhone($this->_prepare_string_for_shopgate($JTL_Client->cTel));
    $customer->setMobile($this->_prepare_string_for_shopgate($JTL_Client->cMobil));
    $customer->setMail($this->_prepare_string_for_shopgate($JTL_Client->cMail));
    
    if ($first)
    {
      $customerAdress = array();
      if ($JTL_Client->cAnrede == 'w')
        $gender = 'f';
      elseif ($JTL_Client->cAnrede == 'm')
        $gender = 'm';
      else
        $gender = '';
      $address = new ShopgateAddress();
      $address->setId('C'.$JTL_Client->kKunde);
      $address->setIsInvoiceAddress(1);
      $address->setIsDeliveryAddress(1);
      if (!empty($gender))
        $address->setGender($gender);
      $address->setFirstName($this->_prepare_string_for_shopgate($JTL_Client->cVorname));
      $address->setLastName($this->_prepare_string_for_shopgate($JTL_Client->cNachname));
      $address->setBirthday($this->_prepare_string_for_shopgate($JTL_Client->dGeburtstag));
      $address->setCompany($this->_prepare_string_for_shopgate($JTL_Client->cFirma));
      $address->setStreet1($this->_prepare_string_for_shopgate($JTL_Client->cStrasse.(empty($JTL_Client->cHausnummer) ? '' : ' '.$JTL_Client->cHausnummer)));
      $address->setStreet2($this->_prepare_string_for_shopgate($JTL_Client->cAdressZusatz));
      $address->setCity($this->_prepare_string_for_shopgate($JTL_Client->cOrt));
      $address->setZipcode($this->_prepare_string_for_shopgate($JTL_Client->cPLZ));
      $address->setCountry($this->_prepare_string_for_shopgate($JTL_Client->cLand));
      $address->setPhone($this->_prepare_string_for_shopgate($JTL_Client->cTel));
      $address->setMobile($this->_prepare_string_for_shopgate($JTL_Client->cMobil));
      $address->setMail($this->_prepare_string_for_shopgate($JTL_Client->cMail));
      $customerAdress[] = $address;
      $customer->setAddresses($customerAdress);
    }
    
    if (SG_DEBUG_LOGGING)
      $this->log('Submit Customer', ShopgateLogger::LOGTYPE_DEBUG); // AGI debug_logging
    
    return $customer;
  }
  /// EOF CUSTOMER-FUNCTIONS ///
  
  
  /// BOF ORDER-FUNCTIONS ///
  public function addOrder(ShopgateOrder $order)
  {
    $this->agiAddOrderIndividual($order);
    if (SG_DEBUG_LOGGING)
      $this->log('start addOrder()', ShopgateLogger::LOGTYPE_DEBUG); // AGI debug_logging
    
    $additional_comments = array();
    $Einstellungen = getEinstellungen(array(CONF_GLOBAL));
    
    // Check if an order with the number ($order->getOrderNumber())
    // has already been imported from Shopgate
    $tmp = $GLOBALS["DB"]->executeQuery(
      "SELECT COUNT(*) AS icount FROM `xplugin_agi_shopgate_plugin_connect` WHERE `kBestellungShopgate`='".$order->getOrderNumber()."'",
      1
    );
    if (SG_DEBUG_LOGGING)
      $this->log("QRY: SELECT COUNT(*) AS icount FROM `xplugin_agi_shopgate_plugin_connect` WHERE `kBestellungShopgate`='".$order->getOrderNumber()."'", ShopgateLogger::LOGTYPE_DEBUG); // AGI debug_logging
    $dbOrder = $tmp->icount;
 
    // If an order is found with the order number ($order->getOrderNumber()) 
    // abort:
    if (!empty($dbOrder)) {
      throw new ShopgateLibraryException(ShopgateLibraryException::PLUGIN_DUPLICATE_ORDER);
    }

    // Has the order been placed by an existing customer?
    // Check if a user with the $customerId already exists and save the order
    // together with the user
    /* 
    // Bis Version 1.4 wurde anhand der "getExternalCustomerId" geprüft, ob ein Kunde bereits vorhanden ist
    // Dann teilte mir Sopgate mit, dass die Kunden-Ids im fall eines Shopsystemwechsels nicht mehr aktualisiert werden
    // Dadurch werden die Bestellungen im Shop den falschen Kunden zugeordnet was natürlich zu massiven Problemen führt
    // Daher erfolgt die prüfung ab version 1.5 Anhand der Kunden-Id und der E-Mail-Adresse. Nur wenn beides stimmt, wird der Kunde zugeordnet
    // ansonsten erfolgt die Bestellung als Gastbestellung
    $customerId = $order->getExternalCustomerId();
    $tmp = $GLOBALS["DB"]->executeQuery(
      "SELECT `kKunde` FROM `tkunde` WHERE `kKunde`='".$customerId."' AND `nRegistriert`=1",
      1
    );
    if (SG_DEBUG_LOGGING)
      $this->log("QRY: SELECT `kKunde` FROM `tkunde` WHERE `kKunde`='".$customerId."' AND `nRegistriert`=1", ShopgateLogger::LOGTYPE_DEBUG); // AGI debug_logging
    $dbClient = $tmp->kKunde;
    // Ende bis v 1.4 */
    /* BOF Version 1.5 */
    $cMail = $this->_prepare_string_for_jtl($order->getMail());
    $tmp = $GLOBALS["DB"]->executeQuery(
      "SELECT `kKunde` FROM `tkunde` WHERE `cMail`='".$GLOBALS['DB']->escape($cMail)."' AND `nRegistriert`=1",
      1
    );
    if (SG_DEBUG_LOGGING)
      $this->log("QRY: SELECT `kKunde` FROM `tkunde` WHERE `cMail`='".$GLOBALS['DB']->escape($cMail)."' AND `nRegistriert`=1", ShopgateLogger::LOGTYPE_DEBUG); // AGI debug_logging
    $dbClient = $tmp;
    /* EOF Version 1.5 */
    // If a user does not exist: create a guest account in the shop system
    // ...
    $invoice = $order->getInvoiceAddress();
    if (empty($dbClient))
    {
      $JTL_Client = new Kunde();
      if ($invoice->getGender() == 'f')
        $JTL_Client -> cAnrede  = 'w';
      elseif ($invoice->getGender() == 'm')
        $JTL_Client -> cAnrede  = 'm';
      else
        $JTL_Client -> cAnrede  = '';
      
      $JTL_Client -> cVorname = $this->_prepare_string_for_jtl($invoice->getFirstName());
      $JTL_Client -> cNachname= $this->_prepare_string_for_jtl($invoice->getLastName());
      $JTL_Client -> cFirma   = $this->_prepare_string_for_jtl($invoice->getCompany());	
      $JTL_Client -> cStrasse = $this->_prepare_string_for_jtl($invoice->getStreetName1());
      $JTL_Client -> cHausnummer  = $this->_prepare_string_for_jtl($invoice->getStreetNumber1());
      $JTL_Client -> cAdressZusatz= $this->_prepare_string_for_jtl($invoice->getStreet2()); // 1.6
      $JTL_Client -> cPLZ     = $this->_prepare_string_for_jtl($invoice->getZipcode());
      $JTL_Client -> cOrt     = $this->_prepare_string_for_jtl($invoice->getCity());
      //$JTL_Client -> cBundesland  = $this->_prepare_string_for_jtl($invoice->getState());
      $JTL_Client -> cLand    = $this->_prepare_string_for_jtl($invoice->getCountry());
      $JTL_Client -> cTel     = $this->_prepare_string_for_jtl($order->getPhone());
      $JTL_Client -> cMobil   = $this->_prepare_string_for_jtl($order->getMobile());
      //$JTL_Client -> cFax     = $invoice->();
      $JTL_Client -> cMail    = $this->_prepare_string_for_jtl($order->getMail());
      //$JTL_Client -> cUSTID   = $invoice->();
      //$JTL_Client -> cWWW     = $invoice->();
      $JTL_Client -> nRegistriert = 0;
      $JTL_Client -> kKundengruppe= $this->kKundengruppe;
      $JTL_Client -> kSprache   = $this->kSprache;
      $JTL_Client -> dErstellt = date('Y-m-d');
      $JTL_Client -> cAktiv    = 'Y';
      $JTL_Client -> cAbgeholt = 'N';
      $JTL_Client -> cHerkunft = 'Shopgate';
      if (SG_DEBUG_LOGGING)
        $this->log("Save new client", ShopgateLogger::LOGTYPE_DEBUG); // AGI debug_logging
      $JTL_Client -> insertInDB();
    }
    else
      $JTL_Client = new Kunde((int)$dbClient->kKunde);
      
    // create invoice address
    $JTL_Invoice = new Rechnungsadresse();
    $JTL_Invoice->kKunde = $JTL_Client->kKunde;
    if ($invoice->getGender() == 'f')
      $JTL_Invoice -> cAnrede  = 'w';
    elseif ($invoice->getGender() == 'm')
      $JTL_Invoice -> cAnrede  = 'm';
    else
      $JTL_Invoice -> cAnrede  = '';
    $JTL_Invoice->cVorname    = $this->_prepare_string_for_jtl($invoice->getFirstName());
    $JTL_Invoice->cNachname   = $this->_prepare_string_for_jtl($invoice->getLastName());
    $JTL_Invoice->cFirma      = $this->_prepare_string_for_jtl($invoice->getCompany());
    $JTL_Invoice->cStrasse    = $this->_prepare_string_for_jtl($invoice->getStreetName1());
    $JTL_Invoice->cHausnummer = $this->_prepare_string_for_jtl($invoice->getStreetNumber1());
    $JTL_Invoice->cAdressZusatz= $this->_prepare_string_for_jtl($invoice->getStreet2());
    $JTL_Invoice->cPLZ        = $this->_prepare_string_for_jtl($invoice->getZipcode());
    $JTL_Invoice->cOrt        = $this->_prepare_string_for_jtl($invoice->getCity());
    //$JTL_Invoice->cBundesland = $this->_prepare_string_for_jtl($invoice->getState());
    $JTL_Invoice->cLand       = $this->_prepare_string_for_jtl($invoice->getCountry());
    $JTL_Invoice->cTel        = $this->_prepare_string_for_jtl($invoice->getPhone());
    $JTL_Invoice->cMobil      = $this->_prepare_string_for_jtl($invoice->getMobile());
    $JTL_Invoice->cMail       = $this->_prepare_string_for_jtl($invoice->getMail());
    if (SG_DEBUG_LOGGING)
      $this->log("Save Invoice-Address", ShopgateLogger::LOGTYPE_DEBUG); // AGI debug_logging
    //$JTL_Invoice-> insertInDB(); // don't know, why it does not work
    
    $inv_address = new stdClass();
    $inv_address->kKunde    = $JTL_Client->kKunde;
    $inv_address->cAnrede   = $JTL_Invoice -> cAnrede;
    $inv_address->cVorname  = $JTL_Invoice -> cVorname;
    $inv_address->cNachname = verschluesselXTEA(trim($JTL_Invoice -> cNachname));
    $inv_address->cFirma    = verschluesselXTEA(trim($JTL_Invoice -> cFirma));
    $inv_address->cStrasse  = verschluesselXTEA(trim($JTL_Invoice -> cStrasse));
    $inv_address->cHausnummer = $JTL_Invoice -> cHausnummer;
    $inv_address->cAdressZusatz = $JTL_Invoice -> cAdressZusatz;
    $inv_address->cPLZ      = $JTL_Invoice -> cPLZ;
    $inv_address->cOrt      = $JTL_Invoice -> cOrt;
    $inv_address->cLand     = $JTL_Invoice -> cLand;
    $inv_address->cTel      = $JTL_Invoice -> cTel;
    $inv_address->cMobil    = $JTL_Invoice -> cMobil;
    $inv_address->cMail     = empty($JTL_Invoice -> cMail) ? $JTL_Client -> cMail : $JTL_Invoice -> cMail;
    $JTL_Invoice->kRechnungsadresse = $GLOBALS["DB"]->insertRow("trechnungsadresse", $inv_address);
    
    
    // check delivery-address
    $equal_address = true;
    $delivery = $order->getDeliveryAddress();
    $JTL_Delivery = new Lieferadresse();
    $JTL_Delivery->kKunde = $JTL_Client->kKunde;
    if ($delivery->getGender() == 'f')
      $JTL_Delivery -> cAnrede  = 'w';
    elseif ($delivery->getGender() == 'm')
      $JTL_Delivery -> cAnrede  = 'm';
    else
      $JTL_Delivery -> cAnrede  = '';
    $JTL_Delivery->cVorname    = $this->_prepare_string_for_jtl($delivery->getFirstName());
    $JTL_Delivery->cNachname   = $this->_prepare_string_for_jtl($delivery->getLastName());
    $JTL_Delivery->cFirma      = $this->_prepare_string_for_jtl($delivery->getCompany());
    $JTL_Delivery->cStrasse    = $this->_prepare_string_for_jtl($delivery->getStreetName1());
    $JTL_Delivery->cHausnummer = $this->_prepare_string_for_jtl($delivery->getStreetNumber1());
    $JTL_Delivery->cAdressZusatz = $this->_prepare_string_for_jtl($delivery->getStreet2());
    $JTL_Delivery->cPLZ        = $this->_prepare_string_for_jtl($delivery->getZipcode());
    $JTL_Delivery->cOrt        = $this->_prepare_string_for_jtl($delivery->getCity());
    //$JTL_Delivery->cBundesland = $this->_prepare_string_for_jtl($delivery->getState());
    $JTL_Delivery->cLand       = $this->_prepare_string_for_jtl($delivery->getCountry());
    $JTL_Delivery->cTel        = $this->_prepare_string_for_jtl($delivery->getPhone());
    $JTL_Delivery->cMobil      = $this->_prepare_string_for_jtl($delivery->getMobile());
    $JTL_Delivery->cMail       = $this->_prepare_string_for_jtl($delivery->getMail());
    if (
      $JTL_Delivery->cVorname != $JTL_Client->cVorname ||
      $JTL_Delivery->cNachname != $JTL_Client->cNachname ||
      $JTL_Delivery->cStrasse != $JTL_Client->cStrasse ||
      $JTL_Delivery->cPLZ != $JTL_Client->cPLZ ||
      $JTL_Delivery->cOrt != $JTL_Client->cOrt ||
      $JTL_Delivery->cMail != $JTL_Client->cMail
    )
    {
      $equal_address = false;
      if (SG_DEBUG_LOGGING)
        $this->log("Save delivery-Address", ShopgateLogger::LOGTYPE_DEBUG); // AGI debug_logging
      //$JTL_Delivery-> insertInDB(); same like saving invoice-Address
      $del_address = new stdClass();
      $del_address->kKunde    = $JTL_Client->kKunde;
      $del_address->cAnrede   = $JTL_Delivery -> cAnrede;
      $del_address->cVorname  = $JTL_Delivery -> cVorname;
      $del_address->cNachname = verschluesselXTEA(trim($JTL_Delivery -> cNachname));
      $del_address->cFirma    = verschluesselXTEA(trim($JTL_Delivery -> cFirma));
      $del_address->cStrasse  = verschluesselXTEA(trim($JTL_Delivery -> cStrasse));
      $del_address->cHausnummer = $JTL_Delivery -> cHausnummer;
      $del_address->cAdressZusatz = $JTL_Delivery -> cAdressZusatz;
      $del_address->cPLZ      = $JTL_Delivery -> cPLZ;
      $del_address->cOrt      = $JTL_Delivery -> cOrt;
      $del_address->cLand     = $JTL_Delivery -> cLand;
      $del_address->cTel      = $JTL_Delivery -> cTel;
      $del_address->cMobil    = $JTL_Delivery -> cMobil;
      $del_address->cMail     = $JTL_Delivery -> cMail;
      $JTL_Delivery->kLieferadresse = $GLOBALS["DB"]->insertRow("tlieferadresse", $del_address);
    }
    
    if (SG_DEBUG_LOGGING)
      $this->log("QRY: After delivery", ShopgateLogger::LOGTYPE_DEBUG); // AGI debug_logging
    // look for kWaehrung
    $tmp = $GLOBALS["DB"]->executeQuery(
      "SELECT `kWaehrung` FROM `twaehrung` WHERE `cISO`='".$order->getCurrency()."'",
      1
    );
    if (SG_DEBUG_LOGGING)
      $this->log("QRY: SELECT `kWaehrung` FROM `twaehrung` WHERE `cISO`='".$order->getCurrency()."'", ShopgateLogger::LOGTYPE_DEBUG); // AGI debug_logging
    $kWaehrung = $tmp->kWaehrung;
    if (empty($kWaehrung))
    {
      $tmp = $GLOBALS["DB"]->executeQuery(
        "SELECT `kWaehrung` FROM `twaehrung` WHERE `cStandard`='Y'",
        1
      );
      if (SG_DEBUG_LOGGING)
        $this->log("QRY: SELECT `kWaehrung` FROM `twaehrung` WHERE `cStandard`='Y'", ShopgateLogger::LOGTYPE_DEBUG); // AGI debug_logging
      $kWaehrung = $tmp->kWaehrung;
      $additional_comments[] = 'Währung '.$order->getCurrency().' nicht korrekt zugeordnet, bitte prüfen.';
    }
    
    $string_comments = '';
    if (!empty($additional_comments))
    {
      $string_comments = chr(10).implode(chr(10), $additional_comments);
    }
    
    // create Basket
    if (SG_DEBUG_LOGGING)
      $this->log("Start Basket", ShopgateLogger::LOGTYPE_DEBUG); // AGI debug_logging
    $basket = new stdClass();
    $basket->kKunde = $JTL_Client->kKunde;
    $basket->kLieferadresse = 0;
    $basket->kZahlungsInfo  = 0;
    if (SG_DEBUG_LOGGING)
      $this->log("Save Basket", ShopgateLogger::LOGTYPE_DEBUG); // AGI debug_logging
    $kWarenkorb = $GLOBALS["DB"]->insertRow("twarenkorb", $basket);
    
    // add items to baset
    $oItems = $order->getItems();
    $verfuegbarkeitResult_arr = array('cArtikelName_arr' => array());
    $aWarenkorbVersandklassen = array();
    foreach($oItems as $orderItem) 
    {
      if (SG_DEBUG_LOGGING)
        $this->log("get and save article-item ".$orderItem->getItemNumber(), ShopgateLogger::LOGTYPE_DEBUG); // AGI debug_logging
      // try to load article
      $oArtikelOptionen = new stdClass();
      $oArtikelOptionen->nMerkmale = 1;
      $oArtikelOptionen->nAttribute = 1;
      $oArtikelOptionen->nArtikelAttribute = 1;
      $oArtikelOptionen->nKategorie = 0;
      $oArtikelOptionen->nKeinLagerbestandBeachten = 1;//??
      $oArtikelOptionen->nVariationKombiKinder = 1;
      
      $article = new Artikel();
      $article->fuelleArtikel(
        $orderItem->getItemNumber(),
        $oArtikelOptionen,
        $JTL_Client -> kKundengruppe,
        $JTL_Client -> kSprache
      );
      
      $calcprice = round($orderItem->getUnitAmountWithTax()*100/(100+$orderItem->getTaxPercent()),5); /* 20140514*/
      $attributeSum = 0;
      $options = $orderItem->getOptions();
      if (!empty($options))
        $attributeSum = $this->_get_optionsPriceNetto($options, $orderItem->getTaxPercent());
      $calcpriceEinzel = $calcprice-$attributeSum;
      
      /* BOF 20160510 Konfiguration */
      $cUnique = '';
      $tmpOptions = array();
      $tmpKonfigurationArticle = array();
      if (!empty($options)) {
        foreach ($options as $checkOption)
        {
          $option_number = $checkOption->getOptionNumber();
          if (substr($option_number,0,10) == 'agiKonfig_')
            $tmpKonfigurationArticle[] = $checkOption;
          else
            $tmpOptions[] = $checkOption;
        }
      }
      $options = $tmpOptions;
      if ($this->bAllowKonfiguration) {
        if (!empty($tmpKonfigurationArticle)) {
          $cUnique = gibUID(10);
          $iCountKonfig = 0;
          foreach ($tmpKonfigurationArticle as $oKonfigurationOption) {
            $option_number = $oKonfigurationOption->getOptionNumber();
            $value_number = $oKonfigurationOption->getValueNumber();
            if ($value_number) {
              //array(kKonfiggruppe ,kKonfigitem, kArtikel);
              $aKonfigKeys = explode('_', $value_number);
              if (count($aKonfigKeys) == 3) {
                $kItem = new Konfigitem($aKonfigKeys[1]);
                if ($kItem->getKonfiggruppe() == $aKonfigKeys[0] && $kItem->getArtikelKey() == $aKonfigKeys[2]) {
                  $tmpArticle = $kItem->getArtikel();
                  $fKItemCalcprice = round($oKonfigurationOption->getAdditionalAmountWithTax()*100/(100+$tmpArticle->fMwSt),5);
                  $warenkorbpos = new stdClass();
                  $warenkorbpos->kWarenkorb      = $kWarenkorb;
                  $warenkorbpos->kArtikel        = $kItem->getArtikelKey();
                  $warenkorbpos->kVersandklasse  = $tmpArticle->kVersandklasse;
                  $warenkorbpos->cName           = $kItem->getName(); //$this->_prepare_string_for_jtl($oKonfigurationOption->getValue());
                  $warenkorbpos->cLieferstatus   = '';
                  $warenkorbpos->cArtNr          = $tmpArticle->cArtNr;
                  $warenkorbpos->cEinheit        = $tmpArticle->cEinheit;
                  $warenkorbpos->fPreisEinzelNetto = $fKItemCalcprice;
                  $warenkorbpos->fPreis          = $fKItemCalcprice;
                  $warenkorbpos->fMwSt           = $tmpArticle->fMwSt;
                  $warenkorbpos->nAnzahl         = $orderItem->getQuantity();
                  $warenkorbpos->nPosTyp         = C_WARENKORBPOS_TYP_ARTIKEL;
                  $warenkorbpos->cHinweis        = '';
                  $warenkorbpos->cUnique         = $cUnique;
                  $warenkorbpos->kKonfigitem     = $aKonfigKeys[1];
                  $warenkorbpos->kBestellpos     = 0;
                  if (SG_DEBUG_LOGGING)
                    $this->log("save basket-position", ShopgateLogger::LOGTYPE_DEBUG); // AGI debug_logging
                  $kWarenkorbPos = $GLOBALS["DB"]->insertRow("twarenkorbpos", $warenkorbpos);
                  
                  $iCountKonfig++;
                }
              }
            }
          }
          //if ($iCountKonfig == 0)
            //$cUnique = '';
        }
      }

      /* EOF 20160510 Konfiguration */
      $warenkorbpos = new stdClass();
      $warenkorbpos->kWarenkorb      = $kWarenkorb;
      $warenkorbpos->kArtikel        = $orderItem->getItemNumber();
      $warenkorbpos->kVersandklasse  = $article->kVersandklasse;
      $warenkorbpos->cName           = $this->_prepare_string_for_jtl($orderItem->getName());
      $warenkorbpos->cLieferstatus   = $article->cLieferstatus;
      $warenkorbpos->cArtNr          = $this->_prepare_string_for_jtl($orderItem->getItemNumberPublic());
      $warenkorbpos->cEinheit        = $article->cEinheit;
      $warenkorbpos->fPreisEinzelNetto = $calcpriceEinzel; // 20140514 //$orderItem->getUnitAmount();
      $warenkorbpos->fPreis          = $calcprice; // 20140514 //$orderItem->getUnitAmount();
      $warenkorbpos->fMwSt           = $orderItem->getTaxPercent();
      $warenkorbpos->nAnzahl         = $orderItem->getQuantity();
      $warenkorbpos->nPosTyp         = C_WARENKORBPOS_TYP_ARTIKEL;
      $warenkorbpos->cHinweis        = '';
      $warenkorbpos->cUnique         = $cUnique;
      $warenkorbpos->kKonfigitem     = 0;
      $warenkorbpos->kBestellpos     = 0;
      if (SG_DEBUG_LOGGING)
        $this->log("save basket-position", ShopgateLogger::LOGTYPE_DEBUG); // AGI debug_logging
      $kWarenkorbPos = $GLOBALS["DB"]->insertRow("twarenkorbpos", $warenkorbpos);
      
      if ($article->cLagerBeachten == "Y" && $article->cLagerKleinerNull == "Y" && $Einstellungen['global']['global_lieferverzoegerung_anzeigen'] === "Y")
      {
        if ($warenkorbpos->nAnzahl > $article->fLagerbestand)
          $verfuegbarkeitResult_arr['cArtikelName_arr'][] = $article->cName;
      }
      
      // Eigenschaften?
      $wpEigenschaftArr = array();
			if(!empty($options)) 
      {
        if (SG_DEBUG_LOGGING)
          $this->log("article has option", ShopgateLogger::LOGTYPE_DEBUG); // AGI debug_logging
        foreach($options as $option)
        {
          // search for attribute in Database
          $tmp = $GLOBALS["DB"]->executeQuery(
            "SELECT `kEigenschaft` FROM `teigenschaft` WHERE `kArtikel`='".$orderItem->getItemNumber()."' AND `kEigenschaft`='".$option->getOptionNumber()."'",
            1
          );
          if (SG_DEBUG_LOGGING)
            $this->log("QRY: SELECT `kEigenschaft` FROM `teigenschaft` WHERE `kArtikel`='".$orderItem->getItemNumber()."' AND `kEigenschaft`='".$option->getOptionNumber()."'", ShopgateLogger::LOGTYPE_DEBUG); // AGI debug_logging
          if (!empty($tmp->kEigenschaft))
          {
            $kEigenschaft = $tmp->kEigenschaft;
            $tmp2 = $GLOBALS["DB"]->executeQuery(
              "SELECT `kEigenschaftWert` FROM `teigenschaftwert` WHERE `kEigenschaft`='".$kEigenschaft."' AND `kEigenschaftWert`='".$option->getValueNumber()."'",
              1
            );
            if (SG_DEBUG_LOGGING)
              $this->log("QRY: SELECT `kEigenschaftWert` FROM `teigenschaftwert` WHERE `kEigenschaft`='".$kEigenschaft."' AND `kEigenschaftWert`='".$option->getValueNumber()."'", ShopgateLogger::LOGTYPE_DEBUG); // AGI debug_logging
            if (!empty($tmp2->kEigenschaftWert))
            {
              $kEigenschaftWert = $tmp2->kEigenschaftWert;
            }
            else
              $kEigenschaftWert = 0;
          }
          else
          {
            $kEigenschaft = 0;
            $kEigenschaftWert = 0;
          }
          
          $wpEigenschaft = new stdClass();
          $wpEigenschaft->kWarenkorbPos      = $kWarenkorbPos;
          $wpEigenschaft->kEigenschaft       = $kEigenschaft;
          $wpEigenschaft->kEigenschaftWert   = $kEigenschaftWert;
          $wpEigenschaft->cEigenschaftName   = $this->_prepare_string_for_jtl($option->getName());
          $wpEigenschaft->cEigenschaftWertName = $this->_prepare_string_for_jtl($option->getValue());
          $wpEigenschaft->cFreifeldWert      = $this->_prepare_string_for_jtl($option->getValue());
          $wpEigenschaft->fAufpreis          = round($option->getAdditionalAmountWithTax()*100/(100+$orderItem->getTaxPercent()), 4);
          if (SG_DEBUG_LOGGING)
            $this->log("save basket-position-option", ShopgateLogger::LOGTYPE_DEBUG); // AGI debug_logging
          $kWarenkorbPosEigenschaft = $GLOBALS["DB"]->insertRow("twarenkorbposeigenschaft", $wpEigenschaft);
          $wpEigenschaftArr[] = $wpEigenschaft;
        }
      }
      
      $inputs = $orderItem->getInputs();
      if (!empty($inputs))
      {
        if (SG_DEBUG_LOGGING)
          $this->log("article has inputOptions", ShopgateLogger::LOGTYPE_DEBUG); // AGI debug_logging
        foreach($inputs as $inputOption)
        {
          // search for attribute in Database
          $tmp = $GLOBALS["DB"]->executeQuery(
            "SELECT `kEigenschaft` FROM `teigenschaft` WHERE `kArtikel`='".$orderItem->getItemNumber()."' AND `kEigenschaft`='".$inputOption->getInputNumber()."'",
            1
          );
          if (SG_DEBUG_LOGGING)
            $this->log("QRY: SELECT `kEigenschaft` FROM `teigenschaft` WHERE `kArtikel`='".$orderItem->getItemNumber()."' AND `kEigenschaft`='".$inputOption->getInputNumber()."'", ShopgateLogger::LOGTYPE_DEBUG); // AGI debug_logging
          if (!empty($tmp->kEigenschaft))
          {
            $kEigenschaft = $tmp->kEigenschaft;
            $kEigenschaftWert = 0;
          }
          else
          {
            $kEigenschaft = 0;
            $kEigenschaftWert = 0;
          }
          
          $wpEigenschaft = new stdClass();
          $wpEigenschaft->kWarenkorbPos      = $kWarenkorbPos;
          $wpEigenschaft->kEigenschaft       = $kEigenschaft;
          $wpEigenschaft->kEigenschaftWert   = $kEigenschaftWert;
          $wpEigenschaft->cEigenschaftName   = $this->_prepare_string_for_jtl($inputOption->getLabel());
          $wpEigenschaft->cEigenschaftWertName = $this->_prepare_string_for_jtl($inputOption->getUserInput());
          $wpEigenschaft->cFreifeldWert      = $this->_prepare_string_for_jtl($inputOption->getUserInput());
          $wpEigenschaft->fAufpreis          = round($inputOption->getAdditionalAmountWithTax()*100/(100+$orderItem->getTaxPercent()), 4);
          if (SG_DEBUG_LOGGING)
            $this->log("save basket-position-inputOption", ShopgateLogger::LOGTYPE_DEBUG); // AGI debug_logging
          $kWarenkorbPosEigenschaft = $GLOBALS["DB"]->insertRow("twarenkorbposeigenschaft", $wpEigenschaft);
          $wpEigenschaftArr[] = $wpEigenschaft;
        }
      }
    
      aktualisiereLagerbestand($article, $warenkorbpos->nAnzahl, $wpEigenschaftArr);
      aktualisiereBestseller($warenkorbpos->kArtikel, $warenkorbpos->nAnzahl);
      if (!in_array($article->kVersandklasse, $aWarenkorbVersandklassen))
        $aWarenkorbVersandklassen[] = $article->kVersandklasse;
    }
    if (count($verfuegbarkeitResult_arr['cArtikelName_arr']) > 0) 
    {
      $cHinweis = str_replace("%s", "", $GLOBALS['oSprache']->gibWert('orderExpandInventory', 'basket'));
      $verfuegbarkeitResult_arr['cHinweis'] = $cHinweis;
    }
    
    $tmp = $GLOBALS["DB"]->executeQuery(
      "SELECT `kSteuerzone` FROM `tsteuerzoneland` WHERE `cISO`='".$this->_prepare_string_for_jtl($JTL_Delivery->cLand)."'",
      1
    );
    if (SG_DEBUG_LOGGING)
      $this->log("QRY: SELECT `kSteuerzone` FROM `tsteuerzoneland` WHERE `cISO`='".$this->_prepare_string_for_jtl($JTL_Delivery->cLand)."'", ShopgateLogger::LOGTYPE_DEBUG); // AGI debug_logging
    if (!empty($tmp))
    {
      $kSteuerzone = $tmp->kSteuerzone;
      $tmp = $GLOBALS["DB"]->executeQuery(
        "SELECT `kSteuerklasse` FROM `tsteuerklasse` WHERE `cStandard`='Y'",
        1
      );
      if (SG_DEBUG_LOGGING)
        $this->log("QRY: SELECT `kSteuerklasse` FROM `tsteuerklasse` WHERE `cStandard`='Y'", ShopgateLogger::LOGTYPE_DEBUG); // AGI debug_logging
      $kSteuerklasse = $tmp->kSteuerklasse;
      $tmp = $GLOBALS["DB"]->executeQuery(
        "SELECT `fSteuersatz` FROM `tsteuersatz` WHERE `kSteuerklasse`=$kSteuerklasse AND `kSteuerzone`=$kSteuerzone",
        1
      );
      if (SG_DEBUG_LOGGING)
        $this->log("QRY: SELECT `fSteuersatz` FROM `tsteuersatz` WHERE `kSteuerklasse`=$kSteuerklasse AND `kSteuerzone`=$kSteuerzone", ShopgateLogger::LOGTYPE_DEBUG); // AGI debug_logging
      $fSteuersatz = $tmp->fSteuersatz;
    }
    else
      $fSteuersatz = $this->fallback_steuersatz;
    
    $aKupons = $order->getExternalCoupons();
    $addKuponBestellung = array();
    if (!empty($aKupons))
    {
      foreach ($aKupons as $oKupon)
      {
        $cInfo = $oKupon->getInternalInfo();
        if (!empty($cInfo))
        {
          $aInfo = $this->jsonDecode($cInfo, true);
          $kKupon = array_key_exists('kKupon', $aInfo) ? $aInfo['kKupon'] : 0;
          if (!empty($kKupon))
          {
            $fAmount = $oKupon->getAmount();
            $fAmountNet = round($fAmount*100/(100+$fSteuersatz),4);
            $warenkorbpos = new stdClass();
            $warenkorbpos->kWarenkorb      = $kWarenkorb;
            $warenkorbpos->kArtikel        = 0;
            $warenkorbpos->kVersandklasse  = 0;
            $warenkorbpos->cName           = $this->_prepare_string_for_jtl($oKupon->getName());
            $warenkorbpos->cLieferstatus   = '';
            $warenkorbpos->cArtNr          = '';
            $warenkorbpos->cEinheit        = '';
            $warenkorbpos->fPreisEinzelNetto = $fAmountNet*-1;
            $warenkorbpos->fPreis          = $fAmountNet*-1;
            $warenkorbpos->fMwSt           = $fSteuersatz;
            $warenkorbpos->nAnzahl         = 1;
            $warenkorbpos->nPosTyp         = C_WARENKORBPOS_TYP_KUPON;
            $warenkorbpos->cHinweis        = '';
            $warenkorbpos->cUnique         = '';
            $warenkorbpos->kKonfigitem     = 0;
            $warenkorbpos->kBestellpos     = 0;
            if (SG_DEBUG_LOGGING)
              $this->log("add basket-external-coupon-position", ShopgateLogger::LOGTYPE_DEBUG); // AGI debug_logging
            $kWarenkorbPos = $GLOBALS["DB"]->insertRow("twarenkorbpos", $warenkorbpos);
            
            $GLOBALS["DB"]->executeQuery("UPDATE `tkupon` SET `nVerwendungenBisher`=`nVerwendungenBisher`+1 WHERE `kKupon`=".$kKupon,4);
            
            $KuponKunde = new stdClass();
            $KuponKunde->kKupon=$kKupon;
            $KuponKunde->kKunde=$JTL_Client->kKunde;
            $KuponKunde->dErstellt="now()";
            $KuponKunde->nVerwendungen=1;

            $KuponKundeBisher = $GLOBALS["DB"]->executeQuery("SELECT `nVerwendungen` FROM `tkuponkunde` WHERE `kKupon`=$kKupon AND `kKunde`=".$JTL_Client->kKunde,1);
            if ($KuponKundeBisher)
            {
              if ($KuponKundeBisher->nVerwendungen>0)
                $KuponKunde->nVerwendungen += $KuponKundeBisher->nVerwendungen;
            }
            $GLOBALS["DB"]->executeQuery("DELETE FROM `tkuponkunde` WHERE `kKupon`=$kKupon AND `kKunde`=".$JTL_Client->kKunde,4);
            $GLOBALS["DB"]->insertRow('tkuponkunde',$KuponKunde);
            $addKuponBestellung[] = $kKupon;
          }
          elseif (array_key_exists('kundenguthaben', $aInfo) && $aInfo['kundenguthaben'] == $JTL_Client->kKunde)
          {
            $fAmount = $oKupon->getAmount();
            $fAmountNet = round($fAmount*100/(100+$fSteuersatz),4);
            $warenkorbpos = new stdClass();
            $warenkorbpos->kWarenkorb      = $kWarenkorb;
            $warenkorbpos->kArtikel        = 0;
            $warenkorbpos->kVersandklasse  = 0;
            $warenkorbpos->cName           = $this->_prepare_string_for_jtl($oKupon->getName());
            $warenkorbpos->cLieferstatus   = '';
            $warenkorbpos->cArtNr          = '';
            $warenkorbpos->cEinheit        = '';
            $warenkorbpos->fPreisEinzelNetto = $fAmountNet*-1;
            $warenkorbpos->fPreis          = $fAmountNet*-1;
            $warenkorbpos->fMwSt           = $fSteuersatz;
            $warenkorbpos->nAnzahl         = 1;
            $warenkorbpos->nPosTyp         = C_WARENKORBPOS_TYP_ARTIKEL;
            $warenkorbpos->cHinweis        = '';
            $warenkorbpos->cUnique         = '';
            $warenkorbpos->kKonfigitem     = 0;
            $warenkorbpos->kBestellpos     = 0;
            if (SG_DEBUG_LOGGING)
              $this->log("add basket-external-coupon-position", ShopgateLogger::LOGTYPE_DEBUG); // AGI debug_logging
            $kWarenkorbPos = $GLOBALS["DB"]->insertRow("twarenkorbpos", $warenkorbpos);
            
            // lese Kundenguthaben und speichere neues Guthaben
            $fAmount *= 1;
            $GLOBALS["DB"]->executeQuery("UPDATE `tkunde` SET `fGuthaben`=`fGuthaben`-$fAmount WHERE `kKunde`=".$JTL_Client->kKunde,4);
          }
        }
      }
    }
    
    // add shipping
    // load default shipping_name from export-plugin
    $cInfo = $order->getShippingInfos()->getInternalShippingInfo();
    $bShippingInfo = false;
    if (!empty($cInfo))
    {
      $aInfo = $this->jsonDecode($cInfo, true);
      if (array_key_exists('kShipping', $aInfo) && array_key_exists('cShipping', $aInfo))
      {
        if (!empty($aInfo['kShipping']))
        {
          $nShippingKey = $aInfo['kShipping'];
          $cShippingName = $this->_prepare_string_for_jtl($aInfo['cShipping']);
          $bShippingInfo = true;
        }
      }
    }
    if (!$bShippingInfo)
    {
      $cShippingName = $this->_prepare_string_for_jtl($order->getShippingInfos()->getName());
      $nShippingKey = $this->getKVersandartByName($cShippingName, $aWarenkorbVersandklassen, $JTL_Client->kKundengruppe, $JTL_Delivery->cLand);
    }
    
    $shippingAmountWithoutTax = $order->getAmountShipping();
    $fSteuersatz = $order->getShippingTaxPercent();
    if(!empty($fSteuersatz)) {
      // remove tax from shipping costs
      $shippingAmountWithoutTax /= 1+$fSteuersatz/100;
    }
    $warenkorbpos = new stdClass();
    $warenkorbpos->kWarenkorb      = $kWarenkorb;
    $warenkorbpos->kArtikel        = 0;
    $warenkorbpos->kVersandklasse  = 0;
    $warenkorbpos->cName           = $cShippingName; //'Versand'; // THERE IS NO SHOPGATE SOLUTION
    $warenkorbpos->cLieferstatus   = '';
    $warenkorbpos->cArtNr          = '';
    $warenkorbpos->cEinheit        = '';
    $warenkorbpos->fPreisEinzelNetto = $shippingAmountWithoutTax;
    $warenkorbpos->fPreis          = $shippingAmountWithoutTax;
    $warenkorbpos->fMwSt           = $fSteuersatz;
    $warenkorbpos->nAnzahl         = 1;
    $warenkorbpos->nPosTyp         = C_WARENKORBPOS_TYP_VERSANDPOS;
    $warenkorbpos->cHinweis        = '';
    $warenkorbpos->cUnique         = '';
    $warenkorbpos->kKonfigitem     = 0;
    $warenkorbpos->kBestellpos     = 0;
    if (SG_DEBUG_LOGGING)
      $this->log("add basket-shipping-position", ShopgateLogger::LOGTYPE_DEBUG); // AGI debug_logging
    $kWarenkorbPos = $GLOBALS["DB"]->insertRow("twarenkorbpos", $warenkorbpos);
    
    // get payment-infos
    $payment_result = $this->plugin_handle_payment($order);
    $paymentComment = $payment_result['paymentComment'];
    $kZahlart       = $payment_result['kZahlart'];
    $cZahlart       = $payment_result['cZahlart'];
    $cGebuehr       = $payment_result['cGebuehr'];
    if (array_key_exists('tZahlungseingang', $payment_result))
      $tZahlungseingang = $payment_result['tZahlungseingang'];
    
    if (!empty($paymentComment))
    {
      $paymentComment = chr(10).'Payment'.chr(10).$paymentComment;
    }
    
    // add payment
    if($order->getAmountShopPayment() != 0)
    {
      if (SG_DEBUG_LOGGING)
        $this->log("prepare payment-infos", ShopgateLogger::LOGTYPE_DEBUG); // AGI debug_logging
			$paymentInfos = $order->getPaymentInfos();
      $paymentAmountWithoutTax = $order->getAmountShopPayment();
			$fSteuersatz = $order->getPaymentTaxPercent();
      if(!empty($fSteuersatz)) {
        // remove tax from shipping costs
        $paymentAmountWithoutTax /= 1+$fSteuersatz/100;
      }
      
			$warenkorbpos = new stdClass();
      $warenkorbpos->kWarenkorb      = $kWarenkorb;
      $warenkorbpos->kArtikel        = 0;
      $warenkorbpos->kVersandklasse  = 0;
      $warenkorbpos->cName           = $cGebuehr;
      $warenkorbpos->cLieferstatus   = '';
      $warenkorbpos->cArtNr          = '';
      $warenkorbpos->cEinheit        = '';
      $warenkorbpos->fPreisEinzelNetto = $paymentAmountWithoutTax;
      $warenkorbpos->fPreis          = $paymentAmountWithoutTax;
      $warenkorbpos->fMwSt           = $fSteuersatz;
      $warenkorbpos->nAnzahl         = 1;
      $warenkorbpos->nPosTyp         = C_WARENKORBPOS_TYP_ZAHLUNGSART;
      $warenkorbpos->cHinweis        = '';
      $warenkorbpos->cUnique         = '';
      $warenkorbpos->kKonfigitem     = 0;
      $warenkorbpos->kBestellpos     = 0;
      $kWarenkorbPos = $GLOBALS["DB"]->insertRow("twarenkorbpos", $warenkorbpos);
      if (SG_DEBUG_LOGGING)
        $this->log("add payment-position", ShopgateLogger::LOGTYPE_DEBUG); // AGI debug_logging
		}
    
    if (SG_DEBUG_LOGGING)
      $this->log("start order-object", ShopgateLogger::LOGTYPE_DEBUG); // AGI debug_logging
    
    /* BOF Build order-number */
    $tmp_order_number = '';
    if (array_key_exists('use_sg_number', $this->jtl_plugin_configuration))
    {
      if ($this->jtl_plugin_configuration['use_sg_number'] == 'yes')
        $tmp_order_number = $order->getOrderNumber();
      else
        $tmp_order_number = baueBestellnummer();
      $tmp_order_number = $this->jtl_plugin_configuration['number_prefix'].$tmp_order_number.$this->jtl_plugin_configuration['number_suffix'];
    }
    else
      $tmp_order_number = baueBestellnummer().'SG';
    /* EOF Build order-number */
    
    // create Order-object
    $JTL_Order = new Bestellung();
    $JTL_Order->kRechnungsadresse = $JTL_Invoice->kRechnungsadresse;
    $JTL_Order->kLieferadresse    = $equal_address ? 0 : $JTL_Delivery->kLieferadresse;
    $JTL_Order->kKunde            = $JTL_Client->kKunde;
    $JTL_Order->kZahlungsart      = $kZahlart;
    $JTL_Order->kVersandart       = $nShippingKey; // hope it works
    $JTL_Order->kWaehrung         = $kWaehrung;
    $JTL_Order->kSprache          = $JTL_Client -> kSprache;
    $JTL_Order->fGesamtsumme      = $order->getAmountComplete();
    $JTL_Order->cBestellNr        = $tmp_order_number;
    $JTL_Order->cKommentar        = 'shopgate '.$order->getOrderNumber().$paymentComment; //.$string_comments.$paymentComment;
    $JTL_Order->cVersandartName   = $cShippingName;
    $JTL_Order->cZahlungsartName  = $cZahlart;
    $JTL_Order->dErstellt         = date('Y-m-d H:i:s');
    $JTL_Order->cStatus           = $order->getIsPaid() ? BESTELLUNG_STATUS_BEZAHLT : BESTELLUNG_STATUS_OFFEN;
    if ($order->getIsPaid())
      $JTL_Order->dBezahltDatum = $order->getPaymentTime('Y-m-d');
    $JTL_Order->kWarenkorb        = $kWarenkorb;
    $JTL_Order->cAbgeholt         = 'N';
    /*
    if ($order->getIsShippingBlocked() == 1)
    {
      $JTL_Order->cKommentar .= chr(10).chr(10).'SHOPGATE : Versand blockiert!';
    }
    */
    $custom_fields_array = array();
    foreach ($order->getCustomFields() as $row) {
      $custom_fields_array[] = $this->_prepare_string_for_jtl($row->getLabel().' ['.$row->getInternalFieldName().']: '.$row->getValue());
    }
    if (!empty($custom_fields_array)) {
      $JTL_Order->cKommentar .= chr(10).chr(10).implode(chr(10), $custom_fields_array);
    }
    if (SG_DEBUG_LOGGING)
      $this->log("save order in JTL", ShopgateLogger::LOGTYPE_DEBUG); // AGI debug_logging
    $kBestellung = $JTL_Order->insertInDB();
    $GLOBALS["DB"]->executeQuery("UPDATE `tkunde` SET `cAbgeholt`='N' WHERE `kKunde`=".$JTL_Client->kKunde,4);
    
    $bestellid  = new stdClass();
    $bestellid->cId = gibUID(40, $kBestellung.md5(time()));
    $bestellid->kBestellung = $kBestellung;
    $bestellid->dDatum = "now()";
    $GLOBALS["DB"]->insertRow('tbestellid',$bestellid);
    
    $bestellstatus  = new stdClass();
    $bestellstatus->kBestellung = $kBestellung;
    $bestellstatus->dDatum ="now()";
    $bestellstatus->cUID = gibUID(40, (time().$kBestellung).substr(time(),-8));
    $GLOBALS["DB"]->insertRow('tbestellstatus',$bestellstatus);
    
    foreach ($addKuponBestellung as $row)
    {
      $KuponBestellung = new stdClass();
      $KuponBestellung->kKupon = $row;
      $KuponBestellung->kBestellung = $kBestellung;
      $GLOBALS['DB']->insertRow('tkuponbestellung', $KuponBestellung);
    }
    
    // safe in seperate table
    $ordersShopgateOrder  = new stdClass();
    $ordersShopgateOrder->kBestellung  = $kBestellung;
    $ordersShopgateOrder->kBestellungShopgate = $order->getOrderNumber();
    $ordersShopgateOrder->created      = date('Y-m-d H:i:s');
    $ordersShopgateOrder->modified     = date('Y-m-d H:i:s');
    $ordersShopgateOrder->sgShopNumber = $this->request_shop_number;
    if (SG_DEBUG_LOGGING)
      $this->log("save order in plugin-db", ShopgateLogger::LOGTYPE_DEBUG); // AGI debug_logging
    $GLOBALS["DB"]->insertRow("xplugin_agi_shopgate_plugin_connect", $ordersShopgateOrder);
    
    if (isset($tZahlungseingang))
    {
      $tZahlungseingang->kBestellung  = $kBestellung;
      if (SG_DEBUG_LOGGING)
        $this->log("save payment-information in payment-db", ShopgateLogger::LOGTYPE_DEBUG); // AGI debug_logging
      $GLOBALS["DB"]->insertRow("tzahlungseingang", $tZahlungseingang);
    }

    // Bestellmail versenden
    if (array_key_exists('cSendOrderConfirmation',$this->jtl_plugin_configuration) && $this->jtl_plugin_configuration['cSendOrderConfirmation'] == 'Y')
    {
      require_once DIR_BASE_PATH.'/includes/mailTools.php';
      $obj = new stdClass();
      $obj->cVerfuegbarkeit_arr = $verfuegbarkeitResult_arr;
      $bestellung = new Bestellung($kBestellung);
      $bestellung->fuelleBestellung(0);
      $obj->tkunde = $JTL_Client;
      $obj->tbestellung = $bestellung;
      sendeMail(MAILTEMPLATE_BESTELLBESTAETIGUNG, $obj);
    }
    
    // Return the order ID and number of the order saved to the database
    return array(
      'external_order_id' => $kBestellung,        // this is used to inform a developer in case there was a problem
      'external_order_number' => $JTL_Order->cBestellNr // this is used to inform a merchant in case there was a problem
    );
  }
  protected function _get_optionsPriceNetto($options, $tax = 19)
  {
    $sumNet = 0;
    if (!empty($options))
    {
      foreach($options as $option)
        $sumNet += round($option->getAdditionalAmountWithTax()*100/(100+$tax), 4);
    }
    return $sumNet;
  }
  public function updateOrder(ShopgateOrder $order)
  {
    $this->agiUpdateOrderIndividual($order);
    if (SG_DEBUG_LOGGING)
      $this->log("start updateOrder", ShopgateLogger::LOGTYPE_DEBUG); // AGI debug_logging
    // The database connection can be established here
    // Check if the order in question has already been imported into the system
    // with the Shopgate order number ($order->getOrderNumber()).
    // the order cannot be found in the system:
    $tmp = $GLOBALS["DB"]->executeQuery(
      "SELECT `kBestellung`, `history` FROM `xplugin_agi_shopgate_plugin_connect` WHERE `kBestellungShopgate`='".$order->getOrderNumber()."'",
      1
    );
    if (SG_DEBUG_LOGGING)
      $this->log("QRY: SELECT `kBestellung`, `history` FROM `xplugin_agi_shopgate_plugin_connect` WHERE `kBestellungShopgate`='".$order->getOrderNumber()."'", ShopgateLogger::LOGTYPE_DEBUG); // AGI debug_logging
    $dbOrderID = $tmp->kBestellung;
    $dbOrderHistory = $tmp->history;
    
    if (empty($dbOrderID)) {
      throw new ShopgateLibraryException(ShopgateLibraryException::PLUGIN_ORDER_NOT_FOUND, "Shopgate order number: '{$order->getOrderNumber()}'.");
    }
    
    // load some order-information
    $dbOrder = $GLOBALS["DB"]->executeQuery(
      "SELECT `cBestellNr`, `dBezahltDatum`, `cKommentar` FROM `tbestellung` WHERE `kBestellung`='".$dbOrderID."'",
      1
    );
    
    if (SG_DEBUG_LOGGING)
      $this->log("QRY: SELECT `cBestellNr`, `dBezahltDatum`, `cKommentar` FROM `tbestellung` WHERE `kBestellung`='".$dbOrderID."'", ShopgateLogger::LOGTYPE_DEBUG); // AGI debug_logging
    $something_changed = false;
    $history_comments = array();
    $updateOrder  = new stdClass();
    if ($order->getUpdatePayment() == 1) 
    {
      if (SG_DEBUG_LOGGING)
        $this->log("payment changed", ShopgateLogger::LOGTYPE_DEBUG); // AGI debug_logging
      // Payment status or PaymentInfos has changed
      if ($order->getIsPaid() && $dbOrder->dBezahltDatum == '0000-00-00')
      {
        $updateOrder->dBezahltDatum = $order->getPaymentTime('Y-m-d');
        $updateOrder->cStatus       = BESTELLUNG_STATUS_BEZAHLT;
        //$updateOrder->cKommentar    = empty($dbOrder->cKommentar) ? '' : $dbOrder->cKommentar.chr(10);
        $history_comments[] = 'Bestellstatus von Shopgate geändert: Zahlung erhalten';
        
        // get payment-infos
        $payment_result = $this->plugin_handle_payment($order);
        $paymentComment = $payment_result['paymentComment'];
        $kZahlart       = $payment_result['kZahlart'];
        $cZahlart       = $payment_result['cZahlart'];
        if (array_key_exists('tZahlungseingang', $payment_result))
          $tZahlungseingang = $payment_result['tZahlungseingang'];
        
        if (!empty($paymentComment))
        {
          $paymentComment = chr(10).'Payment '.date('Y-m-d').chr(10).$paymentComment;
          if (!isset($updateOrder->cKommentar))
            $updateOrder->cKommentar = '';
          $updateOrder->cKommentar .= $paymentComment;
        }
        // add payment information to shop
        if (isset($tZahlungseingang))
        {
          $tZahlungseingang->kBestellung  = $dbOrderID;
          if (SG_DEBUG_LOGGING)
            $this->log("save payment-information in payment-db for Paypal", ShopgateLogger::LOGTYPE_DEBUG); // AGI debug_logging
          $GLOBALS["DB"]->insertRow("tzahlungseingang", $tZahlungseingang);
        }
        else
        {
          $objBestellung = new Bestellung($dbOrderID);
          $fBetrag       = $objBestellung->fGesamtsummeKundenwaehrung;
          $objISO = $GLOBALS["DB"]->executeQuery("SELECT cISO FROM twaehrung WHERE kWaehrung = '".$objBestellung->kWaehrung."'", 1);
          $cISO = $objISO->cISO;
          // safe in seperate table
          $ordersPayment  = new stdClass();
          $ordersPayment->kBestellung       = $dbOrderID;
          $ordersPayment->cZahlungsanbieter = $cZahlart;
          $ordersPayment->fBetrag           = $fBetrag;
          $ordersPayment->cISO              = $cISO;
          $ordersPayment->dZeit             = $order->getPaymentTime('Y-m-d H:i:s');
          $ordersPayment->cAbgeholt         = 'N';
          if (SG_DEBUG_LOGGING)
            $this->log("save payment-information in payment-db", ShopgateLogger::LOGTYPE_DEBUG); // AGI debug_logging
          $GLOBALS["DB"]->insertRow("tzahlungseingang", $ordersPayment);
          $something_changed = true;
        }
      }
      elseif (!$order->getIsPaid() && $dbOrder->dBezahltDatum != '0000-00-00')
      {
        $updateOrder->dBezahltDatum = '0000-00-00';
        $updateOrder->cStatus       = BESTELLUNG_STATUS_OFFEN;
        $history_comments[] = 'Bestellstatus von Shopgate geändert: Zahlung noch nicht erhalten';
        $something_changed = true;
      }
    }

    if ($order->getUpdateShipping() == 1) 
    {
      
      if (SG_DEBUG_LOGGING)
        $this->log("shipping changed", ShopgateLogger::LOGTYPE_DEBUG); // AGI debug_logging
      // Released or blocked for shipping
      if ($order->getIsShippingBlocked() == 1)
      {
        $history_comments[] = 'Versand blockiert!';
        //if (!isset($updateOrder->cKommentar))
          //$updateOrder->cKommentar = '';
        //$updateOrder->cKommentar .= chr(10).date('Y-m-d').' Versand blockiert!';
        $something_changed = true;
      }
      else
      {
        $history_comments[] = 'Versand freigegeben';
        if (!isset($updateOrder->cKommentar))
          $updateOrder->cKommentar = '';
        $updateOrder->cKommentar = str_replace(chr(10).chr(10).'SHOPGATE : Versand blockiert!','', $dbOrder->cKommentar);
        $something_changed = true;
      }
    }
    
    if ($something_changed)
    {
      if (SG_DEBUG_LOGGING)
        $this->log("something changed and must be saved", ShopgateLogger::LOGTYPE_DEBUG); // AGI debug_logging
      $GLOBALS["DB"]->updateRow("tbestellung", "kbestellung", $dbOrderID, $updateOrder);
      $aUpdate = array(
        'modified'  => date('Y-m-d H:i:s')
      );
      if (!empty($history_comments))
        $aUpdate['history'] = $dbOrderHistory.chr(10).chr(10).'Update '.date('Y-m-d H:i:s').chr(10).implode(chr(10), $history_comments);
        
      if (!empty($aUpdate))
      {
        $uQuery = "UPDATE `xplugin_agi_shopgate_plugin_connect` SET ";
        $i = 0;
        foreach ($aUpdate as $key => $value)
        {
          if($i > 0)
            $uQuery .= ", ";
          $uQuery .= "`$key`='$value'";
          $i++;
        }
        $uQuery .= " WHERE `kbestellung`=$dbOrderID";
        if (!empty($this->request_shop_number))
          $uQuery .= " AND `sgShopNumber`=".$this->request_shop_number;
        if (SG_DEBUG_LOGGING)
          $this->log($uQuery, ShopgateLogger::LOGTYPE_DEBUG); // AGI debug_logging
        $GLOBALS["DB"]->executeQuery($uQuery, 3);
      }
    }
    
    // Return the order ID and number of the order saved to the database
    return array(
      'external_order_id' => $dbOrderID,        // this is used to inform a developer in case there was a problem
      'external_order_number' => $dbOrder->cBestellNr // this is used to inform a merchant in case there was a problem
    );
  }
  /// EOF ORDER-FUNCTIONS ///
  
  
  /// BOF CRONJOB-FUNCTIONS ///
  public function cron($jobname, $params, &$message, &$errorcount) 
  {
    switch($jobname) {
			case "set_shipping_completed":
				$aProcessLog = $this->cronSetShippingCompleted($message, $errorcount);
				break;
      case "cancel_orders":
				$aProcessLog = $this->cronGetCancelOrder($message, $errorcount);
				break;
			default:
				throw new ShopgateLibraryException(ShopgateLibraryException::PLUGIN_CRON_UNSUPPORTED_JOB, "{$jobname}", true);
		}
  }
  
  protected function cronSetShippingCompleted(&$message, &$errorcount)
  {
    $cQuery = "SELECT 
        xp.kBestellung, xp.kBestellungShopgate, tb.dVersandDatum
      FROM
        `xplugin_agi_shopgate_plugin_connect` AS xp, `tbestellung` AS tb
      WHERE
        xp.kBestellung=tb.kBestellung AND xp.sgVersandstatusGesendet=0 AND tb.dVersandDatum>'0000-00-00'
    ";
    if (!empty($this->request_shop_number))
      $cQuery .= " AND `sgShopNumber`=".$this->request_shop_number;
    if (SG_DEBUG_LOGGING)
      $this->log("QRY: ".$cQuery, ShopgateLogger::LOGTYPE_DEBUG); // AGI debug_logging
    
    $builder = new ShopgateBuilder($this->config);
    $api = $builder->buildMerchantApi();
    
    $aRes = $GLOBALS["DB"]->executeQuery($cQuery, 9);
    foreach ($aRes as $row)
    {
      try 
      {
        $resp = $api->setOrderShippingCompleted($row['kBestellungShopgate']);
        if ($resp->error == 0)
        {
          $aUpdate = array(
            'sgVersandstatusGesendet'  => 1
          );
          if (!empty($aUpdate))
          {
            $uQuery = "UPDATE `xplugin_agi_shopgate_plugin_connect` SET ";
            $i = 0;
            foreach ($aUpdate as $key => $value)
            {
              if($i > 0)
                $uQuery .= ", ";
              $uQuery .= "`$key`='".(is_string($value) ? mysql_real_escape_string($value) : $value)."'";
              $i++;
            }
            $uQuery .= " WHERE `kBestellung`=".$row['kBestellung'];
            if (!empty($this->request_shop_number))
              $uQuery .= " AND `sgShopNumber`=".$this->request_shop_number;
            if (SG_DEBUG_LOGGING)
              $this->log($uQuery, ShopgateLogger::LOGTYPE_DEBUG); // AGI debug_logging
            $GLOBALS["DB"]->executeQuery($uQuery, 3);
          }
        }
        else
        {
          $this->log('cronSetShippingCompleted(): Code '.$resp->error.': '.$resp->error_text, ShopgateLogger::LOGTYPE_DEBUG);
        }
      }
      catch (Exception $e) 
      {
        $eCode = $e->getCode();
        if ($eCode == 204 || $eCode == 201)
        {
          $aUpdate = array(
            'sgVersandstatusGesendet'  => 1
          );
          if (!empty($aUpdate))
          {
            $uQuery = "UPDATE `xplugin_agi_shopgate_plugin_connect` SET ";
            $i = 0;
            foreach ($aUpdate as $key => $value)
            {
              if($i > 0)
                $uQuery .= ", ";
              $uQuery .= "`$key`='".(is_string($value) ? mysql_real_escape_string($value) : $value)."'";
              $i++;
            }
            $uQuery .= " WHERE `kBestellung`=".$row['kBestellung'];
            if (!empty($this->request_shop_number))
              $uQuery .= " AND `sgShopNumber`=".$this->request_shop_number;
            if (SG_DEBUG_LOGGING)
              $this->log($uQuery, ShopgateLogger::LOGTYPE_DEBUG); // AGI debug_logging
            $GLOBALS["DB"]->executeQuery($uQuery, 3);
            $message = 'Bestellung '.$row['kBestellungShopgate'].' war bereits manuell auf versendet gesetzt. JTL-Datenbank wurde aktualisiert.';
            $errorcount++;
          }
        }
        else
          echo 'Exception abgefangen: ',  $e->getMessage(), "\n";
      }
    }
  }

  protected function cronGetCancelOrder(&$message, &$errorcount)
  {
    $cQuery = "SELECT 
        xp.kBestellung, xp.kBestellungShopgate, tb.dVersandDatum
      FROM
        `xplugin_agi_shopgate_plugin_connect` AS xp, `tbestellung` AS tb
      WHERE
        xp.kBestellung=tb.kBestellung AND xp.sgBestellungStorniert=0 AND tb.cStatus='".BESTELLUNG_STATUS_STORNO."'
    ";
    if (!empty($this->request_shop_number))
      $cQuery .= " AND `sgShopNumber`=".$this->request_shop_number;
    if (SG_DEBUG_LOGGING)
      $this->log("QRY: ".$cQuery, ShopgateLogger::LOGTYPE_DEBUG); // AGI debug_logging
    
    $builder = new ShopgateBuilder($this->config);
    $api = $builder->buildMerchantApi();
    
    $aRes = $GLOBALS["DB"]->executeQuery($cQuery, 9);
    foreach ($aRes as $row)
    {
      try 
      {
        $resp = $api->cancelOrder($row['kBestellungShopgate'], 1);
        if ($resp->error == 0)
        {
          $aUpdate = array(
            'sgBestellungStorniert'  => 1
          );
          if (!empty($aUpdate))
          {
            $uQuery = "UPDATE `xplugin_agi_shopgate_plugin_connect` SET ";
            $i = 0;
            foreach ($aUpdate as $key => $value)
            {
              if($i > 0)
                $uQuery .= ", ";
              $uQuery .= "`$key`='".(is_string($value) ? mysql_real_escape_string($value) : $value)."'";
              $i++;
            }
            $uQuery .= " WHERE `kBestellung`=".$row['kBestellung'];
            if (!empty($this->request_shop_number))
              $uQuery .= " AND `sgShopNumber`=".$this->request_shop_number;
            if (SG_DEBUG_LOGGING)
              $this->log($uQuery, ShopgateLogger::LOGTYPE_DEBUG); // AGI debug_logging
            $GLOBALS["DB"]->executeQuery($uQuery, 3);
          }
        }
        else
        {
          $this->log('cronGetCancelOrder(): Code '.$resp->error.': '.$resp->error_text, ShopgateLogger::LOGTYPE_DEBUG);
        }
      }
      catch (Exception $e) 
      {
        $eCode = $e->getCode();
        if ($eCode == 222 || $eCode == 201)
        {
          $aUpdate = array(
            'sgBestellungStorniert'  => 1
          );
          if (!empty($aUpdate))
          {
            $uQuery = "UPDATE `xplugin_agi_shopgate_plugin_connect` SET ";
            $i = 0;
            foreach ($aUpdate as $key => $value)
            {
              if($i > 0)
                $uQuery .= ", ";
              $uQuery .= "`$key`='".(is_string($value) ? mysql_real_escape_string($value) : $value)."'";
              $i++;
            }
            $uQuery .= " WHERE `kBestellung`=".$row['kBestellung'];
            if (!empty($this->request_shop_number))
              $uQuery .= " AND `sgShopNumber`=".$this->request_shop_number;
            if (SG_DEBUG_LOGGING)
              $this->log($uQuery, ShopgateLogger::LOGTYPE_DEBUG); // AGI debug_logging
            $GLOBALS["DB"]->executeQuery($uQuery, 3);
            $message = 'Bestellung '.$row['kBestellungShopgate'].' war bereits manuell storniert gesetzt. JTL-Datenbank wurde aktualisiert.';
            $errorcount++;
          }
        }
        else
          echo 'Exception abgefangen: ',  $e->getMessage(), "\n";
      }
    }
  }
  /// EOF CRONJOB-FUNCTIONS ///
  
  /// BOF STOCK AND CART_FUNCTIONS ///
  public function checkStock(ShopgateCart $shopgateCart) 
  {
    $this->agiCheckStockIndividual($shopgateCart);
    $items = $shopgateCart->getItems();
    
    $this->_loadCreateItemsConfig();
    
    $aItemResponse = array();
    foreach($items as $checkItem)
    {
      $itemId = $checkItem->getItemNumber();
      $itemQuantity = 100;
      $itemOptions = $checkItem->getOptions();
      $itemInputs = $checkItem->getInputs();
      
      // prepare response
      $oItemResponse = new ShopgateCartItem();
      $oItemResponse->setItemNumber($itemId);
      $oItemResponse->setOptions($itemOptions);
      $oItemResponse->setInputs($itemInputs);
      
      // load Article
      $this->_loadOExportArticle($itemId);
      if (!empty($this->oExportArticle->kArtikel))
      {
        $this->_itemCheckStatusAktiv();
        $this->_itemLoadAttributes();
        
        $oItemResponse = new ShopgateCartItem();
        $oItemResponse->setItemNumber($itemId);
        
        if ($this->oExportArticle->cStatusAktiv == 'inactive')
        {
          $oItemResponse->setIsBuyable(0);
          $oItemResponse->setStockQuantity(0);
          $oItemResponse->setError(300);
          $oItemResponse->setErrorText('cart item out of stock');
        }
        elseif ($this->oExportArticle->cStatusAktiv == 'stock')
        {
          $oItemResponse->setStockQuantity($this->oExportArticle->fLagerbestand);
          /*
          if ($this->oExportArticle->fLagerbestand < $itemQuantity)
          {
            $oItemResponse->setQtyBuyable($this->oExportArticle->fLagerbestand);
            $oItemResponse->setIsBuyable(0);
            $oItemResponse->setError(302);
            $oItemResponse->setErrorText('cart item requested quantity not available');
          }
          else
          */
          if($this->oExportArticle->fLagerbestand < 1)
          {
            $oItemResponse->setError(300);
            $oItemResponse->setErrorText('cart item out of stock');
          }
          else
            $oItemResponse->setIsBuyable(1);
        }
        else
        {
          if (!$this->oExportArticle->nUseStock)
          {
            $oItemResponse->setStockQuantity(100);
          }
          else
            $oItemResponse->setStockQuantity($this->oExportArticle->fLagerbestand);
          $oItemResponse->setIsBuyable(1);
        }
        /* BOF 20160510 Konfiguration */
        $tmpOptions = array();
        $tmpKonfigurationArticle = array();
        if (!empty($itemOptions)) {
          foreach ($itemOptions as $checkOption)
          {
            $option_number = $checkOption->getOptionNumber();
            if (substr($option_number,0,10) == 'agiKonfig_')
              $tmpKonfigurationArticle[] = $checkOption;
            else
              $tmpOptions[] = $checkOption;
          }
        }
        $itemOptions = $tmpOptions;
        if ($this->bAllowKonfiguration && $this->oExportArticle->bHasKonfig && !empty($this->oExportArticle->oKonfig_arr)) {
          if (!empty($tmpKonfigurationArticle)) {
            foreach ($tmpKonfigurationArticle as $oKonfigurationOption) {
              $option_number = $oKonfigurationOption->getOptionNumber();
              $value_number = $oKonfigurationOption->getValueNumber();
              if ($value_number) {
                //array(kKonfiggruppe ,kKonfigitem, kArtikel);
                $aKonfigKeys = explode('_', $value_number);
                if (count($aKonfigKeys) == 3) {
                  $kItem = new Konfigitem($aKonfigKeys[1]);
                  if ($kItem->getKonfiggruppe() == $aKonfigKeys[0] && $kItem->getArtikelKey() == $aKonfigKeys[2]) {
                    $tmpArticle = $kItem->getArtikel();
                    if ($tmpArticle->inWarenkorbLegbar == -1) {
                      $oItemResponse->setIsBuyable(0);
                      $oItemResponse->setError(301);
                      $oItemResponse->setErrorText('cart item product-option not found');
                    }
                    else {
                      if ($tmpArticle->cLagerBeachten == 'Y' && $tmpArticle->cLagerKleinerNull == 'N' && $tmpArticle->fLagerbestand <= 0) {
                        $oItemResponse->setIsBuyable(0);
                        $oItemResponse->setError(301);
                        $oItemResponse->setErrorText('cart item product-option not found');
                      }
                    }
                  }
                  else {
                    $oItemResponse->setIsBuyable(0);
                    $oItemResponse->setError(301);
                    $oItemResponse->setErrorText('cart item product-option not found');
                  }
                }
                else {
                  $oItemResponse->setIsBuyable(0);
                  $oItemResponse->setError(301);
                  $oItemResponse->setErrorText('cart item product-option not found');
                }
              }
            }
          }
        }
        elseif(!empty($tmpKonfigurationArticle)) {
          $bAllowForBasket = false;
          $oItemResponse->setIsBuyable(0);
          $oItemResponse->setError(301);
          $oItemResponse->setErrorText('cart item product has no options');
        }
        /* EOF 20160510 Konfiguration */
        if (!empty($itemOptions) && !empty($this->oExportArticle->aSelectAttribute))
        {
          foreach ($itemOptions as $checkOption)
          {
            $option_number = $checkOption->getOptionNumber();
            $value_number = $checkOption->getValueNumber();
            if (
              array_key_exists($option_number, $this->oExportArticle->aSelectAttribute) 
              && array_key_exists($value_number, $this->oExportArticle->aSelectAttribute[$option_number]['aWerte']))
            {
              $aOption = $this->oExportArticle->aSelectAttribute[$option_number]['aWerte'][$value_number];
              if ($this->oExportArticle->cStatusAktiv == 'active' || ($this->oExportArticle->cStatusAktiv == 'stock' && ($aOption['fLagerbestand'] > 0 || $this->oExportArticle->cLagerVariation == 'N')))
              {
                if ($this->oExportArticle->cStatusAktiv == 'stock')
                {
                  if ($this->oExportArticle->cLagerVariation == 'Y' && $aOption['fLagerbestand'] <= 0)
                  {
                    $oItemResponse->setStockQuantity(0);
                    $oItemResponse->setIsBuyable(0);
                    $oItemResponse->setError(300);
                    $oItemResponse->setErrorText('cart item out of stock');
                  }
                  /*
                  elseif ($this->oExportArticle->cLagerVariation == 'Y' && $aOption['fLagerbestand'] < $itemQuantity)
                  {
                    $oItemResponse->setStockQuantity($aOption['fLagerbestand']);
                    $oItemResponse->setIsBuyable(0);
                    $oItemResponse->setQtyBuyable($aOption['fLagerbestand']);
                    $oItemResponse->setError(302);
                    $oItemResponse->setErrorText('cart item requested quantity not available');
                  }
                  */
                }
              }
              else
              {
                $oItemResponse->setIsBuyable(0);
                $oItemResponse->setStockQuantity(0);
                $oItemResponse->setError(300);
                $oItemResponse->setErrorText('cart item out of stock');
              }
            }
            else
            {
              $oItemResponse->setError(301);
              $oItemResponse->setErrorText('cart item product-option not found');
            }
          }
        }
        elseif (!empty($itemOptions) && empty($this->oExportArticle->aSelectAttribute))
        {
          $oItemResponse->setError(301);
          $oItemResponse->setErrorText('cart item product has no options');
        }
      }
      else
      {
        $oItemResponse->setError(301);
        $oItemResponse->setErrorText('cart item product not found');
      }
      
      $aItemResponse[] = $oItemResponse;
    }
    return $aItemResponse;
  }
  
  public function checkCart(ShopgateCart $shopgateCart)
  {
    $this->agiCheckCartIndividual($shopgateCart);
    $items = $shopgateCart->getItems();
    
    $this->_loadCreateItemsConfig();
    
    $aItemResponse = array();
    $aPositionen   = array();
    $aWarenWertGet = array('brutto'=>0, 'netto'=>0);
    $aWarenWertSet = array('brutto'=>0, 'netto'=>0);
    $_SESSION['Warenkorb'] = new Warenkorb();
    $_SESSION['Kunde'] = new Kunde();
    foreach($items as $checkItem)
    {
      $itemId = $checkItem->getItemNumber();
      $itemsTaxPercent = $checkItem->getTaxPercent();
      $itemQuantity = $checkItem->getQuantity();
      $itemOptions = $checkItem->getOptions();
      $itemInputs = $checkItem->getInputs();
      $aOptionsWert  = array('brutto'=>0, 'netto'=>0);
      
      $aWarenWertGet['brutto'] += $itemQuantity*$checkItem->getUnitAmountWithTax();
      $aWarenWertGet['netto'] += $itemQuantity*$checkItem->getUnitAmount();
      $bAllowForBasket = true;
      
      // prepare response
      $oItemResponse = new ShopgateCartItem();
      $oItemResponse->setItemNumber($itemId);
      $oItemResponse->setUnitAmount($checkItem->getUnitAmount());
      $oItemResponse->setUnitAmountWithTax($checkItem->getUnitAmountWithTax());
      $oItemResponse->setOptions($itemOptions);
      $oItemResponse->setInputs($itemInputs);
      
      // load Article
      $this->_loadOExportArticle($itemId);
      if (!empty($this->oExportArticle->kArtikel))
      {
        $this->_itemCheckStatusAktiv();
        $this->_itemLoadAttributes();
        
        $tmpKats = array();
        if(count($this->oExportArticle->oKategorie_arr) > 0)
        {
          foreach ($this->oExportArticle->oKategorie_arr as $oCategorie)
            $tmpKats[] = $oCategorie->kKategorie;
        }
        $aPosition = array(
          'kArtikel'    => $this->oExportArticle->kArtikel,
          'cArtNr'      => $this->oExportArticle->cArtNr,
          'aKategorien' => $tmpKats
        );
        $aPositionen[] = $aPosition;
        
        //$oItemResponse = new ShopgateCartItem();
        $oItemResponse->setItemNumber($itemId);
        
        if ($this->oExportArticle->cStatusAktiv == 'inactive')
        {
          $oItemResponse->setIsBuyable(0);
          $oItemResponse->setQtyBuyable(0);
          $oItemResponse->setStockQuantity(0);
          $oItemResponse->setError(300);
          $oItemResponse->setErrorText('cart item out of stock');
          $bAllowForBasket = false;
        }
        elseif ($this->oExportArticle->cStatusAktiv == 'stock')
        {
          $oItemResponse->setStockQuantity($this->oExportArticle->fLagerbestand);
          if ($this->oExportArticle->fLagerbestand < $itemQuantity)
          {
            $oItemResponse->setQtyBuyable($this->oExportArticle->fLagerbestand);
            $oItemResponse->setIsBuyable(0);
            $oItemResponse->setError(302);
            $oItemResponse->setErrorText('cart item requested quantity not available');
            if($this->oExportArticle->fLagerbestand < 1)
            {
              $oItemResponse->setError(300);
              $oItemResponse->setErrorText('cart item out of stock');
            }
            $bAllowForBasket = false;
          }
          else
          {
            if ($this->oExportArticle->fMindestbestellmenge > $itemQuantity)
            {
              $oItemResponse->setQtyBuyable($this->oExportArticle->fMindestbestellmenge);
              $oItemResponse->setIsBuyable(0);
              $oItemResponse->setError(304);
              $oItemResponse->setErrorText('requested quantity is lower than required minimum quantity');
              $bAllowForBasket = false;
            }
            else
            {
              $oItemResponse->setQtyBuyable($itemQuantity);
              $oItemResponse->setIsBuyable(1);
            }
          }
        }
        else
        {
          $oItemResponse->setStockQuantity($this->oExportArticle->fLagerbestand);
          $oItemResponse->setQtyBuyable($itemQuantity);
          $oItemResponse->setIsBuyable(1);
        }
        $oEigenschaftwerte_arr = array();
        /* BOF 20160510 Konfiguration */
        $tmpOptions = array();
        $tmpKonfigurationArticle = array();
        if (!empty($itemOptions)) {
          foreach ($itemOptions as $checkOption)
          {
            $option_number = $checkOption->getOptionNumber();
            if (substr($option_number,0,10) == 'agiKonfig_')
              $tmpKonfigurationArticle[] = $checkOption;
            else
              $tmpOptions[] = $checkOption;
          }
        }
        $itemOptions = $tmpOptions;
        if ($this->bAllowKonfiguration && $this->oExportArticle->bHasKonfig && !empty($this->oExportArticle->oKonfig_arr)) {
          if (!empty($tmpKonfigurationArticle)) {
            foreach ($tmpKonfigurationArticle as $oKonfigurationOption) {
              $option_number = $oKonfigurationOption->getOptionNumber();
              $value_number = $oKonfigurationOption->getValueNumber();
              if ($value_number) {
                //array(kKonfiggruppe ,kKonfigitem, kArtikel);
                $aKonfigKeys = explode('_', $value_number);
                if (count($aKonfigKeys) == 3) {
                  $kItem = new Konfigitem($aKonfigKeys[1]);
                  if ($kItem->getKonfiggruppe() == $aKonfigKeys[0] && $kItem->getArtikelKey() == $aKonfigKeys[2]) {
                    $tmpArticle = $kItem->getArtikel();
                    if ($tmpArticle->inWarenkorbLegbar == -1) {
                      $bAllowForBasket = false;
                      $oItemResponse->setIsBuyable(0);
                      $oItemResponse->setError(301);
                      $oItemResponse->setErrorText('cart item product-option not found');
                    }
                    else {
                      if ($tmpArticle->cLagerBeachten == 'Y' && $tmpArticle->cLagerKleinerNull == 'N' && $tmpArticle->fLagerbestand <= 0) {
                        $bAllowForBasket = false;
                        $oItemResponse->setIsBuyable(0);
                        $oItemResponse->setError(301);
                        $oItemResponse->setErrorText('cart item product-option not found');
                      }
                    }
                  }
                  else {
                    $bAllowForBasket = false;
                    $oItemResponse->setIsBuyable(0);
                    $oItemResponse->setError(301);
                    $oItemResponse->setErrorText('cart item product-option not found');
                  }
                }
                else {
                  $bAllowForBasket = false;
                  $oItemResponse->setIsBuyable(0);
                  $oItemResponse->setError(301);
                  $oItemResponse->setErrorText('cart item product-option not found');
                }
              }
            }
          }
        }
        elseif(!empty($tmpKonfigurationArticle)) {
          $bAllowForBasket = false;
          $oItemResponse->setIsBuyable(0);
          $oItemResponse->setError(301);
          $oItemResponse->setErrorText('cart item product has no options');
        }
        /* EOF 20160510 Konfiguration */
        if (!empty($itemOptions) && !empty($this->oExportArticle->aSelectAttribute))
        {
          foreach ($itemOptions as $checkOption)
          {
            $option_number = $checkOption->getOptionNumber();
            $value_number = $checkOption->getValueNumber();
            if (
              array_key_exists($option_number, $this->oExportArticle->aSelectAttribute) 
              && array_key_exists($value_number, $this->oExportArticle->aSelectAttribute[$option_number]['aWerte']))
            {
              $fOptionPrice = $checkOption->getAdditionalAmountWithTax()/100;
              $aOptionsWert['brutto'] += $fOptionPrice;
              $aOptionsWert['netto']  += round($fOptionPrice*100/(100+$itemsTaxPercent),5);
              $aOption = $this->oExportArticle->aSelectAttribute[$option_number]['aWerte'][$value_number];
              
              $oEigenschaftwerte = new stdClass();
              $oEigenschaftwerte->kEigenschaft = $option_number;
              $oEigenschaftwerte->cFreifeldWert= '';
              $oEigenschaftwerte->kEigenschaftWert=$value_number;
              $oEigenschaftwerte->fAufpreisNetto = round($fOptionPrice*100/(100+$itemsTaxPercent),5);
              $oEigenschaftwerte_arr[] = $oEigenschaftwerte;
              
              if ($this->oExportArticle->cStatusAktiv == 'active' || ($this->oExportArticle->cStatusAktiv == 'stock' && ($aOption['fLagerbestand'] > 0 || $this->oExportArticle->cLagerVariation == 'N')))
              {
                if ($this->oExportArticle->cStatusAktiv == 'stock')
                {
                  if ($this->oExportArticle->cLagerVariation == 'Y' && $aOption['fLagerbestand'] <= 0)
                  {
                    $oItemResponse->setStockQuantity(0);
                    $oItemResponse->setIsBuyable(0);
                    $oItemResponse->setError(300);
                    $oItemResponse->setErrorText('cart item out of stock');
                    $bAllowForBasket = false;
                  }
                  elseif ($this->oExportArticle->cLagerVariation == 'Y' && $aOption['fLagerbestand'] < $itemQuantity)
                  {
                    $oItemResponse->setStockQuantity($aOption['fLagerbestand']);
                    $oItemResponse->setIsBuyable(0);
                    $oItemResponse->setQtyBuyable($aOption['fLagerbestand']);
                    $oItemResponse->setError(302);
                    $oItemResponse->setErrorText('cart item requested quantity not available');
                    $bAllowForBasket = false;
                  }
                }
              }
              else
              {
                $oItemResponse->setIsBuyable(0);
                $oItemResponse->setStockQuantity(0);
                $oItemResponse->setError(300);
                $oItemResponse->setErrorText('cart item out of stock');
                $bAllowForBasket = false;
              }
            }
            else
            {
              $oItemResponse->setError(301);
              $oItemResponse->setErrorText('cart item product-option not found');
            }
          }
        }
        elseif (!empty($itemOptions) && empty($this->oExportArticle->aSelectAttribute))
        {
          $oItemResponse->setError(301);
          $oItemResponse->setErrorText('cart item product has no options');
        }
        
        if (!empty($itemInputs) && !empty($this->oExportArticle->aInputAttribute))
        {
          foreach ($itemInputs as $checkOption)
          {
            $option_number = $checkOption->getInputNumber();
            if (array_key_exists($option_number, $this->oExportArticle->aInputAttribute))
            {
              $oEigenschaftwerte = new stdClass();
              $oEigenschaftwerte->kEigenschaft = $option_number;
              $oEigenschaftwerte->cFreifeldWert= $checkOption->getUserInput();
              $oEigenschaftwerte->kEigenschaftWert=0;
              $oEigenschaftwerte->fAufpreisNetto=0;
              $oEigenschaftwerte_arr[] = $oEigenschaftwerte;
            }
          }
        }
      }
      else
      {
        $oItemResponse->setError(301);
        $oItemResponse->setErrorText('cart item product not found');
        $bAllowForBasket = false;
      }
      
      if ($bAllowForBasket)
      {
        $NeuePosition = new WarenkorbPos();
        $NeuePosition->Artikel = $this->oExportArticle;
        $NeuePosition->nAnzahl = $itemQuantity;
        $NeuePosition->kArtikel = $NeuePosition->Artikel->kArtikel;
        $NeuePosition->kVersandklasse = $NeuePosition->Artikel->kVersandklasse;
        $NeuePosition->kSteuerklasse = $NeuePosition->Artikel->kSteuerklasse;
        $NeuePosition->fPreisEinzelNetto = $checkItem->getUnitAmount();
        $NeuePosition->fPreis = $checkItem->getUnitAmount()+$aOptionsWert['netto'];		
        $NeuePosition->cArtNr = $NeuePosition->Artikel->cArtNr;
        $NeuePosition->nPosTyp = C_WARENKORBPOS_TYP_ARTIKEL;
        $NeuePosition->cEinheit = $NeuePosition->Artikel->cEinheit;
        $NeuePosition->fGesamtgewicht = $NeuePosition->gibGesamtgewicht();
        $NeuePosition->WarenkorbPosEigenschaftArr = array();
        
        if(is_array($NeuePosition->Artikel->Variationen) && count($NeuePosition->Artikel->Variationen) > 0)
        {
          foreach ($NeuePosition->Artikel->Variationen as $eWert)
          {
            foreach ($oEigenschaftwerte_arr as $oEigenschaftwerte)
            {
              //gleiche Eigenschaft suchen
              if ($oEigenschaftwerte->kEigenschaft == $eWert->kEigenschaft)
              {
                if ($eWert->cTyp=="FREIFELD" || $eWert->cTyp=="PFLICHT-FREIFELD")
                {
                  $NeueWarenkorbPosEigenschaft = new WarenkorbPosEigenschaft();
                  $NeueWarenkorbPosEigenschaft->kEigenschaft = $oEigenschaftwerte->kEigenschaft;
                  $NeueWarenkorbPosEigenschaft->kEigenschaftWert = 0;
                  $NeueWarenkorbPosEigenschaft->fGewichtsdifferenz = 0;
                  $NeueWarenkorbPosEigenschaft->fAufpreis = 0;
                  $NeuePosition->WarenkorbPosEigenschaftArr[] = $NeueWarenkorbPosEigenschaft;
                }
                elseif ($oEigenschaftwerte->kEigenschaftWert > 0)
                {
                  $EigenschaftWert = new EigenschaftWert($oEigenschaftwerte->kEigenschaftWert);
                  $Eigenschaft = new Eigenschaft($oEigenschaftwerte->kEigenschaft);
                  // Varkombi Kind?
                  if($NeuePosition->Artikel->kVaterArtikel > 0)
                  {
                    if ($Eigenschaft->kArtikel == $NeuePosition->Artikel->kVaterArtikel)
                    {
                      $NeueWarenkorbPosEigenschaft = new WarenkorbPosEigenschaft();
                      $NeueWarenkorbPosEigenschaft->kEigenschaft = $oEigenschaftwerte->kEigenschaft;
                      $NeueWarenkorbPosEigenschaft->kEigenschaftWert = $oEigenschaftwerte->kEigenschaftWert;
                      $NeueWarenkorbPosEigenschaft->fGewichtsdifferenz = $EigenschaftWert->fGewichtDiff;
                      $NeueWarenkorbPosEigenschaft->fAufpreis = $oEigenschaftwerte->fAufpreisNetto;
                      
                      $NeuePosition->WarenkorbPosEigenschaftArr[] = $NeueWarenkorbPosEigenschaft;
                      $NeuePosition->fGesamtgewicht += $itemQuantity*$EigenschaftWert->fGewichtDiff;
                    }
                  }
                  else
                  {
                    if ($Eigenschaft->kArtikel == $NeuePosition->kArtikel)
                    {
                      $NeueWarenkorbPosEigenschaft = new WarenkorbPosEigenschaft();
                      $NeueWarenkorbPosEigenschaft->kEigenschaft = $oEigenschaftwerte->kEigenschaft;
                      $NeueWarenkorbPosEigenschaft->kEigenschaftWert = $oEigenschaftwerte->kEigenschaftWert;
                      $NeueWarenkorbPosEigenschaft->fGewichtsdifferenz = $EigenschaftWert->fGewichtDiff;
                      $NeueWarenkorbPosEigenschaft->fAufpreis = $oEigenschaftwerte->fAufpreisNetto;
                      
                      $NeuePosition->WarenkorbPosEigenschaftArr[] = $NeueWarenkorbPosEigenschaft;
                      $NeuePosition->fGesamtgewicht += $itemQuantity*$EigenschaftWert->fGewichtDiff;
                    }
                  }
                }
              }
            }
          }
        }
        
        $_SESSION['Warenkorb']->PositionenArr[] = $NeuePosition;
        
        $aWarenWertSet['brutto'] += $itemQuantity*($checkItem->getUnitAmountWithTax()+$aOptionsWert['brutto']);
        $aWarenWertSet['netto'] += $itemQuantity*($checkItem->getUnitAmount()+$aOptionsWert['netto']);
      }
      $aItemResponse[] = $oItemResponse;
    }
    
    $aCustomerResponse = array('customer_groups'=>array());
    $kKunde = 0;
    $kKundengruppe = $this->kKundengruppe;
    $fKundeGuthabenBrutto = 0;
    $cKundenMail = $shopgateCart->getMail();
    if (!empty($cKundenMail))
    {
      $cMail = $this->_prepare_string_for_jtl($cKundenMail);
      $tmp = $GLOBALS["DB"]->executeQuery(
        "SELECT `kKunde`, `kKundengruppe`, `fGuthaben` FROM `tkunde` WHERE `cMail`='".$GLOBALS['DB']->escape($cMail)."' AND `nRegistriert`=1",
        1
      );
      if (!empty($tmp))
      {
        $kKundengruppe = $tmp->kKundengruppe;
        $kKunde = $tmp->kKunde;
        $fKundeGuthabenBrutto = $tmp->fGuthaben;
        $_SESSION['Kunde']->loadFromDB($kKunde);
      }
    }
    else
    {
      $nKundengruppe = 0; //$shopgateCart->getExternalCustomerGroupId();
      if (!empty($nKundengruppe))
      {
        $oKundengruppe = $GLOBALS['DB']->executeQuery(
          "SELECT `kKundengruppe` FROM `tkundengruppe` WHERE `kKundengruppe`='".(int)$nKundengruppe."'", 
          1
        );
        if(isset($oKundengruppe->kKundengruppe) && $oKundengruppe->kKundengruppe > 0)
          $kKundengruppe = $oKundengruppe->kKundengruppe;
      }
    }
    $group = new ShopgateCartCustomerGroup();
    $group->setId($kKundengruppe);
    $aCustomerResponse = new ShopgateCartCustomer();
    $aCustomerResponse->setCustomerGroups(array($group));
    
    $deliveryCountry = false;
    $oDelivery = $shopgateCart->getDeliveryAddress();
    if ($oDelivery)
    {
      $tmp = $this->_prepare_string_for_jtl($oDelivery->getCountry());
      if (!empty($tmp))
        $deliveryCountry = $tmp;
    }
    
    // ermittle Steuersatz
    $cLandSteuersatz = !$deliveryCountry ? $this->cLieferLand : $deliveryCountry;
    $tmp = $GLOBALS["DB"]->executeQuery(
      "SELECT `kSteuerzone` FROM `tsteuerzoneland` WHERE `cISO`='$cLandSteuersatz'",
      1
    );
    if (!empty($tmp))
    {
      $kSteuerzone = $tmp->kSteuerzone;
      $tmp = $GLOBALS["DB"]->executeQuery(
        "SELECT `kSteuerklasse` FROM `tsteuerklasse` WHERE `cStandard`='Y'",
        1
      );
      $kSteuerklasse = $tmp->kSteuerklasse;
      $tmp = $GLOBALS["DB"]->executeQuery(
        "SELECT `fSteuersatz` FROM `tsteuersatz` WHERE `kSteuerklasse`=$kSteuerklasse AND `kSteuerzone`=$kSteuerzone",
        1
      );
      $fSteuersatz = $tmp->fSteuersatz;
    }
    else
      $fSteuersatz = $this->fallback_steuersatz;
    
    $aCouponsResponse = array();
    $aGuthabenKupon = array();
    $aNeukundenKupon = array();
    $fSummeKupons = array('brutto'=>0, 'netto'=>0);
    $coupons = $shopgateCart->getExternalCoupons();
    $hatVersandkupon = false;
    $cnt_coupons = 0;
    foreach ($coupons as $checkCoupon)
    {
      //suche code
      $cCode = $checkCoupon->getCode();
      if (!empty($cCode))
      {
        $tmpCode = $this->_prepare_string_for_jtl($cCode);
        if ($tmpCode == 'GUTHABENVERRECHNUNG')
        {
          $aGuthabenKupon = array(
            'amount_netto'  => $checkCoupon->getAmountNet()/100
          );
        }
        elseif($tmpCode == 'NEUKUNDENRABATT')
        {
          $aNeukundenKupon = array(
            'amount_netto'  => $checkCoupon->getAmountNet()/100
          );
        }
        else
        {
          $cnt_coupons++;
          $oKupon = $GLOBALS["DB"]->executeQuery(
            "SELECT * FROM `tkupon` WHERE `cCode`='".$GLOBALS['DB']->escape($tmpCode)."'", 
            1
          );
          if($oKupon && $cnt_coupons <=1 )
          {
            $kuponSprache = $GLOBALS["DB"]->executeQuery("SELECT * FROM `tkuponsprache` WHERE `kKupon`=".$oKupon->kKupon." AND `cISOSprache`='".$_SESSION['cISOSprache']."'", 1);
            if ($oKupon->cWertTyp == 'prozent')
            {
              $kuponwert = array(
                'brutto'  => round($aWarenWertSet['brutto']*$oKupon->fWert/100, 5),
                'netto'   => round($aWarenWertSet['netto']*$oKupon->fWert/100, 5)
              );
            }
            else
            {
              $kuponwert = array(
                'brutto'  => $oKupon->fWert,
                'netto'   => round($oKupon->fWert*100/(100+$fSteuersatz), 5)
              );
            }
            $checkResult = $this->_checkCoupon($oKupon, $kKunde, $kKundengruppe, $aWarenWertSet, $aPositionen, $deliveryCountry);
            $kupon = new ShopgateExternalCoupon();
            $kupon->setIsValid($checkResult > 0 ? 0 : 1);
            $kupon->setNotValidMessage($checkResult > 0 ? $this->_getKuponErrorMessage($checkResult) : '');
            $kupon->setCode($this->_pss($oKupon->cCode));
            $kupon->setName($this->_pss($kuponSprache ? $kuponSprache->cName : $oKupon->cName));
            $kupon->setDescription('');
            //$kupon->setAmountGross($kuponwert['brutto']);
            $kupon->setAmountNet($kuponwert['netto']);
            $kupon->setTaxType('auto');
            $kupon->setIsFreeShipping($checkResult == 0 && $oKupon->cKuponTyp == 'versandkupon' ? 1 : 0);
            $aInfo = array('kKupon'=>$oKupon->kKupon);
            $kupon->setInternalInfo($checkResult == 0 ? $this->jsonEncode($aInfo) : ''); // ID des Coupons, um ihn bei Add-Order schnell zuordnen zu können
            $aCouponsResponse[] = $kupon;
            if ($checkResult == 0 && $oKupon->cKuponTyp == 'versandkupon')
              $hatVersandkupon = true;
            if ($checkResult == 0)
            {
              $fSummeKupons['brutto'] += $kuponwert['brutto'];
              $fSummeKupons['netto'] += $kuponwert['netto'];
            }
          }
          else
          {
            $kupon = $checkCoupon;
            $kupon->setIsValid(0);
            if ($cnt_coupons > 1)
              $kupon->setNotValidMessage($this->_pss('Sie können nur einen Rabattcode verwenden'));
            else
              $kupon->setNotValidMessage('Der Gutscheincode ist nicht korrekt');
            $aCouponsResponse[] = $kupon;
          }
        }
      }
    }
    
    $sgCoupons = $shopgateCart->getShopgateCoupons();
    foreach ($sgCoupons as $checkCoupon)
    {
      if ($checkCoupon->getIsFreeShipping())
        $hatVersandkupon = true;
    }
    
    $fWarenkorbWarenwert = array(
      'brutto'=>$aWarenWertSet['brutto']-$fSummeKupons['brutto'], 
      'netto'=>$aWarenWertSet['netto']-$fSummeKupons['netto']
    );
    
    // füge Guthabenkupon ein
    if (!empty($kKunde) && $fKundeGuthabenBrutto > 0 && $this->jtl_plugin_configuration['cGuthabenverrechnung'] == 'Y')
    {
      // erlaubtes Guthaben darf nicht höher als der Warenwert sein
      $fErlaubtesGuthaben = $fKundeGuthabenBrutto > $fWarenkorbWarenwert['brutto'] ? $fWarenkorbWarenwert['brutto'] : $fKundeGuthabenBrutto;
      if ($fErlaubtesGuthaben > 0)
      {
        $kupon = new ShopgateExternalCoupon();
        $kupon->setIsValid(1);
        $kupon->setNotValidMessage('');
        $kupon->setCode('GUTHABENVERRECHNUNG');
        $kupon->setName('Guthabenverrechnung');
        $kupon->setDescription('');
        //$kupon->setAmountGross($fErlaubtesGuthaben);
        $kupon->setAmountNet(round($fErlaubtesGuthaben*100/(100+$fSteuersatz), 5));
        $kupon->setTaxType('auto');
        $kupon->setIsFreeShipping(0);
        $aInfo = array('kundenguthaben'=>$kKunde);
        $kupon->setInternalInfo($this->jsonEncode($aInfo));
        $aCouponsResponse[] = $kupon;
      }
    }
    elseif (!empty($aGuthabenKupon))
    {
      $kupon = new ShopgateExternalCoupon();
      $kupon->setIsValid(0);
      $kupon->setNotValidMessage('Guthabenverrechnung nicht möglich');
      $kupon->setCode('GUTHABENVERRECHNUNG');
      $kupon->setName('Guthabenverrechnung');
      $kupon->setDescription('');
      $kupon->setAmountNet($aGuthabenKupon['amount_netto']);
      $kupon->setTaxType('auto');
      $kupon->setIsFreeShipping(0);
      $kupon->setInternalInfo('');
      $aCouponsResponse[] = $kupon;
    }
    
    // Füge möglichen neukundencoupon ein
    $delete_newclients_coupon = true;
    if ($kKunde > 0)
    {
      $tmp = $GLOBALS["DB"]->executeQuery(
        "SELECT COUNT(*) AS icount FROM `tbestellung` WHERE `kKunde`=".$kKunde,
        1
      );
      if ($tmp->icount == 0)
      {
        // neukundenkoupon ist möglich!
        $oKupons = $GLOBALS["DB"]->executeQuery(
          "SELECT * FROM `tkupon` WHERE `cKuponTyp`='neukundenkupon' AND `cAktiv`='Y' AND (`kKundengruppe`=-1 OR `kKundengruppe`=$kKundengruppe)", 
          2
        );
        $found = false;
        foreach ($oKupons as $oKupon)
        {
          $checkResult = $this->_checkCoupon($oKupon, $kKunde, $kKundengruppe, $aWarenWertSet, $aPositionen, $deliveryCountry);
          if ($checkResult == 0)
          {
            // Kupon gültig
            $found = true;
            break;
          }
        }
        if ($found)
        {
          if ($oKupon->cWertTyp == 'prozent')
          {
            $kuponwert = array(
              'brutto'  => round($aWarenWertSet['brutto']*$oKupon->fWert/100, 5),
              'netto'   => round($aWarenWertSet['netto']*$oKupon->fWert/100, 5)
            );
          }
          else
          {
            $kuponwert = array(
              'brutto'  => $oKupon->fWert,
              'netto'   => round($oKupon->fWert*100/(100+$fSteuersatz), 5)
            );
          }
          $kupon = new ShopgateExternalCoupon();
          $kupon->setIsValid(1);
          $kupon->setNotValidMessage('');
          $kupon->setCode('NEUKUNDENRABATT');
          $kupon->setName($this->_pss($kuponSprache ? $kuponSprache->cName : $oKupon->cName));
          $kupon->setDescription('');
          //$kupon->setAmountGross($kuponwert['brutto']);
          $kupon->setAmountNet($kuponwert['netto']);
          $kupon->setTaxType('auto');
          $kupon->setIsFreeShipping(0);
          $aInfo = array('kKupon'=>$oKupon->kKupon);
          $kupon->setInternalInfo($checkResult == 0 ? $this->jsonEncode($aInfo) : ''); // ID des Coupons, um ihn bei Add-Order schnell zuordnen zu können
          $aCouponsResponse[] = $kupon;
          $delete_newclients_coupon = false;
        }
      }
    }
    if ($delete_newclients_coupon && !empty($aNeukundenKupon))
    {
      $kupon = new ShopgateExternalCoupon();
      $kupon->setIsValid(0);
      $kupon->setNotValidMessage('Neukundenrabatt nicht möglich');
      $kupon->setCode('NEUKUNDENRABATT');
      $kupon->setName('Neukundenrabatt');
      $kupon->setDescription('');
      $kupon->setAmountNet($aNeukundenKupon['amount_netto']);
      $kupon->setTaxType('auto');
      $kupon->setIsFreeShipping(0);
      $kupon->setInternalInfo('');
      $aCouponsResponse[] = $kupon;
    }
    $aShippingMethods = array();
    
    $deliveryZip = $oDelivery ? $this->_prepare_string_for_jtl($oDelivery->getZipcode()) : false;
    $oVersandart_arr = $this->_gibMoeglicheVersandarten($deliveryCountry, $deliveryZip, gibVersandklassen($_SESSION['Warenkorb']), $kKundengruppe);
    /* BOF AGI 20150505 hide_shipping */
    $aVersteckeVersandart = array();
    if ($deliveryZip)
    {
      // BOF agws_freeVA_PLZ - Plugin
      $agws_freeVA = array();
      $aTeinstellungenFields = array("'agws_freeVA_PLZ'","'agws_freeVA_PLZ_vname'","'agws_freeVA_PLZ_mwert'","'agws_freeVA_PLZ_plzarr'");
      $aRes = $GLOBALS["DB"]->executeQuery("SELECT * FROM `tplugineinstellungen` WHERE `cName` IN (".implode(',', $aTeinstellungenFields).")", 9);
      foreach ($aRes as $row)
        $agws_freeVA[$row['cName']] = $row['cWert'];
      
      if (array_key_exists('agws_freeVA_PLZ', $agws_freeVA) && $agws_freeVA['agws_freeVA_PLZ'] == 'Y' && !empty($agws_freeVA['agws_freeVA_PLZ_vname']))
      {
        // kVersandart ermitteln
        $tmp = $GLOBALS["DB"]->executeQuery(
          "SELECT `kVersandart` FROM `tversandart` WHERE `cName`='".$agws_freeVA['agws_freeVA_PLZ_vname']."'",
          1
        );
        if ($tmp)
        {
          $kVersandart = $tmp -> kVersandart;
          $tmp = str_replace('-',',', $agws_freeVA['agws_freeVA_PLZ_plzarr']);
          if (!empty($tmp))
          {
            $plz = explode(',', $tmp);
            if (in_array($deliveryZip, $plz))
            {
              if (!empty($agws_freeVA['agws_freeVA_PLZ_mwert']) && $agws_freeVA['agws_freeVA_PLZ_mwert'] > 0)
              {
                if ($agws_freeVA['agws_freeVA_PLZ_mwert'] > $fWarenkorbWarenwert['brutto'])
                  $aVersteckeVersandart[] = $kVersandart;
              }
            }
            else
              $aVersteckeVersandart[] = $kVersandart;
          }
          else
            $aVersteckeVersandart[] = $kVersandart;
        }
      }
      // EOF agws_freeVA_PLZ - Plugin
    }
    /* EOF AGI 20150505 hide_shipping */
    
    foreach ($oVersandart_arr as $oVersandart)
    {
      if (in_array($oVersandart->kVersandart, $aVersteckeVersandart))
        continue;
      
      $oShippingMethod = new ShopgateShippingMethod();
      $oShippingMethod->setId($oVersandart->kVersandart);
      $oShippingMethod->setTitle($this->_pss($oVersandart->angezeigterName));
      $oShippingMethod->setDescription($this->_pss($this->_remove_html($oVersandart->angezeigterHinweistext)));
      //$oShippingMethod->setSortOrder($oVersandart->nSort);
      if ($oVersandart->fVersandkostenfreiAbX > 0 && $fWarenkorbWarenwert['brutto'] > $oVersandart->fVersandkostenfreiAbX)
      {
        $oShippingMethod->setAmountWithTax(0);
      }
      else
      {
        if ($oVersandart->eSteuer == 'brutto')
          $oShippingMethod->setAmountWithTax($hatVersandkupon ? 0 : $oVersandart->fEndpreis);
        else
        {
          $oShippingMethod->setAmount($hatVersandkupon ? 0 : $oVersandart->fEndpreis);
          $oShippingMethod->setTaxPercent($fSteuersatz);
          //$oShippingMethod->setTaxClass('default');
          $oShippingMethod->setAmountWithTax($hatVersandkupon ? 0 : round($oVersandart->fEndpreis*(100+$fSteuersatz)/100,2));
        }
      }
      $aInfo = array('kShipping'=>$oVersandart->kVersandart,'cShipping'=>$this->_pss($oVersandart->angezeigterName));
      $oShippingMethod->setInternalShippingInfo($this->jsonEncode($aInfo));
      $aShippingMethods[] = $oShippingMethod;
      
    }
    
    /* BOF Zahlungsarten */
    require_once DIR_BASE_PATH.'/includes/bestellvorgang_inc.php';
    $aPaymentMethods = array();
    $checkPossible = array();
    $tmp = $GLOBALS["DB"]->executeQuery("SELECT DISTINCT `kZahlungsart` FROM `tversandartzahlungsart`",2);
    foreach ($tmp as $value)
    {
      if (!in_array($value->kZahlungsart, $checkPossible))
        $checkPossible[] = $value->kZahlungsart;
    }
    $Zahlungsarten = $GLOBALS["DB"]->executeQuery(
      "SELECT * FROM tzahlungsart
			 WHERE (cKundengruppen IS NULL or cKundengruppen=\"\" or cKundengruppen like \"%".$kKundengruppe.";%\") AND tzahlungsart.nActive = 1 AND tzahlungsart.nNutzbar = 1
       ORDER BY tzahlungsart.nSort"
      ,
      2
    );
    for ($i=0;$i<count($Zahlungsarten);$i++)
    {
      if(!$Zahlungsarten[$i]->kZahlungsart)
        continue;
      if (!in_array($Zahlungsarten[$i]->kZahlungsart,$checkPossible))
        continue;
      
      $kPlugin = gibkPluginAuscModulId($Zahlungsarten[$i]->cModulId);
      if ($kPlugin == 0)
      {
        $tmp = explode('_',$Zahlungsarten[$i]->cModulId);
        $aCheckDir = array('heidelpay','paypal','ipayment','worldpay','sofortueberweisung','wirecard','postfinance','paymentpartner','saferpay','clickandbuy','mbqc','ut','billpay');
        if (in_array($tmp[1], $aCheckDir))
        {
          if (!is_dir(PFAD_ROOT . PFAD_INCLUDES_MODULES . $tmp[1]))
            continue;
        }
      }
      $einstellungen = $GLOBALS["DB"]->executeQuery("select * from teinstellungen where kEinstellungenSektion=".CONF_ZAHLUNGSARTEN." and cModulId=\"".$Zahlungsarten[$i]->cModulId."\"",2);
      foreach ($einstellungen as $einstellung)
      {
        $Zahlungsarten[$i]->einstellungen[$einstellung->cName] = $einstellung->cWert;
      }
      
      //Einstellungen beachten
      if (!zahlungsartGueltig($Zahlungsarten[$i]))
        continue;
      $pay = new ShopgatePaymentMethod();
      $pay -> setId($Zahlungsarten[$i]->kZahlungsart.'_'.$this->_pss($Zahlungsarten[$i]->cName));
      $aPaymentMethods[] = $pay;
    }
    /* EOF Zahlungsarten */
    
    $response = array(
      'currency'  => $_SESSION['Waehrung']->cISO,
      'customer'  => $aCustomerResponse,
      'external_coupons'  => $aCouponsResponse,
      'shipping_methods'  => $aShippingMethods,
      'items'     => $aItemResponse
    );
    
    $response['payment_methods'] = $aPaymentMethods;
    if (method_exists($this, 'agiCheckCart_CheckPayment'))
      $response = $this->agiCheckCart_CheckPayment($response, $shopgateCart);
    return $response;
  }
  public function redeemCoupons(ShopgateCart $shopgateCart)
  {
    $this->agiCheckCartIndividual($shopgateCart);
    $items = $shopgateCart->getItems();
    
    $this->_loadCreateItemsConfig();
    
    $aPositionen   = array();
    $aWarenWertGet = array('brutto'=>0, 'netto'=>0);
    $aWarenWertSet = array('brutto'=>0, 'netto'=>0);
    
    foreach($items as $checkItem)
    {
      $itemId = $checkItem->getItemNumber();
      $itemsTaxPercent = $checkItem->getTaxPercent();
      $itemQuantity = $checkItem->getQuantity();
      $itemOptions = $checkItem->getOptions();
      $itemInputs = $checkItem->getInputs();
      $aOptionsWert  = array('brutto'=>0, 'netto'=>0);
      
      $aWarenWertGet['brutto'] += $itemQuantity*$checkItem->getUnitAmountWithTax();
      $aWarenWertGet['netto'] += $itemQuantity*$checkItem->getUnitAmount();
      $bAllowForBasket = true;
      
      // load Article
      $this->_loadOExportArticle($itemId);
      if (!empty($this->oExportArticle->kArtikel))
      {
        $this->_itemCheckStatusAktiv();
        $this->_itemLoadAttributes();
        
        $tmpKats = array();
        if(count($this->oExportArticle->oKategorie_arr) > 0)
        {
          foreach ($this->oExportArticle->oKategorie_arr as $oCategorie)
            $tmpKats[] = $oCategorie->kKategorie;
        }
        $aPosition = array(
          'kArtikel'    => $this->oExportArticle->kArtikel,
          'cArtNr'      => $this->oExportArticle->cArtNr,
          'aKategorien' => $tmpKats
        );
        $aPositionen[] = $aPosition;
        
        if ($this->oExportArticle->cStatusAktiv == 'inactive')
        {
          $bAllowForBasket = false;
        }
        elseif ($this->oExportArticle->cStatusAktiv == 'stock')
        {
          if ($this->oExportArticle->fLagerbestand < $itemQuantity)
          {
            $bAllowForBasket = false;
          }
          else
          {
            if ($this->oExportArticle->fMindestbestellmenge > $itemQuantity)
            {
              $bAllowForBasket = false;
            }
          }
        }
        
        /* BOF 20160510 Konfiguration */
        $tmpOptions = array();
        $tmpKonfigurationArticle = array();
        if (!empty($itemOptions)) {
          foreach ($itemOptions as $checkOption)
          {
            $option_number = $checkOption->getOptionNumber();
            if (substr($option_number,0,10) == 'agiKonfig_')
              $tmpKonfigurationArticle[] = $checkOption;
            else
              $tmpOptions[] = $checkOption;
          }
        }
        $itemOptions = $tmpOptions;
        if ($this->bAllowKonfiguration && $this->oExportArticle->bHasKonfig && !empty($this->oExportArticle->oKonfig_arr)) {
          if (!empty($tmpKonfigurationArticle)) {
            foreach ($tmpKonfigurationArticle as $oKonfigurationOption) {
              $option_number = $oKonfigurationOption->getOptionNumber();
              $value_number = $oKonfigurationOption->getValueNumber();
              if ($value_number) {
                //array(kKonfiggruppe ,kKonfigitem, kArtikel);
                $aKonfigKeys = explode('_', $value_number);
                if (count($aKonfigKeys) == 3) {
                  $kItem = new Konfigitem($aKonfigKeys[1]);
                  if ($kItem->getKonfiggruppe() == $aKonfigKeys[0] && $kItem->getArtikelKey() == $aKonfigKeys[2]) {
                    $tmpArticle = $kItem->getArtikel();
                    if ($tmpArticle->inWarenkorbLegbar == -1) {
                      $bAllowForBasket = false;
                    }
                    else {
                      if ($tmpArticle->cLagerBeachten == 'Y' && $tmpArticle->cLagerKleinerNull == 'N' && $tmpArticle->fLagerbestand <= 0) {
                        $bAllowForBasket = false;
                      }
                    }
                  }
                  else {
                    $bAllowForBasket = false;
                  }
                }
                else {
                  $bAllowForBasket = false;
                }
              }
            }
          }
        }
        elseif(!empty($tmpKonfigurationArticle)) {
          $bAllowForBasket = false;
        }
        /* EOF 20160510 Konfiguration */
        if (!empty($itemOptions) && !empty($this->oExportArticle->aSelectAttribute))
        {
          foreach ($itemOptions as $checkOption)
          {
            $option_number = $checkOption->getOptionNumber();
            $value_number = $checkOption->getValueNumber();
            if (
              array_key_exists($option_number, $this->oExportArticle->aSelectAttribute) 
              && array_key_exists($value_number, $this->oExportArticle->aSelectAttribute[$option_number]['aWerte']))
            {
              $fOptionPrice = $checkOption->getAdditionalAmountWithTax()/100;
              $aOptionsWert['brutto'] += $fOptionPrice;
              $aOptionsWert['netto']  += round($fOptionPrice*100/(100+$itemsTaxPercent),5);
              $aOption = $this->oExportArticle->aSelectAttribute[$option_number]['aWerte'][$value_number];
              if ($this->oExportArticle->cStatusAktiv == 'active' || ($this->oExportArticle->cStatusAktiv == 'stock' && ($aOption['fLagerbestand'] > 0 || $this->oExportArticle->cLagerVariation == 'N')))
              {
                if ($this->oExportArticle->cStatusAktiv == 'stock')
                {
                  if ($this->oExportArticle->cLagerVariation == 'Y' && $aOption['fLagerbestand'] <= 0)
                  {
                    $bAllowForBasket = false;
                  }
                  elseif ($this->oExportArticle->cLagerVariation == 'Y' && $aOption['fLagerbestand'] < $itemQuantity)
                  {
                    $bAllowForBasket = false;
                  }
                }
              }
              else
              {
                $bAllowForBasket = false;
              }
            }
          }
        }
      }
      else
      {
        $bAllowForBasket = false;
      }
      
      if ($bAllowForBasket)
      {
        $aWarenWertSet['brutto'] += $itemQuantity*($checkItem->getUnitAmountWithTax()+$aOptionsWert['brutto']);
        $aWarenWertSet['netto'] += $itemQuantity*($checkItem->getUnitAmount()+$aOptionsWert['netto']);
      }
    }
    
    $kKunde = 0;
    $kKundengruppe = $this->kKundengruppe;
    $fKundeGuthabenBrutto = 0;
    $cKundenMail = $shopgateCart->getMail();
    if (!empty($cKundenMail))
    {
      $cMail = $this->_prepare_string_for_jtl($cKundenMail);
      $tmp = $GLOBALS["DB"]->executeQuery(
        "SELECT `kKunde`, `kKundengruppe`, `fGuthaben` FROM `tkunde` WHERE `cMail`='".$GLOBALS['DB']->escape($cMail)."' AND `nRegistriert`=1",
        1
      );
      if (!empty($tmp))
      {
        $kKundengruppe = $tmp->kKundengruppe;
        $kKunde = $tmp->kKunde;
        $fKundeGuthabenBrutto = $tmp->fGuthaben;
      }
    }
    else
    {
      $nKundengruppe = 0; //$shopgateCart->getExternalCustomerGroupId();
      if (!empty($nKundengruppe))
      {
        $oKundengruppe = $GLOBALS['DB']->executeQuery(
          "SELECT `kKundengruppe` FROM `tkundengruppe` WHERE `kKundengruppe`='".(int)$nKundengruppe."'", 
          1
        );
        if(isset($oKundengruppe->kKundengruppe) && $oKundengruppe->kKundengruppe > 0)
          $kKundengruppe = $oKundengruppe->kKundengruppe;
      }
    }
    
    $deliveryCountry = false;
    $oDelivery = $shopgateCart->getDeliveryAddress();
    if ($oDelivery)
    {
      $tmp = $this->_prepare_string_for_jtl($oDelivery->getCountry());
      if (!empty($tmp))
        $deliveryCountry = $tmp;
    }
    
    // ermittle Steuersatz
    $cLandSteuersatz = !$deliveryCountry ? $this->cLieferLand : $deliveryCountry;
    $tmp = $GLOBALS["DB"]->executeQuery(
      "SELECT `kSteuerzone` FROM `tsteuerzoneland` WHERE `cISO`='$cLandSteuersatz'",
      1
    );
    if (!empty($tmp))
    {
      $kSteuerzone = $tmp->kSteuerzone;
      $tmp = $GLOBALS["DB"]->executeQuery(
        "SELECT `kSteuerklasse` FROM `tsteuerklasse` WHERE `cStandard`='Y'",
        1
      );
      $kSteuerklasse = $tmp->kSteuerklasse;
      $tmp = $GLOBALS["DB"]->executeQuery(
        "SELECT `fSteuersatz` FROM `tsteuersatz` WHERE `kSteuerklasse`=$kSteuerklasse AND `kSteuerzone`=$kSteuerzone",
        1
      );
      $fSteuersatz = $tmp->fSteuersatz;
    }
    else
      $fSteuersatz = $this->fallback_steuersatz;
    
    $aCouponsResponse = array();
    $aGuthabenKupon = array();
    $fSummeKupons = array('brutto'=>0, 'netto'=>0);
    $coupons = $shopgateCart->getExternalCoupons();
    foreach ($coupons as $checkCoupon)
    {
      //suche code
      $cCode = $checkCoupon->getCode();
      if (!empty($cCode))
      {
        $tmpCode = $this->_prepare_string_for_jtl($cCode);
        if ($tmpCode == 'GUTHABENVERRECHNUNG')
        {
          $aGuthabenKupon = array(
            'amount_netto'  => $checkCoupon->getAmountNet()/100
          );
        }
        else
        {
          $oKupon = $GLOBALS["DB"]->executeQuery(
            "SELECT * FROM `tkupon` WHERE `cCode`='".$GLOBALS['DB']->escape($tmpCode)."'", 
            1
          );
          if($oKupon)
          {
            $kuponSprache = $GLOBALS["DB"]->executeQuery("SELECT * FROM `tkuponsprache` WHERE `kKupon`=".$oKupon->kKupon." AND `cISOSprache`='".$_SESSION['cISOSprache']."'", 1);
            if ($oKupon->cWertTyp == 'prozent')
            {
              $kuponwert = array(
                'brutto'  => round($aWarenWertSet['brutto']*$oKupon->fWert/100, 5),
                'netto'   => round($aWarenWertSet['netto']*$oKupon->fWert/100, 5)
              );
            }
            else
            {
              $kuponwert = array(
                'brutto'  => $oKupon->fWert,
                'netto'   => round($oKupon->fWert*100/(100+$fSteuersatz), 5)
              );
            }
            $checkResult = $this->_checkCoupon($oKupon, $kKunde, $kKundengruppe, $aWarenWertSet, $aPositionen, $deliveryCountry);
            $kupon = new ShopgateExternalCoupon();
            $kupon->setIsValid($checkResult > 0 ? 0 : 1);
            $kupon->setNotValidMessage($checkResult > 0 ? $this->_getKuponErrorMessage($checkResult) : '');
            $kupon->setCode($this->_pss($oKupon->cCode));
            $kupon->setName($this->_pss($kuponSprache ? $kuponSprache->cName : $oKupon->cName));
            $kupon->setDescription('');
            //$kupon->setAmountGross($kuponwert['brutto']);
            $kupon->setAmountNet($kuponwert['netto']);
            $kupon->setTaxType('auto');
            $kupon->setIsFreeShipping($checkResult == 0 && $oKupon->cKuponTyp == 'versandkupon' ? 1 : 0);
            $aInfo = array('kKupon'=>$oKupon->kKupon);
            $kupon->setInternalInfo($checkResult == 0 ? $this->jsonEncode($aInfo) : ''); // ID des Coupons, um ihn bei Add-Order schnell zuordnen zu können
            $aCouponsResponse[] = $kupon;
            if ($checkResult == 0)
            {
              $fSummeKupons['brutto'] += $kuponwert['brutto'];
              $fSummeKupons['netto'] += $kuponwert['netto'];
            }
          }
          else
          {
            $kupon = $checkCoupon;
            $kupon->setIsValid(0);
            $kupon->setNotValidMessage('Der Gutscheincode ist nicht korrekt');
            $aCouponsResponse[] = $kupon;
          }
        }
      }
    }
    
    $fWarenkorbWarenwert = array(
      'brutto'=>$aWarenWertSet['brutto']-$fSummeKupons['brutto'], 
      'netto'=>$aWarenWertSet['netto']-$fSummeKupons['netto']
    );
    
    // füge Guthabenkupon ein
    if (!empty($kKunde) && $fKundeGuthabenBrutto > 0 && !empty($aGuthabenKupon) && $this->jtl_plugin_configuration['cGuthabenverrechnung'] == 'Y')
    {
      // erlaubtes Guthaben darf nicht höher als der Warenwert sein
      $fErlaubtesGuthaben = $fKundeGuthabenBrutto > $fWarenkorbWarenwert['brutto'] ? $fWarenkorbWarenwert['brutto'] : $fKundeGuthabenBrutto;
      if ($fErlaubtesGuthaben > 0)
      {
        $kupon = new ShopgateExternalCoupon();
        $kupon->setIsValid(1);
        $kupon->setNotValidMessage('');
        $kupon->setCode('GUTHABENVERRECHNUNG');
        $kupon->setName('Guthabenverrechnung');
        $kupon->setDescription('');
        //$kupon->setAmountGross($fErlaubtesGuthaben);
        $kupon->setAmountNet(round($fErlaubtesGuthaben*100/(100+$fSteuersatz), 5));
        $kupon->setTaxType('auto');
        $kupon->setIsFreeShipping(0);
        $aInfo = array('kundenguthaben'=>$kKunde);
        $kupon->setInternalInfo($this->jsonEncode($aInfo));
        $aCouponsResponse[] = $kupon;
      }
    }
    elseif (!empty($aGuthabenKupon))
    {
      $kupon = new ShopgateExternalCoupon();
      $kupon->setIsValid(0);
      $kupon->setNotValidMessage('Guthabenverrechnung nicht möglich');
      $kupon->setCode('GUTHABENVERRECHNUNG');
      $kupon->setName('Guthabenverrechnung');
      $kupon->setDescription('');
      $kupon->setAmountNet($aGuthabenKupon['amount_netto']);
      $kupon->setIsFreeShipping(0);
      $kupon->setInternalInfo('');
      $aCouponsResponse[] = $kupon;
    }
    
    $response = array(
      'external_coupons'  => $aCouponsResponse
    );
    return $response;
  }
  protected function _checkCoupon($oKupon, $kKunde, $kKundengruppe, $aWarenwert, $aPositionen, $deliveryCountry=false)
  {
    $ungueltig = 0;
    if ($oKupon->cAktiv!="Y")
      $ungueltig = 1;
    if (strtotime($oKupon->dGueltigBis) < time() && $oKupon->dGueltigBis != '0000-00-00 00:00:00')
      $ungueltig = 2;
    if (strtotime($oKupon->dGueltigAb)>time())
      $ungueltig = 3;
    if ($oKupon->fMindestbestellwert>$aWarenwert['netto'])
      $ungueltig = 4;
    if ($oKupon->kKundengruppe>0 && $oKupon->kKundengruppe!=$kKundengruppe)
      $ungueltig = 5;
    if ($oKupon->nVerwendungen>0 && $oKupon->nVerwendungen<=$oKupon->nVerwendungenBisher)
      $ungueltig = 6;
    if ($oKupon->cArtikel && $this->_hatKuponfaehigeArtikel($oKupon,$aPositionen))
      $ungueltig = 7;
    if ($oKupon->cKategorien && $oKupon->cKategorien!=-1 && $this->_hatKuponfaehigeKategorien($oKupon,$aPositionen))
      $ungueltig = 8;
    if ($oKupon->cKunden!=-1 && $kKunde>0 && strpos($oKupon->cKunden, $kKunde . ";") === false && $oKupon->cKuponTyp!="neukundenkupon")
      $ungueltig = 9;    
    if ($oKupon->cKuponTyp === 'versandkupon' && $deliveryCountry && strpos($oKupon->cLieferlaender, $deliveryCountry) === false)
      $ungueltig = 10;
      
    if ($kKunde>0)
    {
      //hat der kunde schon die max. Verwendungsanzahl erreicht?
      $anz = $GLOBALS["DB"]->executeQuery("SELECT `nVerwendungen` FROM `tkuponkunde` WHERE `kKunde`=".$kKunde." AND `kKupon`=".$oKupon->kKupon, 1);
      if ($anz)
      {
        if ($anz->nVerwendungen>=$oKupon->nVerwendungenProKunde && $oKupon->nVerwendungenProKunde>0)
          $ungueltig = 6;
      }
    }
    
    return $ungueltig;
  }

  private function _hatKuponfaehigeArtikel($oKupon,$aPositionen)
  {
    if (is_array($aPositionen))
    {
      foreach ($aPositionen as $Pos)
      {
        if (eregi($Pos['cArtNr'].";",$oKupon->cArtikel))
          return false;
      }
    }
    else
      return false;
    return true;
  }

  private function _hatKuponfaehigeKategorien($oKupon,$aPositionen)
  {
    if (is_array($aPositionen))
    {
      $kats = array();
      foreach ($aPositionen as $Pos)
      {
        $tKats = $Pos['aKategorien'];
        if(pruefeIstVariKind($Pos['kArtikel']))
        {
          $kArtikel = gibkVaterArtikel($Pos['kArtikel']);
          $tKats = array();
          $Kats_arr = $GLOBALS["DB"]->executeQuery("SELECT `kKategorie` FROM `tkategorieartikel` WHERE `kArtikel` = " . $kArtikel, 2);
          if (is_array($Kats_arr))
          {
            foreach ($Kats_arr as $Kat)
            {
              if (!in_array($Kat->kKategorie,$tKats))
                $tKats[] = $Kat->kKategorie;
            }
          }
        }
        if (!empty($tKats))
          $kats = array_merge($kats, $tKats);
      }
      $kats = array_unique($kats);
      foreach ($kats as $kat)
      {
        if(stristr($oKupon->cKategorien, $kat . ";") !== false)		
          return false;
      }
    }
    else
      return false;
    return true;
  }

  private function _getKuponErrorMessage($code)
  {
    switch ($code)
    {
      case 1: return 'Der Kupon ist nicht aktiv (1)'; break;
      case 2: return 'Der Kupon ist abgelaufen (2)'; break;
      case 3: return 'Der Kupon ist nicht aktiv (3)'; break;
      case 4: return 'Der Mindestbestellwert für diesen Kupon ist nicht erreicht (4)'; break;
      case 5: return 'Kupon ist nicht aktiv (5)'; break;
      case 6: return 'Kupon ist nicht mehr aktiv (6)'; break;
      case 7: return 'Artikel im Warenkorb sind nicht Kupon-fähig (7)'; break;
      case 8: return 'Artikel im Warenkorb sind nicht Kupon-fähig (8)'; break;
      case 9: return 'Kupon ist nicht aktiv (9)'; break;
      case 10: return 'Der Kupon kann nicht mit dem gewählten Lieferland kombiniert werden. (10)'; break;
      default: return 'Der Kupon kann nicht genutzt werden';
    }
  }
  private function _gibMoeglicheVersandarten($lieferland, $plz, $versandklassen, $kKundengruppe)
  {
    $moeglicheVersandarten = array();
    $minVersand = 10000;
    
    if (empty($versandklassen))
      $versandklassen = $this->jtl_plugin_configuration['nStandardVersandklasse'];
    if (!$lieferland)
      $lieferland = $this->cLieferLand;
    $cISO = $lieferland;

    $cNurAbhaengigeVersandart = "N";
    if (JTL_VERSION < 400)
    {
      if (normalerArtikelversand($lieferland) == false)
        $cNurAbhaengigeVersandart = "Y";
    }
    else
    {
      if (VersandartHelper::normalerArtikelversand($lieferland) == false)
        $cNurAbhaengigeVersandart = "Y";
    }
      
    $versandarten = $GLOBALS["DB"]->executeQuery(
      "SELECT *
      FROM `tversandart`
      WHERE `cNurAbhaengigeVersandart`='" . $cNurAbhaengigeVersandart . "'
        AND `cLaender` LIKE \"%" . $cISO . "%\"
        AND (`cVersandklassen`='-1' OR (`cVersandklassen` LIKE \"% " . $versandklassen . " %\" OR `cVersandklassen` LIKE \"% " . $versandklassen . "\"))
        AND (`cKundengruppen`='-1' OR `cKundengruppen` LIKE '%;" . $kKundengruppe . ";%') ORDER BY `nSort`",
      2
    );
    $cnt = count($versandarten);

    for ($i=0;$i<$cnt;$i++)
    {
      if ($plz)
        $versandarten[$i]->Zuschlag = $this->_gibVersandZuschlag($versandarten[$i],$cISO,$plz);
      else
        $versandarten[$i]->Zuschlag = null;
      $versandarten[$i]->fEndpreis = berechneVersandpreis($versandarten[$i], $cISO, null);
      $name_spr = $GLOBALS["DB"]->executeQuery("SELECT `cName`, `cLieferdauer`, `cHinweistext` FROM `tversandartsprache` WHERE `kVersandart`=".$versandarten[$i]->kVersandart." AND `cISOSprache`='".$_SESSION['cISOSprache']."'",1);
      $versandarten[$i]->angezeigterName = $name_spr->cName;
			$versandarten[$i]->angezeigterHinweistext = $name_spr->cHinweistext;
			$versandarten[$i]->cLieferdauer = $name_spr->cLieferdauer;
      if ($versandarten[$i]->fEndpreis==-1)
      {
        unset($versandarten[$i]);
        continue;
      }
      if ($versandarten[$i]->fEndpreis<$minVersand)
        $minVersand = $versandarten[$i]->fEndpreis;
    }
    
    //$versandarten = array_merge($versandarten);

    //auf anzeige filtern
    foreach ($versandarten as $versandart)
    {
      switch ($versandart->cAnzeigen)
      {
        case 'guenstigste':
          if ($versandart->fEndpreis<=$minVersand)
            array_push($moeglicheVersandarten,$versandart);
          break;
        case 'immer':
            array_push($moeglicheVersandarten,$versandart);
          break;
      }
    }
    return $moeglicheVersandarten;
  }
  private function _gibVersandZuschlag($versandart,$cISO,$plz)
  {
    $waehrung = $_SESSION['Waehrung'];
    $versandzuschlaege = $GLOBALS["DB"]->executeQuery("SELECT * FROM `tversandzuschlag` WHERE `kVersandart`=".$versandart->kVersandart." AND `cISO`='".$cISO."'",2);
    foreach ($versandzuschlaege as $versandzuschlag)
    {
      $plz_x = $GLOBALS["DB"]->executeQuery("SELECT * FROM `tversandzuschlagplz` WHERE ((`cPLZAb`<='".$plz."' AND `cPLZBis`>='".$plz."') OR `cPLZ`='".$plz."') AND `kVersandzuschlag`=".$versandzuschlag->kVersandzuschlag,1);
      if ($plz_x && $plz_x->kVersandzuschlagPlz>0)
      {
        $versandzuschlag->fZuschlag = number_format($versandzuschlag->fZuschlag*$waehrung->fFaktor, 2, $waehrung->cTrennzeichenCent, $waehrung->cTrennzeichenTausend);
        return $versandzuschlag;
      }
    }
  }
  /// EOF STOCK AND CART_FUNCTIONS ///
  
  /// GET REVIEWS ///
  public function createReviews($limit = null, $offset = null, array $uids = array())
  {
    if (!empty($uids))
    {
      $inUids = '';
      foreach ($uids as $value)
      {
        if (!empty($inUids))
          $inUids .= ',';
        $inUids .= "'".(int)$value."'";
      }
      $q = "SELECT * FROM `tbewertung` WHERE `kArtikel` IN ($inUids)";
      if ($this->aTeinstellungenFields['bewertung_freischalten'] == 'Y')
        $q .= " AND `nAktiv`=1";
      $q .= " ORDER BY `kBewertung`";
    }
    else
    {
      $q = "SELECT * FROM `tbewertung`";
      if ($this->aTeinstellungenFields['bewertung_freischalten'] == 'Y')
        $q .= " WHERE `nAktiv`=1";
      $q .= " ORDER BY `kBewertung`";
    }
    $sLimit = '';
    if (!empty($limit))
      $sLimit = (int) $limit;
    if (!empty($limit) && !empty($offset))
      $sLimit = ((int) $offset).','.$sLimit;
    if (!empty($sLimit))
      $sLimit = ' LIMIT '.$sLimit;
    $result = $GLOBALS["DB"]->executeQuery($q.$sLimit, 9);
    if (!empty($result))
    {
      foreach ($result as $review)
      {
        $oReview = new Shopgate_Model_Review();
        $oReview->setUid($review['kBewertung']);
        $oReview->setItemUid($review['kArtikel']);
        $oReview->setTitle($this->_pss($review['cTitel']));
        $oReview->setText($this->_pss($this->_remove_html($review['cText'])));
        $oReview->setScore($review['nSterne'] * 2);
        $oReview->setReviewerName($this->_pss($review['cName']));
        $oReview->setDate($review['dDatum'].' 00:00:00');
        $this->addReviewModel($oReview);
      }
    }
  }
  
  public function getSettings()
  {
    $settings = array(
      'allowed_address_countries' => array(),
      'allowed_shipping_countries'=> array(),
			'customer_groups'       => array(),
      'tax'                   => array(
        'product_tax_classes'   => array(),
        'customer_tax_classes'  => array(array('id' => 1, 'key' => "default")),
        'tax_rates'             => array(),
        'tax_rules'             => array()
      )
		);
    
    // Zahlungsarten
    if (true)
    {
      $checkPossible = array();
      $tmp = $GLOBALS["DB"]->executeQuery("SELECT DISTINCT `kZahlungsart` FROM `tversandartzahlungsart`",2);
      foreach ($tmp as $value)
      {
        if (!in_array($value->kZahlungsart, $checkPossible))
          $checkPossible[] = $value->kZahlungsart;
      }
      $settings['payment_methods'] = array();
      $aRes = $GLOBALS["DB"]->executeQuery("SELECT * FROM `tzahlungsart` WHERE `nActive`=1 AND `nNutzbar`=1", 2);
      foreach ($aRes as $row)
      {
        if (in_array($row->kZahlungsart,$checkPossible))
        {
          $pay = array(
            'id' => $row->kZahlungsart.'_'.$this->_pss($row->cName)
          );
          $settings['payment_methods'][] = $pay;
        }
      }
    }
    
    $aLaender = gibBelieferbareLaender($this->kKundengruppe, $this->aTeinstellungenFields['kundenregistrierung_nur_lieferlaender'] == 'Y');
    if (empty($aLaender))
      $aLaender = gibBelieferbareLaender($this->kKundengruppe);
    $aSteuerLaender = array();
    foreach ($aLaender as $oLand)
    {
      $settings['allowed_address_countries'][] = array(
        'country' => $oLand->cISO,
        'state'   => array('all')
      );
      $settings['allowed_shipping_countries'][] = array(
        'country' => $oLand->cISO,
        'state'   => array('all')
      );
      $aSteuerLaender[$oLand->cISO] = $oLand->cISO;
    }
    $aRes = $GLOBALS["DB"]->executeQuery("SELECT * FROM `tkundengruppe`", 2);
    foreach ($aRes as $row)
    {
      $group = array(
        'id' => $row->kKundengruppe,
        'name' => $this->_pss($row->cName),
        'is_default' => $row->cStandard == 'Y' ? 1 : 0
      );
      $settings['customer_groups'][] = $group;
    }
    $aSteuerklassen = $GLOBALS["DB"]->executeQuery("SELECT * FROM `tsteuerklasse`", 2);
    $def_tax_class = '';
    foreach ($aSteuerklassen as $row)
    {
      $group = array(
        'id' => $row->kSteuerklasse,
        'key' => $this->_pss($row->cName)
      );
      $settings['tax']['product_tax_classes'][] = $group;
      if ($row->cStandard == 'Y')
        $def_tax_class = $row->cName;
    }
    if (empty($def_tax_class))
    {
      $def_rate = current($settings['tax']['product_tax_classes']);
      $def_tax_class = $def_rate['key'];
    }
    
    // Alle Steuerinfos laden
    $cQuery = "SELECT 
      ts.kSteuersatz, ts.kSteuerzone, ts.kSteuerklasse, ts.fSteuersatz, ts.nPrio,
      tz.cName AS cNameSteuerzone,
      tk.cName AS cNameSteuerklasse, tk.cStandard,
      tl.cISO 
    FROM tsteuersatz AS ts
    LEFT JOIN tsteuerzone AS tz ON ts.kSteuerzone=tz.kSteuerzone
    LEFT JOIN tsteuerklasse AS tk ON ts.kSteuerklasse=tk.kSteuerklasse
    LEFT JOIN tsteuerzoneland AS tl ON ts.kSteuerzone=tl.kSteuerzone
    WHERE ts.fSteuersatz > 0";
    $tax_rates = $GLOBALS["DB"]->executeQuery($cQuery, 9);
    $rules_set = array();
    foreach ($tax_rates as $tax)
    {
      $settings['tax']['tax_rates'][] = array(
        'id'            => $tax['kSteuersatz'].'_'.$tax['kSteuerzone'].'_'.$tax['cISO'],
        'key'           => $this->_pss($tax['fSteuersatz'].' '.$tax['cNameSteuerklasse'].' '.$tax['cNameSteuerzone'].' '.$tax['cISO']),
        'display_name'  => $this->_pss($this->tSprache['vat']),
        'tax_percent'   => $tax['fSteuersatz'],
        'country'       => $this->_pss($tax['cISO']),
        'state'         => '',
        'zipcode_type'  => 'all'
      );
      $cKeyTmp = $tax['kSteuerzone'].'_'.$tax['kSteuerklasse'];
      if (!in_array($cKeyTmp, $rules_set))
      {
        $settings['tax']['tax_rules'][] = array(
          'id'        => $cKeyTmp,
          'name'      => $this->_pss($tax['cNameSteuerzone'].' '.$tax['cNameSteuerklasse']),
          'priority'  => 100-$tax['nPrio'],
          'product_tax_classes'   => array(array('id'=> $tax['kSteuerklasse'], 'key' => $this->_pss($tax['cNameSteuerklasse']))),
          'customer_tax_classes'  => array(array('id'=> 1, 'key' => 'default')),
          'tax_rates' => array()
        );
        $rules_key = count($rules_set);
        $rules_set[] = $cKeyTmp;
      }
      else
        $rules_key = array_search($cKeyTmp, $rules_set);
      /*
      if (!array_key_exists($cKeyTmp, $settings['tax']['tax_rules']))
      {
        $settings['tax']['tax_rules'][$cKeyTmp] = array(
          'name'      => $this->_pss($tax['cNameSteuerzone'].' '.$tax['cNameSteuerklasse']),
          'priority'  => 100-$tax['nPrio'],
          'product_tax_classes'   => array(array('key' => $this->_pss($tax['cNameSteuerklasse']))),
          'customer_tax_classes'  => array(array('key' => 'default')),
          'tax_rates' => array()
        );
      }
      $settings['tax']['tax_rules'][$cKeyTmp]['tax_rates'][] = array(
        'id'  => $tax['kSteuersatz'].'_'.$tax['kSteuerzone'].'_'.$tax['cISO'],
        'key' => $this->_pss($tax['fSteuersatz'].' '.$tax['cNameSteuerklasse'].' '.$tax['cNameSteuerzone'].' '.$tax['cISO'])
      );
      */
      $settings['tax']['tax_rules'][$rules_key]['tax_rates'][] = array(
        'id'  => $tax['kSteuersatz'].'_'.$tax['kSteuerzone'].'_'.$tax['cISO'],
        'key' => $this->_pss($tax['fSteuersatz'].' '.$tax['cNameSteuerklasse'].' '.$tax['cNameSteuerzone'].' '.$tax['cISO'])
      );
      unset($aSteuerLaender[$tax['cISO']]);
    }
    /*
    foreach ($aSteuerLaender as $setLand => $setLandIso)
    {
      foreach ($aSteuerklassen as $aSteuerklasse)
      {
        $settings['tax']['tax_rates'][] = array(
          'id'            => '0_0_'.$setLandIso,
          'key'           => $this->_pss('0 '.$aSteuerklasse->cName.' steuerfrei '.$setLandIso),
          'display_name'  => $this->_pss($this->tSprache['vat']),
          'tax_percent'   => 0,
          'country'       => $this->_pss($setLandIso),
          'state'         => '',
          'zipcode_type'  => 'all'
        );
      }
      $cKeyTmp = '0_'.$aSteuerklasse->kSteuerklasse;
      if (!array_key_exists($cKeyTmp, $settings['tax']['tax_rules']))
      {
        $settings['tax']['tax_rules'][$cKeyTmp] = array(
          'name'      => $this->_pss('steuerfrei '.$aSteuerklasse->cName),
          'priority'  => 101,
          'product_tax_classes'   => array(array('key' => $this->_pss($aSteuerklasse->cName))),
          'customer_tax_classes'  => array(array('key' => 'default')),
          'tax_rates' => array()
        );
      }
      $settings['tax']['tax_rules'][$cKeyTmp]['tax_rates'][] = array(
        'id'  => '0_0_'.$setLandIso,
        'key' => $this->_pss('0 '.$aSteuerklasse->cName.' steuerfrei '.$setLandIso)
      );
    }
    */
    
    return $settings;
  }
  
  
  //// EMPTY FUNCTION, NOT IMPLEMENTED ////
  protected function createItemsCsv()
  {
    // empty function
  }
  protected function createReviewsCsv()
  {
    // empty function
  }
  protected function createMediaCsv()
  {
    // empty function
  }
  public function syncFavouriteList($customerToken, $items)
  {
    // empty function
  }
  public function getOrders($customerToken, $customerLanguage, $limit = 10, $offset = 0, $orderDateFrom = '', $sortOrder = 'created_desc')
  {
    // empty function
  }
  
  //// FUNCTIONS FOR AGI-JTL-PLUGIN ///
  public function getJTLPluginConfiguration($field)
  {
    if (!array_key_exists($field, $this->jtl_plugin_configuration))
      return null;
    else
      return $this->jtl_plugin_configuration['field'];
  }

  //// BOF FUNCTIONS TO BE OVERWRITTEN IN EXTENDED CLASS ////
  public function agiGetCustomerIndividualAddData($customer)
  {
    return $customer;
  }
  public function agiOverwriteCategoriesArray($aKategories, $limit = null, $offset = null, $uids = array())
  {
    return $aKategories;
  }
  public function agiManipulateCategoriesArrayItem($aItem)
  {
    return $aItem;
  }
  public function agiCreateItemsOnStart() { }
  public function agiManipulateItemObject() { }
  public function agiAddOrderIndividual($order) { }
  public function agiUpdateOrderIndividual($order) { }
  public function agiRegisterCustomerIndividual($user, $pass, $customer) { }
  public function agiCheckStockIndividual($shopgateCart) { }
  public function agiCheckCartIndividual($shopgateCart) { }
  //// EOF FUNCTIONS TO BE OVERWRITTEN IN EXTENDED CLASS ////
  
  /*
   * prepares shopgate String for JTL
   * @var string
   * @return string
   */
  public function _prepare_string_for_jtl($string)
  {
    //$string = utf8_decode($string);
    $string = $this->agi_mb_replace($string);
    $string = iconv('UTF-8', 'windows-1252', $this->clear_text_for_csv($string));
    return $string;
  }
  
  /*
   * prepares shopgate String for Shopgate
   * @var string
   * @return string
   */
  public function _prepare_string_for_shopgate($string, $csv=false)
  {
    //$string = utf8_encode($string);
    $string = iconv('windows-1252', 'UTF-8', $csv ? $this->clear_text_for_csv($string) : $string);
    return $string;
  }
  public function _pss($string, $csv=false)
  {
    if (is_string($string))
      return is_string($string) ? $this->_prepare_string_for_shopgate($string, $csv) : $string;
  }
  public function _setSessionVar($key, $value)
  {
    $_SESSION[$key] = $value;
  }
  /**
   * remove all html-Tags from the given string
   * @param   string  $string
   * @return  string
   */
  private function _remove_html($string)
  {
    $replace_to = array('<br>','<br />','<p>','</p>','&nbsp;','</tr>','</td>','</li>'); // needles
    $replace_with = array(chr(10),chr(10),chr(10).chr(10),chr(10),' ',chr(10),chr(9),chr(10)); // replace-with
    
    $string = str_replace($replace_to, $replace_with, $string);
    $string = strip_tags($string);
    $string = html_entity_decode($string,ENT_QUOTES,"windows-1252");
    $string = trim($string);
    return $string;
  }
  /* BOF CSV-Fallback */
  private function clear_text_for_csv($text, $short = 0, $mask=true)
  {
    $cleared_text = $text;
    $cleared_text = str_replace("\n"," ",$cleared_text);
    $cleared_text = str_replace("\r"," ",$cleared_text);
    $cleared_text = str_replace("\t"," ",$cleared_text);
    $cleared_text = str_replace("\v"," ",$cleared_text);
    if ($mask)
      $cleared_text = str_replace('"','""',$cleared_text);
    //$cleared_text = str_replace("|",",",$cleared_text);
    $cleared_text = preg_replace("/ {2,}/"," ",$cleared_text);
    if ($short > 0)
    {
      $shorten = wordwrap($cleared_text, $short, "|");
      $shorten = explode("|", $shorten);
      $cleared_text = $shorten[0];
    }
    
    $cleared_text = trim($cleared_text);
    return $cleared_text;
  }
  private function encodeValue($value)
  {
    //return is_string($value) ? '"'.utf8_encode($this->clear_text_for_csv($value)).'"' : $value;
    return is_string($value) ? '"'.iconv('ISO-8859-15', 'UTF-8', $this->clear_text_for_csv($value)).'"' : $value;
  }
  /* EOF CSV-Fallback */
  
  public function plugin_handle_payment($order)
  {
    // get payment-infos
    // load jtl payments
    $lang_default = $_SESSION['cISOSprache'];
    $jtlZahlarten = array();
    $aRes = $GLOBALS["DB"]->executeQuery("SELECT z.kZahlungsart, zs.cName, zs.cGebuehrname FROM `tzahlungsart` AS z, `tzahlungsartsprache` AS zs WHERE zs.kZahlungsart=z.kZahlungsart AND zs.cISOSprache = '$lang_default' ORDER BY z.kZahlungsart", 9);
    foreach ($aRes as $row)
      $jtlZahlarten[$row['kZahlungsart']] = array('name'=>$row['cName'],'gebuehrName'=>$row['cGebuehrname']);
    
    $shopgate_payments = array();
    if (!empty($this->jtl_plugin_configuration['payment_relation']))
      $shopgate_payments = unserialize($this->jtl_plugin_configuration['payment_relation']);
    
    $paymentInfos   = $order->getPaymentInfos();
    $check_payment  = $order->getPaymentMethod();
    
    $kZahlart = array_key_exists('OTHER', $shopgate_payments) ? $shopgate_payments['OTHER'] : 1;
    $cZahlart = array_key_exists('OTHER', $shopgate_payments) ? $shopgate_payments['OTHER'] : 'Zahlung';
    $cGebuehr = 'Gebühr';
    
    if (array_key_exists($check_payment, $shopgate_payments))
    {
      $kZahlart = $shopgate_payments[$check_payment]['id'];
      $cZahlart = array_key_exists($kZahlart, $jtlZahlarten) ? $jtlZahlarten[$kZahlart]['name'] : $shopgate_payments[$check_payment]['name'];
      $cGebuehr = array_key_exists($kZahlart, $jtlZahlarten) ? $jtlZahlarten[$kZahlart]['gebuehrName'] : $shopgate_payments[$check_payment]['name'].' Gebühr';
    }
    
    $hinweis_name   = array_key_exists('shopgate_payment_name', $paymentInfos) ? $this->_prepare_string_for_jtl($paymentInfos['shopgate_payment_name']) : $check_payment;
    $paymentComment = '';
    $paymentType    = $check_payment;
    if ($order->getIsPaid())
    {
      $tZahlungseingang = new stdClass();
      $tZahlungseingang->fZahlungsgebuehr   = 0;
      $tZahlungseingang->fBetrag            = $order->getAmountComplete();
      $tZahlungseingang->cISO               = 'EUR';
      $tZahlungseingang->cHinweis           = 'SG: '.$hinweis_name;
      $tZahlungseingang->cAbgeholt          = 'N';
      $tZahlungseingang->dZeit              = $order->getPaymentTime('Y-m-d H:i:s');
    }
    
    if ($check_payment == 'SHOPGATE')
    {
      // Payment Method: Shopgate 
      if ($this->jtl_plugin_configuration['cPaymentRelationSG'] == 'shopgate')
        $cZahlart = 'Shopgate';
      elseif ($this->jtl_plugin_configuration['cPaymentRelationSG'] == 'shopgatename')
        $cZahlart = $hinweis_name;
      $paymentComment .= 'Zahlung: '.$hinweis_name.chr(10);
      if ($order->getIsPaid())
      {
        $tZahlungseingang->cZahlungsanbieter  = 'SHOPGATE';
        $tZahlungseingang->cHinweis           = '';
      }
    }
    elseif ($check_payment == 'MWS')
    {
      // Amazon Payment
      $cZahlart = 'Amazon Payments';
      if ($order->getIsPaid())
      {
        $tZahlungseingang->cZahlungsanbieter  = 'Amazon Payments'; //$cZahlart;
        $tZahlungseingang->cHinweis           = $this->_prepare_string_for_jtl($paymentInfos['mws_order_id']);
      }
    }
    elseif ($check_payment == 'COD')
    {
      // Payment Method: Cash on Delivery
      if ($order->getIsPaid())
      {
        $tZahlungseingang->cZahlungsanbieter  = $cZahlart;
        $tZahlungseingang->cHinweis           = '';
      }
    }
    elseif ($check_payment == 'AUTHN_CC')
    {
      // Authorize.net
      if ($order->getIsPaid())
      {
        $tZahlungseingang->cZahlungsanbieter  = $cZahlart;
        $tZahlungseingang->cHinweis           = $this->_prepare_string_for_jtl($paymentInfos['transaction_id']);
      }
    }
    elseif ($check_payment == 'BRAINTR_CC')
    {
      // Braintree
      if ($order->getIsPaid())
      {
        $tZahlungseingang->cZahlungsanbieter  = $cZahlart;
        $tZahlungseingang->cHinweis           = $this->_prepare_string_for_jtl($paymentInfos['transaction_id']);
      }
    }
    elseif ($check_payment == 'CHASE_CC')
    {
      // Chase Paymentech
      if ($order->getIsPaid())
      {
        $tZahlungseingang->cZahlungsanbieter  = $cZahlart;
        $tZahlungseingang->cHinweis           = $this->_prepare_string_for_jtl($paymentInfos['chase_tx_ref_num']);
      }
    }
    elseif ($check_payment == 'CMPTOP_CC')
    {
      // Computop
      if ($order->getIsPaid())
      {
        $tZahlungseingang->cZahlungsanbieter  = $cZahlart;
        $tZahlungseingang->cHinweis           = $this->_prepare_string_for_jtl($paymentInfos['payment_id']);
      }
    }
    elseif ($check_payment == 'CYBRSRC_CC')
    {
      // CyberSource
      if ($order->getIsPaid())
      {
        $tZahlungseingang->cZahlungsanbieter  = $cZahlart;
        $tZahlungseingang->cHinweis           = '';
      }
    }
    elseif ($check_payment == 'DT_CC')
    {
      // DataTrans
      if ($order->getIsPaid())
      {
        $tZahlungseingang->cZahlungsanbieter  = $cZahlart;
        $tZahlungseingang->cHinweis           = $this->_prepare_string_for_jtl($paymentInfos['upp_transaction_id']);
      }
    }
    elseif ($check_payment == 'DTCASH_CC')
    {
      // DataCash
      if ($order->getIsPaid())
      {
        $tZahlungseingang->cZahlungsanbieter  = $cZahlart;
        $tZahlungseingang->cHinweis           = $this->_prepare_string_for_jtl($paymentInfos['merchantreference']);
      }
    }
    elseif ($check_payment == 'EWAY_CC')
    {
      // eWAY
      if ($order->getIsPaid())
      {
        $tZahlungseingang->cZahlungsanbieter  = $cZahlart;
        $tZahlungseingang->cHinweis           = '';
      }
    }
    elseif ($check_payment == 'FRSTDAT_CC')
    {
      // FirstData US
      if ($order->getIsPaid())
      {
        $tZahlungseingang->cZahlungsanbieter  = $cZahlart;
        $tZahlungseingang->cHinweis           = $this->_prepare_string_for_jtl($paymentInfos['transaction_tag']);
      }
    }
    elseif ($check_payment == 'MASTPAY_CC')
    {
      // Masterpayment CC
      if ($order->getIsPaid())
      {
        $tZahlungseingang->cZahlungsanbieter  = $cZahlart;
        $tZahlungseingang->cHinweis           = $this->_prepare_string_for_jtl($paymentInfos['ident_uid']);
      }
    }
    elseif ($check_payment == 'OGONE_CC')
    {
      // Ogone
      if ($order->getIsPaid())
      {
        $tZahlungseingang->cZahlungsanbieter  = $cZahlart;
        $tZahlungseingang->cHinweis           = $this->_prepare_string_for_jtl($paymentInfos['pay_id']);
      }
    }
    elseif ($check_payment == 'PAYJUNC_CC')
    {
      // PayJunction
      if ($order->getIsPaid())
      {
        $tZahlungseingang->cZahlungsanbieter  = $cZahlart;
        $tZahlungseingang->cHinweis           = $this->_prepare_string_for_jtl($paymentInfos['transaction_id']);
      }
    }
    elseif ($check_payment == 'PAYONE_CC')
    {
      // PayOne
      if ($order->getIsPaid())
      {
        $tZahlungseingang->cZahlungsanbieter  = $cZahlart;
        $tZahlungseingang->cHinweis           = '';
      }
    }
    elseif ($check_payment == 'PP_WSPP_CC')
    {
      // PayPal Website Payments Pro
      if ($order->getIsPaid())
      {
        $tZahlungseingang->cZahlungsanbieter  = $cZahlart;
        $tZahlungseingang->cHinweis           = $this->_prepare_string_for_jtl($paymentInfos['paypal_transaction_id']);
      }
    }
    elseif ($check_payment == 'REALEX_CC')
    {
      // Realex
      if ($order->getIsPaid())
      {
        $tZahlungseingang->cZahlungsanbieter  = $cZahlart;
        $tZahlungseingang->cHinweis           = $this->_prepare_string_for_jtl($paymentInfos['orderid']);
      }
    }
    elseif ($check_payment == 'SAGEPAY_CC')
    {
      // SagePay
      if ($order->getIsPaid())
      {
        $tZahlungseingang->cZahlungsanbieter  = $cZahlart;
        $tZahlungseingang->cHinweis           = $this->_prepare_string_for_jtl($paymentInfos['VPSTxId']);
      }
    }
    elseif ($check_payment == 'STRIPE_CC')
    {
      // Stripe
      if ($order->getIsPaid())
      {
        $tZahlungseingang->cZahlungsanbieter  = $cZahlart;
        $tZahlungseingang->cHinweis           = $this->_prepare_string_for_jtl($paymentInfos['id']);
      }
    }
    elseif ($check_payment == 'TELECSH_CC')
    {
      // Telecash
      if ($order->getIsPaid())
      {
        $tZahlungseingang->cZahlungsanbieter  = $cZahlart;
        $tZahlungseingang->cHinweis           = $this->_prepare_string_for_jtl($paymentInfos['approval_code']);
      }
    }
    elseif ($check_payment == 'TRNSFST_CC')
    {
      // TransFirst
      if ($order->getIsPaid())
      {
        $tZahlungseingang->cZahlungsanbieter  = $cZahlart;
        $tZahlungseingang->cHinweis           = $this->_prepare_string_for_jtl($paymentInfos['response_switch_key']);
      }
    }
    elseif ($check_payment == 'USAEPAY_CC')
    {
      // USA ePay
      if ($order->getIsPaid())
      {
        $tZahlungseingang->cZahlungsanbieter  = $cZahlart;
        $tZahlungseingang->cHinweis           = $this->_prepare_string_for_jtl($paymentInfos['reference_number']);
      }
    }
    elseif ($check_payment == 'HIPAY')
    {
      // HiPay
      if ($order->getIsPaid())
      {
        $tZahlungseingang->cZahlungsanbieter  = $cZahlart;
        $tZahlungseingang->cHinweis           = '';
      }
    }
    elseif ($check_payment == 'SIX_CC')
    {
      // SIX Saferpay
      if ($order->getIsPaid())
      {
        $tZahlungseingang->cZahlungsanbieter  = $cZahlart;
        $tZahlungseingang->cHinweis           = $this->_prepare_string_for_jtl($paymentInfos['transaction_id_six']);
      }
    }
    elseif ($check_payment == 'SKRILL_CC')
    {
      // Skrill
      if ($order->getIsPaid())
      {
        $tZahlungseingang->cZahlungsanbieter  = $cZahlart;
        $tZahlungseingang->cHinweis           = $this->_prepare_string_for_jtl($paymentInfos['mb_transaction_id']);
      }
    }
    elseif ($check_payment == 'WIRECRD_CC')
    {
      // Wirecard
      if ($order->getIsPaid())
      {
        $tZahlungseingang->cZahlungsanbieter  = $cZahlart;
        $tZahlungseingang->cHinweis           = $this->_prepare_string_for_jtl($paymentInfos['transaction_id']);
      }
    }
    elseif ($check_payment == 'WORLDLINE')
    {
      // Worldline
      if ($order->getIsPaid())
      {
        $tZahlungseingang->cZahlungsanbieter  = $cZahlart;
        $tZahlungseingang->cHinweis           = '';
      }
    }
    elseif ($check_payment == 'DEBIT')
    {
      // Lastschrift
      $paymentComment .= 'bank_account_holder: '.$this->_prepare_string_for_jtl($paymentInfos["bank_account_holder"]).chr(10);
      $paymentComment .= 'iban: '.$this->_prepare_string_for_jtl($paymentInfos["iban"]).chr(10);
      $paymentComment .= 'bank_name: '.$this->_prepare_string_for_jtl($paymentInfos["bank_name"]).chr(10);
      $paymentComment .= 'bic: '.$this->_prepare_string_for_jtl($paymentInfos["bic"]).chr(10);
      if ($order->getIsPaid())
      {
        $tZahlungseingang->cZahlungsanbieter  = $cZahlart;
        $tZahlungseingang->cHinweis           = '';
      }
    }
    elseif ($check_payment == 'PAYMRW_DBT')
    {
      // Direct debit via Paymorrow
      if ($order->getIsPaid())
      {
        $tZahlungseingang->cZahlungsanbieter  = $cZahlart;
        $tZahlungseingang->cHinweis           = $this->_prepare_string_for_jtl($paymentInfos['pm_order_transaction_id']);
      }
    }
    elseif ($check_payment == 'PAYONE_DBT')
    {
      // Direct debit via PayOne
      if ($order->getIsPaid())
      {
        $tZahlungseingang->cZahlungsanbieter  = $cZahlart;
        $tZahlungseingang->cHinweis           = '';
      }
    }
    elseif ($check_payment == 'INVOICE')
    {
      // Rechnung
      if ($order->getIsPaid())
      {
        $tZahlungseingang->cZahlungsanbieter  = $cZahlart;
        $tZahlungseingang->cHinweis           = '';
      }
    }
    elseif ($check_payment == 'BILLSAFE')
    {
      // Prüfe ob Plugin existiert und aktiv
      $pData = $GLOBALS["DB"]->executeQuery("SELECT * FROM `tplugin` WHERE `cPluginID`='billsafe_sus_its'", 1);
      if (!empty($pData))
      {
        $oPlugin = Plugin::getPluginById('billsafe_sus_its');
        if (!empty($oPlugin -> oPluginEinstellungAssoc_arr['merchantId']) && !empty($oPlugin -> oPluginEinstellungAssoc_arr['applicationSignature']))
        {
          require_once ($oPlugin -> cAdminmenuPfad . "BillsafeSDK/Sdk.php");
          require_once ($oPlugin -> cAdminmenuPfad . "BillsafeSDK/HttpResponse.php");
          require_once ($oPlugin -> cAdminmenuPfad . "BillsafeSDK/Exception.php");
          require_once ($oPlugin -> cAdminmenuPfad . "BillsafeSDK/HttpClient.php");
          require_once ($oPlugin -> cAdminmenuPfad . "BillsafeSDK/Logger.php");
          require_once ($oPlugin -> cAdminmenuPfad . "BillsafeSDK/LoggerNull.php");
          $oPlugin -> name = 'Billsafe';
          $oPlugin -> caption = 'Billsafe';
          $oPlugin -> bs = new Billsafe_Sdk();
          $oPlugin -> bs -> setCredentials(array('merchantId' => $oPlugin -> oPluginEinstellungAssoc_arr['merchantId'], 'applicationSignature' => $oPlugin -> oPluginEinstellungAssoc_arr['applicationSignature'], 'applicationVersion' => $oPlugin -> nVersion));
          if ($oPlugin -> oPluginEinstellungAssoc_arr["sandboxmodus"] == 'False') {
            $oPlugin -> bs -> setCredentials(array('merchantLicenseLive' => $oPlugin -> oPluginEinstellungAssoc_arr["merchantLicenseLive"]));
            $oPlugin -> bs -> setMode('LIVE');
          } else {
            $oPlugin -> bs -> setCredentials(array('merchantLicenseSandbox' => $oPlugin -> oPluginEinstellungAssoc_arr["merchantLicenseSandbox"]));
            $oPlugin -> bs -> setMode('SANDBOX');
          }
          $comment_params = array('transactionId' => $this->_prepare_string_for_jtl($paymentInfos['billsafe_transaction_id']), 'outputType' => 'STRUCTURED');
          $comment_response = null; $callMethodCount = 0; $callMethodCountMax = 3; 
          while( empty($comment_response) && $callMethodCount < $callMethodCountMax ) 
          {
            usleep(400);
            $comment_response = $oPlugin -> bs -> callMethod('getPaymentInstruction', $comment_params);
            $callMethodCount++;
          }
          if ($comment_response -> ack == 'OK') 
          {
            $comment_text .= 'Bitte überweisen Sie den Rechnungsbetrag von ' . number_format($comment_response -> instruction -> amount, 2) . ' ' . $comment_response -> instruction -> currencyCode . ' auf folgendes Konto:\n';
            $comment_text .= $comment_response -> instruction -> recipient . '\n';
            $comment_text .= $comment_response -> instruction -> bankName . '\n';
            //$comment_text .= 'KTN:  ' . $comment_response -> instruction -> accountNumber . '\n';
            //$comment_text .= 'BLZ:  ' . $comment_response -> instruction -> bankCode . '\n';
            $comment_text .= 'IBAN: ' . $comment_response -> instruction -> iban . '\n';
            $comment_text .= 'BIC:  ' . $comment_response -> instruction -> bic . '\n';
            $comment_text .= 'Verwendungszweck: ' . $comment_response -> instruction -> reference . '\n\n';
            $comment_text .= $comment_response -> instruction -> note . '\n';
            $comment_text .= $comment_response -> instruction -> legalNote . '\n';
            $paymentComment .= $comment_text;
          }
        }
      }
      // Billsafe
      if ($order->getIsPaid())
      {
        $tZahlungseingang->cZahlungsanbieter  = $cZahlart;
        $tZahlungseingang->cHinweis           = $this->_prepare_string_for_jtl($paymentInfos['billsafe_transaction_id']);
      }
    }
    elseif ($check_payment == 'KLARNA_INV')
    {
      // Klarna
      if ($order->getIsPaid())
      {
        $tZahlungseingang->cZahlungsanbieter  = $cZahlart;
        $tZahlungseingang->cHinweis           = $this->_prepare_string_for_jtl($paymentInfos['reservation_number']);
      }
    }
    elseif ($check_payment == 'MSTPAY_INV')
    {
      // Masterpayment Rechnung
      if ($order->getIsPaid())
      {
        $tZahlungseingang->cZahlungsanbieter  = $cZahlart;
        $tZahlungseingang->cHinweis           = $this->_prepare_string_for_jtl($paymentInfos['invoice_number']);
      }
    }
    elseif ($check_payment == 'PAYMRW_INV')
    {
      // Paymorrow Rechnung
      if ($order->getIsPaid())
      {
        $tZahlungseingang->cZahlungsanbieter  = $cZahlart;
        $tZahlungseingang->cHinweis           = $this->_prepare_string_for_jtl($paymentInfos['pm_order_transaction_id']);
      }
    }
    elseif ($check_payment == 'PAYONE_INV')
    {
      // PayOne Rechnung
      if ($order->getIsPaid())
      {
        $tZahlungseingang->cZahlungsanbieter  = $cZahlart;
        $tZahlungseingang->cHinweis           = '';
      }
    }
    elseif ($check_payment == 'WCARD_INV')
    {
      // Wirecard Rechnung
      if ($order->getIsPaid())
      {
        $tZahlungseingang->cZahlungsanbieter  = $cZahlart;
        $tZahlungseingang->cHinweis           = $this->_prepare_string_for_jtl($paymentInfos['response_id']);
      }
    }
    elseif ($check_payment == 'UPAID_MCM')
    {
      // MasterPass
      if ($order->getIsPaid())
      {
        $tZahlungseingang->cZahlungsanbieter  = $cZahlart;
        $tZahlungseingang->cHinweis           = $this->_prepare_string_for_jtl($paymentInfos['upaid_transaction_id']);
      }
    }
    elseif ($check_payment == 'PAYPAL')
    {
      // PayPal
      if (strtolower($this->_prepare_string_for_jtl($paymentInfos["payment_status"])) == 'completed')
      {
        $tZahlungseingang->cZahlungsanbieter  = $cZahlart;
        $tZahlungseingang->fZahlungsgebuehr   = $this->_prepare_string_for_jtl($paymentInfos["mc_fee"]);
        $tZahlungseingang->fBetrag            = $this->_prepare_string_for_jtl($paymentInfos["mc_gross"]);
        $tZahlungseingang->cISO               = $this->_prepare_string_for_jtl($paymentInfos["mc_currency"]);
        $tZahlungseingang->cEmpfaenger        = $this->_prepare_string_for_jtl($paymentInfos["receiver_email"]);
        $tZahlungseingang->cZahler            = $this->_prepare_string_for_jtl($paymentInfos["payer_email"]);
        $tZahlungseingang->cHinweis           = $this->_prepare_string_for_jtl($paymentInfos["transaction_id"]);
      }
    }
    elseif ($check_payment == 'CMPTOP_PP')
    {
      // PayPal via Computop
      if ($order->getIsPaid())
      {
        $tZahlungseingang->cZahlungsanbieter  = $cZahlart;
        $tZahlungseingang->cHinweis           = $this->_prepare_string_for_jtl($paymentInfos['transaction_id']);
      }
    }
    elseif ($check_payment == 'MASTPAY_PP')
    {
      // PayPal via Masterpayment
      if ($order->getIsPaid())
      {
        $tZahlungseingang->cZahlungsanbieter  = $cZahlart;
        $tZahlungseingang->cHinweis           = $this->_prepare_string_for_jtl($paymentInfos['ident_uid']);
      }
    }
    elseif ($check_payment == 'PAYONE_PP')
    {
      // PayPal via PayOne
      if ($order->getIsPaid())
      {
        $tZahlungseingang->cZahlungsanbieter  = $cZahlart;
        $tZahlungseingang->cHinweis           = '';
      }
    }
    elseif ($check_payment == 'SAGEPAY_PP')
    {
      // PayPal via SagePay
      if ($order->getIsPaid())
      {
        $tZahlungseingang->cZahlungsanbieter  = $cZahlart;
        $tZahlungseingang->cHinweis           = $this->_prepare_string_for_jtl($paymentInfos['VPSTxId']);
      }
    }
    elseif ($check_payment == 'SIX_PP')
    {
      // PayPal via Six Saferpay
      if ($order->getIsPaid())
      {
        $tZahlungseingang->cZahlungsanbieter  = $cZahlart;
        $tZahlungseingang->cHinweis           = $this->_prepare_string_for_jtl($paymentInfos['transaction_id_six']);
      }
    }
    elseif ($check_payment == 'WCARD_PP')
    {
      // PayPal via WireCard
      if ($order->getIsPaid())
      {
        $tZahlungseingang->cZahlungsanbieter  = $cZahlart;
        $tZahlungseingang->cHinweis           = $this->_prepare_string_for_jtl($paymentInfos['response_id']);
      }
    }
    elseif ($check_payment == 'PAYU')
    {
      // PAYU
      if ($order->getIsPaid())
      {
        $tZahlungseingang->cZahlungsanbieter  = $cZahlart;
        $tZahlungseingang->cHinweis           = '';
      }
    }
    elseif ($check_payment == 'PREPAY')
    {
      // Vorkasse
      $paymentComment .= 'Referenz: '.$this->_prepare_string_for_jtl($paymentInfos['purpose']);
      if ($order->getIsPaid())
      {
        $tZahlungseingang->cZahlungsanbieter  = $cZahlart;
        $tZahlungseingang->cHinweis           = '';
      }
    }
    elseif ($check_payment == 'PAYONE_PRP')
    {
      // Vorkasse Payone
      if ($order->getIsPaid())
      {
        $tZahlungseingang->cZahlungsanbieter  = $cZahlart;
        $tZahlungseingang->cHinweis           = '';
      }
    }
    elseif ($check_payment == 'SUE')
    {
      // Sofortueberweisung
      if ($order->getIsPaid())
      {
        $tZahlungseingang->cZahlungsanbieter  = $cZahlart;
        $tZahlungseingang->cHinweis           = $this->_prepare_string_for_jtl($paymentInfos["transaction_id"]);
      }
    }
    elseif ($check_payment == 'MASTPAY_SUE')
    {
      // Sofortueberweisung Masterpayment
      if ($order->getIsPaid())
      {
        $tZahlungseingang->cZahlungsanbieter  = $cZahlart;
        $tZahlungseingang->cHinweis           = $this->_prepare_string_for_jtl($paymentInfos["ident_uid"]);
      }
    }
    elseif ($check_payment == 'SKRILL_SUE')
    {
      // Sofortueberweisung Skrill
      if ($order->getIsPaid())
      {
        $tZahlungseingang->cZahlungsanbieter  = $cZahlart;
        $tZahlungseingang->cHinweis           = $this->_prepare_string_for_jtl($paymentInfos["mb_transaction_id"]);
      }
    }
    elseif ($check_payment == 'WCARD_SUE')
    {
      // Sofortueberweisung Wirecard
      if ($order->getIsPaid())
      {
        $tZahlungseingang->cZahlungsanbieter  = $cZahlart;
        $tZahlungseingang->cHinweis           = $this->_prepare_string_for_jtl($paymentInfos["response_id"]);
      }
    }
    elseif ($check_payment == 'SIX_IDEAL')
    {
      // iDEAL via SIX Saferpay
      if ($order->getIsPaid())
      {
        $tZahlungseingang->cZahlungsanbieter  = $cZahlart;
        $tZahlungseingang->cHinweis           = $this->_prepare_string_for_jtl($paymentInfos["transaction_id_six"]);
      }
    }
    else
    {
      $cZahlart = $hinweis_name;
      if ($order->getIsPaid())
      {
        $tZahlungseingang->cZahlungsanbieter  = $cZahlart;
        $tZahlungseingang->cHinweis           = '';
      }
    }
    
    $result = array(
      'cZahlart'        => $cZahlart,
      'cGebuehr'        => $cGebuehr,
      'kZahlart'        => $kZahlart,
      'paymentComment'  => $paymentComment
    );
    if (isset($tZahlungseingang))
      $result['tZahlungseingang'] = $tZahlungseingang;
    return $result;
  }
  
  private function getKVersandartByName($cShippingName, $aWarenkorbVersandklassen, $kKundengruppe, $cLieferland = 'DE')
  {
    $kVersandart = 0;
    $aVersandartSprache = $GLOBALS["DB"]->executeQuery(
      "SELECT * FROM tversandartsprache WHERE `cName`='".$cShippingName."'
      ", 
      2
    );
    if (count($aVersandartSprache) > 1)
    {
      $diff_keys = false;
      $check_key = $aVersandartSprache[0]->kVersandart;
      foreach ($aVersandartSprache as $versandartSprache)
      {
        if ($check_key <> $versandartSprache->kVersandart)
        {
          $diff_keys = true;
          break;
        }
        $check_key = $versandartSprache->kVersandart;
      }
      if ($diff_keys)
      {
        sort($aWarenkorbVersandklassen);
        $cVersandklassen = implode('-', $aWarenkorbVersandklassen);
        $versandarten = $GLOBALS["DB"]->executeQuery("SELECT kVersandart
          FROM tversandart
          WHERE cNurAbhaengigeVersandart='" . $this->cNurAbhaengigeVersandart . "'
            AND cLaender like \"%" . $cLieferland . "%\"
            AND (cVersandklassen='-1' or (cVersandklassen like \"% " . $cVersandklassen . " %\" or cVersandklassen like \"% " . $cVersandklassen . "\"))
            AND (cKundengruppen='-1' or cKundengruppen like '%;" . $kKundengruppe . ";%') order by nSort", 2);
        $cnt = count($versandarten);
        if ($cnt > 0)
        {
          $aVersand = array();
          foreach ($versandarten as $value)
            $aVersand[] = $value->kVersandart;
          foreach ($aVersandartSprache as $oVersandartSprache)
          {
            if (in_array($oVersandartSprache->kVersandart, $aVersand))
            {
              $kVersandart = $oVersandartSprache->kVersandart;
              break;
            }
          }
        }
      }
      else
        $kVersandart = $check_key;
    }
    else if (count($aVersandartSprache) == 1)
      $kVersandart = $aVersandartSprache[0]->kVersandart;
    return $kVersandart;
  }
}

?>